package edu.uoc.tfg.nutriknow;

import android.content.Context;

import androidx.test.InstrumentationRegistry;
import androidx.test.espresso.action.ViewActions;
import androidx.test.espresso.matcher.ViewMatchers;
import androidx.test.ext.junit.rules.ActivityScenarioRule;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.filters.LargeTest;

import org.junit.After;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import edu.uoc.tfg.nutriknow.utils.ActivityUtils;
import edu.uoc.tfg.nutriknow.utils.ErrorMatcher;
import edu.uoc.tfg.nutriknow.views.activities.LoginActivity;
import edu.uoc.tfg.nutriknow.views.activities.RegisterActivity;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;

import static androidx.test.espresso.Espresso.onView;
import static androidx.test.espresso.action.ViewActions.closeSoftKeyboard;
import static androidx.test.espresso.action.ViewActions.typeText;
import static androidx.test.espresso.assertion.ViewAssertions.matches;
import static androidx.test.espresso.matcher.ViewMatchers.isDisplayed;
import static androidx.test.espresso.matcher.ViewMatchers.withEffectiveVisibility;
import static androidx.test.espresso.matcher.ViewMatchers.withId;
import static androidx.test.espresso.matcher.ViewMatchers.withText;
import static org.junit.Assert.assertTrue;


@RunWith(AndroidJUnit4.class)
@LargeTest
public class LoginActivityTest {

    private static final String EMAIL = "a@a.es";
    private static final String PASSWORD = "aaaaaa";
    private static final String INCORRECT_EMAIL = "a@a";
    private static final String INCORRECT_PASSWORD = "aaaa";
    private static final long TIME_TO_WAIT = 2000L;

    @Rule
    public ActivityScenarioRule<LoginActivity> rule = new ActivityScenarioRule<>(LoginActivity.class);

    private final Context targetContext = InstrumentationRegistry.getTargetContext();

    public void setUp(boolean createProfile) throws InterruptedException {
        ActivityUtils.deleteUser(EMAIL, PASSWORD);
        Thread.sleep(TIME_TO_WAIT);
        ActivityUtils.createUser(EMAIL, PASSWORD, createProfile);
        Thread.sleep(TIME_TO_WAIT);
    }

    @After
    public void tearDown() {
        ActivityUtils.deleteUser(EMAIL, PASSWORD);
    }

    @Test
    public void login_with_profile_OK() throws InterruptedException {
        setUp(true);
        onView(withId(R.id.edit_email)).perform(typeText(EMAIL), closeSoftKeyboard());
        onView(withId(R.id.edit_email)).check(matches(withText(EMAIL)));
        onView(withId(R.id.edit_password)).perform(typeText(PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_password)).check(matches(withText(PASSWORD)));
        onView(withId(R.id.btn_login)).perform(ViewActions.click());
        Thread.sleep(TIME_TO_WAIT);
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.init_text)).check(matches(withText(R.string.welcome_activity_message)));
    }

    @Test
    public void login_without_profile_OK() throws InterruptedException {
        setUp(false);
        onView(withId(R.id.edit_email)).perform(typeText(EMAIL), closeSoftKeyboard());
        onView(withId(R.id.edit_email)).check(matches(withText(EMAIL)));
        onView(withId(R.id.edit_password)).perform(typeText(PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_password)).check(matches(withText(PASSWORD)));
        onView(withId(R.id.btn_login)).perform(ViewActions.click());
        Thread.sleep(TIME_TO_WAIT);
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.profile_setup_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.btn_setup)).perform(ViewActions.click());
        Thread.sleep(TIME_TO_WAIT);
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.welcome_activity_message)));
    }

    @Test
    public void login_KO() throws InterruptedException {
        ActivityUtils.deleteUser(EMAIL, PASSWORD);
        Thread.sleep(TIME_TO_WAIT);
        onView(withId(R.id.edit_email)).perform(typeText(EMAIL), closeSoftKeyboard());
        onView(withId(R.id.edit_email)).check(matches(withText(EMAIL)));
        onView(withId(R.id.edit_password)).perform(typeText(PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_password)).check(matches(withText(PASSWORD)));
        onView(withId(R.id.btn_login)).perform(ViewActions.click());
        Thread.sleep(TIME_TO_WAIT);
        onView(withText(R.string.login_failed_error))
                .check(matches(withEffectiveVisibility(ViewMatchers.Visibility.VISIBLE)));
        Thread.sleep(TIME_TO_WAIT);
        assertTrue(ActivityUtils.getActivityInstance() instanceof LoginActivity);
    }

    @Test
    public void go_to_register_user_OK() throws InterruptedException {
        onView(withId(R.id.link_signup)).perform(ViewActions.click());
        Thread.sleep(TIME_TO_WAIT);
        assertTrue(ActivityUtils.getActivityInstance() instanceof RegisterActivity);
    }

    @Test
    public void login_incorrect_email_KO() {
        String error = targetContext.getResources().getString(R.string.incorrect_email_address_error);
        onView(withId(R.id.edit_email)).perform(typeText(INCORRECT_EMAIL), closeSoftKeyboard());
        onView(withId(R.id.edit_email)).check(matches(withText(INCORRECT_EMAIL)));
        onView(withId(R.id.input_email)).check(matches(ErrorMatcher.withError(error)));
    }

    @Test
    public void login_incorrect_password_KO() {
        String error = targetContext.getResources().getString(R.string.incorrect_password_error);
        onView(withId(R.id.edit_email)).perform(typeText(EMAIL), closeSoftKeyboard());
        onView(withId(R.id.edit_email)).check(matches(withText(EMAIL)));
        onView(withId(R.id.edit_password)).perform(typeText(INCORRECT_PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_password)).check(matches(withText(INCORRECT_PASSWORD)));
        onView(withId(R.id.input_password)).check(matches(ErrorMatcher.withError(error)));
    }
}