package edu.uoc.tfg.nutriknow;

import android.content.Context;

import androidx.test.InstrumentationRegistry;
import androidx.test.espresso.matcher.ViewMatchers;
import androidx.test.ext.junit.rules.ActivityScenarioRule;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.filters.LargeTest;

import org.junit.After;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import edu.uoc.tfg.nutriknow.utils.ActivityUtils;
import edu.uoc.tfg.nutriknow.utils.ErrorMatcher;
import edu.uoc.tfg.nutriknow.views.activities.LoginActivity;
import edu.uoc.tfg.nutriknow.views.activities.RegisterActivity;

import static androidx.test.espresso.Espresso.onView;
import static androidx.test.espresso.action.ViewActions.click;
import static androidx.test.espresso.action.ViewActions.closeSoftKeyboard;
import static androidx.test.espresso.action.ViewActions.scrollTo;
import static androidx.test.espresso.action.ViewActions.typeText;
import static androidx.test.espresso.assertion.ViewAssertions.matches;
import static androidx.test.espresso.matcher.ViewMatchers.withEffectiveVisibility;
import static androidx.test.espresso.matcher.ViewMatchers.withId;
import static androidx.test.espresso.matcher.ViewMatchers.withText;
import static org.hamcrest.Matchers.allOf;
import static org.junit.Assert.assertTrue;


@RunWith(AndroidJUnit4.class)
@LargeTest
public class RegisterActivityTest {

    private static final String USERNAME = "username";
    private static final String EMAIL = "b@b.es";
    private static final String PASSWORD = "aaaaaa";
    private static final String INCORRECT_USERNAME = "user";
    private static final String INCORRECT_EMAIL = "b@b";
    private static final String INCORRECT_PASSWORD = "aaaa";
    private static final String INCORRECT_REPEAT_PASSWORD = "bbbbbb";
    private static final long TIME_TO_WAIT = 2000L;

    @Rule
    public ActivityScenarioRule<RegisterActivity> rule = new ActivityScenarioRule<>(RegisterActivity.class);

    private final Context targetContext = InstrumentationRegistry.getTargetContext();

    public void setUp() throws InterruptedException {
        ActivityUtils.deleteUser(EMAIL, PASSWORD);
        Thread.sleep(TIME_TO_WAIT);
    }

    @After
    public void tearDown() {
        ActivityUtils.deleteUser(EMAIL, PASSWORD);
    }

    @Test
    public void register_OK() throws InterruptedException {
        setUp();
        onView(withId(R.id.edit_user)).perform(typeText(USERNAME), closeSoftKeyboard());
        onView(withId(R.id.edit_user)).check(matches(withText(USERNAME)));
        onView(withId(R.id.edit_email)).perform(typeText(EMAIL), closeSoftKeyboard());
        onView(withId(R.id.edit_email)).check(matches(withText(EMAIL)));
        onView(withId(R.id.edit_password)).perform(typeText(PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_password)).check(matches(withText(PASSWORD)));
        onView(allOf(withId(R.id.edit_repeat_password))).perform(scrollTo(), click());
        onView(withId(R.id.edit_repeat_password)).perform(typeText(PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_repeat_password)).check(matches(withText(PASSWORD)));
        onView(withId(R.id.btn_signup)).perform(click());
        Thread.sleep(TIME_TO_WAIT);
        assertTrue(ActivityUtils.getActivityInstance() instanceof LoginActivity);
    }

    @Test
    public void register_KO() throws InterruptedException {
        setUp();
        ActivityUtils.createUser(EMAIL, PASSWORD, false);
        Thread.sleep(TIME_TO_WAIT);
        onView(withId(R.id.edit_user)).perform(typeText(USERNAME), closeSoftKeyboard());
        onView(withId(R.id.edit_user)).check(matches(withText(USERNAME)));
        onView(withId(R.id.edit_email)).perform(typeText(EMAIL), closeSoftKeyboard());
        onView(withId(R.id.edit_email)).check(matches(withText(EMAIL)));
        onView(withId(R.id.edit_password)).perform(typeText(PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_password)).check(matches(withText(PASSWORD)));
        onView(allOf(withId(R.id.edit_repeat_password))).perform(scrollTo(), click());
        onView(withId(R.id.edit_repeat_password)).perform(typeText(PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_repeat_password)).check(matches(withText(PASSWORD)));
        onView(withId(R.id.btn_signup)).perform(click());
        Thread.sleep(TIME_TO_WAIT);
        onView(withText(R.string.register_failed_error))
                .check(matches(withEffectiveVisibility(ViewMatchers.Visibility.VISIBLE)));
        Thread.sleep(TIME_TO_WAIT);
        assertTrue(ActivityUtils.getActivityInstance() instanceof RegisterActivity);
    }

    @Test
    public void register_incorrect_username_KO() {
        String error = targetContext.getResources().getString(R.string.incorrect_username_error);
        onView(withId(R.id.edit_user)).perform(typeText(INCORRECT_USERNAME), closeSoftKeyboard());
        onView(withId(R.id.edit_user)).check(matches(withText(INCORRECT_USERNAME)));
        onView(withId(R.id.input_user)).check(matches(
                ErrorMatcher.withError(error)));
    }

    @Test
    public void register_incorrect_email_KO() {
        String error = targetContext.getResources().getString(R.string.incorrect_email_address_error);
        onView(withId(R.id.edit_user)).perform(typeText(USERNAME), closeSoftKeyboard());
        onView(withId(R.id.edit_user)).check(matches(withText(USERNAME)));
        onView(withId(R.id.edit_email)).perform(typeText(INCORRECT_EMAIL), closeSoftKeyboard());
        onView(withId(R.id.edit_email)).check(matches(withText(INCORRECT_EMAIL)));
        onView(withId(R.id.input_email)).check(matches(
                ErrorMatcher.withError(error)));
    }

    @Test
    public void register_incorrect_password_KO() {
        String error = targetContext.getResources().getString(R.string.incorrect_password_error);
        onView(withId(R.id.edit_user)).perform(typeText(USERNAME), closeSoftKeyboard());
        onView(withId(R.id.edit_user)).check(matches(withText(USERNAME)));
        onView(withId(R.id.edit_email)).perform(typeText(EMAIL), closeSoftKeyboard());
        onView(withId(R.id.edit_email)).check(matches(withText(EMAIL)));
        onView(withId(R.id.edit_password)).perform(typeText(INCORRECT_PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_password)).check(matches(withText(INCORRECT_PASSWORD)));
        onView(withId(R.id.input_password)).check(matches(ErrorMatcher.withError(error)));
    }

    @Test
    public void register_incorrect_repeat_password_KO() {
        String error = targetContext.getResources().getString(R.string.incorrect_password_error);
        onView(withId(R.id.edit_user)).perform(typeText(USERNAME), closeSoftKeyboard());
        onView(withId(R.id.edit_user)).check(matches(withText(USERNAME)));
        onView(withId(R.id.edit_email)).perform(typeText(EMAIL), closeSoftKeyboard());
        onView(withId(R.id.edit_email)).check(matches(withText(EMAIL)));
        onView(withId(R.id.edit_password)).perform(typeText(PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_password)).check(matches(withText(PASSWORD)));
        onView(allOf(withId(R.id.edit_repeat_password))).perform(scrollTo(), click());
        onView(withId(R.id.edit_repeat_password)).perform(typeText(INCORRECT_PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_repeat_password)).check(matches(withText(INCORRECT_PASSWORD)));
        onView(withId(R.id.input_repeat_password)).check(matches(ErrorMatcher.withError(error)));
    }

    @Test
    public void register_not_matching_passwords_KO() {
        String error = targetContext.getResources().getString(R.string.incorrect_repeat_password_error);
        onView(withId(R.id.edit_user)).perform(typeText(USERNAME), closeSoftKeyboard());
        onView(withId(R.id.edit_user)).check(matches(withText(USERNAME)));
        onView(withId(R.id.edit_email)).perform(typeText(EMAIL), closeSoftKeyboard());
        onView(withId(R.id.edit_email)).check(matches(withText(EMAIL)));
        onView(withId(R.id.edit_password)).perform(typeText(PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_password)).check(matches(withText(PASSWORD)));
        onView(allOf(withId(R.id.edit_repeat_password))).perform(scrollTo(), click());
        onView(withId(R.id.edit_repeat_password)).perform(typeText(INCORRECT_REPEAT_PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_repeat_password)).check(matches(withText(INCORRECT_REPEAT_PASSWORD)));
        onView(withId(R.id.input_repeat_password)).check(matches(ErrorMatcher.withError(error)));
    }
}