package edu.uoc.tfg.nutriknow;

import android.content.Context;
import android.os.Build;
import android.view.Gravity;
import android.widget.ImageView;

import androidx.test.espresso.matcher.RootMatchers;
import androidx.test.ext.junit.rules.ActivityScenarioRule;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.filters.LargeTest;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import edu.uoc.tfg.nutriknow.utils.ActivityUtils;
import edu.uoc.tfg.nutriknow.views.activities.LoginActivity;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;

import static androidx.test.InstrumentationRegistry.getTargetContext;
import static androidx.test.espresso.Espresso.onData;
import static androidx.test.espresso.Espresso.onView;
import static androidx.test.espresso.action.ViewActions.click;
import static androidx.test.espresso.action.ViewActions.closeSoftKeyboard;
import static androidx.test.espresso.action.ViewActions.scrollTo;
import static androidx.test.espresso.action.ViewActions.typeText;
import static androidx.test.espresso.assertion.ViewAssertions.matches;
import static androidx.test.espresso.contrib.DrawerActions.open;
import static androidx.test.espresso.matcher.RootMatchers.isDialog;
import static androidx.test.espresso.matcher.ViewMatchers.hasSibling;
import static androidx.test.espresso.matcher.ViewMatchers.isChecked;
import static androidx.test.espresso.matcher.ViewMatchers.isDisplayed;
import static androidx.test.espresso.matcher.ViewMatchers.withContentDescription;
import static androidx.test.espresso.matcher.ViewMatchers.withId;
import static androidx.test.espresso.matcher.ViewMatchers.withParent;
import static androidx.test.espresso.matcher.ViewMatchers.withText;
import static org.hamcrest.Matchers.allOf;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.instanceOf;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;


@RunWith(AndroidJUnit4.class)
@LargeTest
public class WelcomeActivityTest {

    private static final String EMAIL = "a@a.es";
    private static final String PASSWORD = "aaaaaa";
    private static final String ES = "es";
    private static final long TIME_TO_WAIT = 2000L;

    @Rule
    public ActivityScenarioRule<LoginActivity> rule = new ActivityScenarioRule<>(LoginActivity.class);

    @Before
    public void setUp() throws InterruptedException {
        ActivityUtils.deleteUser(EMAIL, PASSWORD);
        Thread.sleep(TIME_TO_WAIT);
        ActivityUtils.createUser(EMAIL, PASSWORD, true);
        Thread.sleep(TIME_TO_WAIT);
        onView(withId(R.id.edit_email)).perform(typeText(EMAIL), closeSoftKeyboard());
        onView(withId(R.id.edit_email)).check(matches(withText(EMAIL)));
        onView(withId(R.id.edit_password)).perform(typeText(PASSWORD), closeSoftKeyboard());
        onView(withId(R.id.edit_password)).check(matches(withText(PASSWORD)));
        onView(withId(R.id.btn_login)).perform(click());
        Thread.sleep(TIME_TO_WAIT);
    }

    @After
    public void tearDown() {
        ActivityUtils.deleteUser(EMAIL, PASSWORD);
    }

    @Test
    public void welcome_init_OK() {
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.welcome_activity_message)));
    }

    @Test
    public void welcome_navigation_view_home_OK() {
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.welcome_activity_message)));
        onView(withId(R.id.drawer_layout)).perform(open(Gravity.END));
        onView(withText(R.string.menu_my_account)).perform(click());
        onView(withId(R.id.user_account_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.drawer_layout)).perform(open(Gravity.END));
        onView(withText(R.string.menu_home)).perform(click());
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.welcome_activity_message)));
    }

    @Test
    public void welcome_navigation_view_my_account_OK() throws InterruptedException {
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.welcome_activity_message)));
        onView(withId(R.id.drawer_layout)).perform(open(Gravity.END));
        onView(withText(R.string.menu_my_account)).perform(click());
        onView(withId(R.id.user_account_fragment)).check(matches(isDisplayed()));
        onView(allOf(instanceOf(ImageView.class), hasSibling(withText(R.string.user_account_allergens_label))))
                .perform(click());
        onView(withId(R.id.profile_setup_fragment)).check(matches(isDisplayed()));
        onView(withContentDescription(R.string.abc_action_bar_up_description)).perform(click());
        onView(withId(R.id.user_account_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.btn_delete)).perform(click());
        onView(withText(R.string.user_delete_dialog_yes_text)).inRoot(isDialog()).check(matches(isDisplayed())).perform(click());
        Thread.sleep(TIME_TO_WAIT);
        assertTrue(ActivityUtils.getActivityInstance() instanceof LoginActivity);
    }

    @Test
    public void welcome_navigation_view_profile_setup_OK() throws InterruptedException {
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.welcome_activity_message)));
        onView(withId(R.id.drawer_layout)).perform(open(Gravity.END));
        onView(withText(R.string.menu_my_account)).perform(click());
        onView(withId(R.id.user_account_fragment)).check(matches(isDisplayed()));
        onView(allOf(instanceOf(ImageView.class), hasSibling(withText(R.string.user_account_allergens_label))))
                .perform(click());
        onView(withId(R.id.profile_setup_fragment)).check(matches(isDisplayed()));
        onView(allOf(withId(R.id.allergens))).perform(scrollTo(), click());
        if (Build.VERSION.SDK_INT > Build.VERSION_CODES.M) {
            onView(withId(R.id.allergens)).perform(typeText("Glu"));
            onData(equalTo("Gluten")).inRoot(RootMatchers.isPlatformPopup()).perform(click());
            onView(allOf(instanceOf(ImageView.class), hasSibling(withId(R.id.input_allergens)))).perform(click());
            onView(allOf(withId(R.id.selectedList))).perform(scrollTo(), click());
            onView(withId(R.id.allergens)).perform(typeText("Mil"));
            onData(equalTo("Milk")).inRoot(RootMatchers.isPlatformPopup()).perform(click());
        } else {
            onView(withId(R.id.allergens)).perform(typeText("Gluten"));
            onView(allOf(instanceOf(ImageView.class), hasSibling(withId(R.id.input_allergens)))).perform(click());
            onView(allOf(withId(R.id.selectedList))).perform(scrollTo(), click());
            onView(withId(R.id.allergens)).perform(typeText("Milk"));
        }
        onView(allOf(instanceOf(ImageView.class), hasSibling(withId(R.id.input_allergens)))).perform(click());
        onView(allOf(withText(R.string.periodicity_weekly_label),
                withParent(withId(R.id.radio_group)))).perform(scrollTo(), click());
        onView(allOf(withText(R.string.periodicity_weekly_label),
                withParent(withId(R.id.radio_group)))).check(matches(isChecked()));
        onView(withId(R.id.btn_setup)).perform(click());
        Thread.sleep(TIME_TO_WAIT);
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
    }

    @Test
    public void welcome_navigation_view_my_searches_OK() {
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.welcome_activity_message)));
        onView(withId(R.id.drawer_layout)).perform(open(Gravity.END));
        onView(withText(R.string.menu_my_searches)).perform(click());
        onView(withId(R.id.searches_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.searches_activity_message)));
    }

    @Test
    public void welcome_navigation_view_my_favorites_OK() {
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.welcome_activity_message)));
        onView(withId(R.id.drawer_layout)).perform(open(Gravity.END));
        onView(withText(R.string.menu_my_favorites)).perform(click());
        onView(withId(R.id.favorites_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.favorites_activity_message)));
    }

    @Test
    public void welcome_navigation_view_settings_OK() throws InterruptedException {
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.welcome_activity_message)));
        onView(withId(R.id.drawer_layout)).perform(open(Gravity.END));
        onView(withText(R.string.menu_settings)).perform(click());
        onView(withId(R.id.settings_fragment)).check(matches(isDisplayed()));
        onView(allOf(withText(R.string.settings_es_label),
                withParent(withId(R.id.radio_group)))).perform(scrollTo(), click());
        onView(allOf(withText(R.string.settings_es_label),
                withParent(withId(R.id.radio_group)))).check(matches(isChecked()));
        onView(withId(R.id.btn_settings)).perform(click());
        Thread.sleep(TIME_TO_WAIT);
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
        assertEquals(ES, ActivityUtils.getActivityInstance().getResources().getConfiguration().locale.getLanguage());
    }

    @Test
    public void welcome_navigation_view_logout_OK() throws InterruptedException {
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.welcome_activity_message)));
        onView(withId(R.id.drawer_layout)).perform(open(Gravity.END));
        onView(withText(R.string.menu_log_out)).perform(click());
        Thread.sleep(TIME_TO_WAIT);
        assertTrue(ActivityUtils.getActivityInstance() instanceof LoginActivity);
    }

    @Test
    public void welcome_bottom_navigation_view_searches_OK() {
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.welcome_activity_message)));
        onView(withId(R.id.searches)).perform(click());
        onView(withId(R.id.searches_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.searches_activity_message)));
    }

    @Test
    public void welcome_bottom_navigation_view_favorites_OK() {
        assertTrue(ActivityUtils.getActivityInstance() instanceof WelcomeActivity);
        onView(withId(R.id.home_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.welcome_activity_message)));
        onView(withId(R.id.favorites)).perform(click());
        onView(withId(R.id.favorites_fragment)).check(matches(isDisplayed()));
        onView(withId(R.id.init_text)).check(matches(withText(R.string.favorites_activity_message)));
    }
}