package edu.uoc.tfg.nutriknow.utils;

import android.app.Activity;

import androidx.annotation.NonNull;
import androidx.test.InstrumentationRegistry;
import androidx.test.runner.lifecycle.ActivityLifecycleMonitorRegistry;

import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.Collection;
import java.util.Date;

import edu.uoc.tfg.nutriknow.model.Profile;
import edu.uoc.tfg.nutriknow.model.User;

import static androidx.test.runner.lifecycle.Stage.RESUMED;

public class ActivityUtils {

    public static FirebaseAuth firebaseAuth;

    public static FirebaseDatabase database;
    public static DatabaseReference dbReference;

    public static Activity getActivityInstance() {
        final Activity[] activity = new Activity[1];
        InstrumentationRegistry.getInstrumentation().runOnMainSync(new Runnable() {
            public void run() {
                Activity currentActivity = null;
                Collection resumedActivities = ActivityLifecycleMonitorRegistry.getInstance().getActivitiesInStage(RESUMED);
                if (resumedActivities.iterator().hasNext()) {
                    currentActivity = (Activity) resumedActivities.iterator().next();
                    activity[0] = currentActivity;
                }
            }
        });

        return activity[0];
    }

    public static void createUser(String email, String password, Boolean createProfile) {
        firebaseAuth = FirebaseAuth.getInstance();
        firebaseAuth.createUserWithEmailAndPassword(email, password)
                .addOnSuccessListener(new OnSuccessListener<AuthResult>() {
                    @Override
                    public void onSuccess(@NonNull AuthResult result) {
                        // Create user in database
                        FirebaseUser currentUser = firebaseAuth.getCurrentUser();

                        User user = new User("username", email);
                        user.setUuid(currentUser.getUid());
                        user.setHasProfile(createProfile);
                        database = FirebaseDatabase.getInstance();
                        dbReference = database.getReference("/users");
                        dbReference.child(currentUser.getUid()).child("/user").setValue(user,
                                new DatabaseReference.CompletionListener() {
                                    @Override
                                    public void onComplete(DatabaseError databaseError,
                                                           DatabaseReference databaseReference) {
                                        if (createProfile) {
                                            Profile profile = new Profile();
                                            profile.setPeriodicity("D");
                                            profile.setCreated(new Date());
                                            dbReference.child(currentUser.getUid()).child("/profile").setValue(profile,
                                                    new DatabaseReference.CompletionListener() {
                                                        @Override
                                                        public void onComplete(DatabaseError databaseError,
                                                                               DatabaseReference databaseReference) {

                                                        }
                                                    });
                                        }
                                    }
                                });
                    }
                });

    }

    public static void deleteUser(String email, String password) {

        firebaseAuth = FirebaseAuth.getInstance();
        firebaseAuth.signInWithEmailAndPassword(email, password).addOnSuccessListener(new OnSuccessListener<AuthResult>() {
            @Override
            public void onSuccess(@NonNull AuthResult result) {
                database = FirebaseDatabase.getInstance();
                dbReference = database.getReference("/users");

                // Remove user from database
                dbReference.child(firebaseAuth.getCurrentUser().getUid()).addListenerForSingleValueEvent(new ValueEventListener() {
                    @Override
                    public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                        for (DataSnapshot child : dataSnapshot.getChildren()) {
                            child.getRef().removeValue();
                        }
                        dataSnapshot.getRef().removeValue();
                        // Delete user from firebase auth
                        firebaseAuth.getCurrentUser().delete().addOnSuccessListener(new OnSuccessListener() {
                            @Override
                            public void onSuccess(Object o) {

                            }
                        });
                    }

                    @Override
                    public void onCancelled(@NonNull DatabaseError databaseError) {

                    }
                });
            }
        });

    }
}
