package edu.uoc.tfg.nutriknow.adapters;

import android.content.Context;
import android.content.res.ColorStateList;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import java.util.List;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.model.IngredientsAnalysis;

/**
 * Adapter for managing the list of ingredients analysis in views
 */
public class IngredientsAnalysisSummaryAdapter extends ArrayAdapter<IngredientsAnalysis.IngredientAnalysis> {

    private final Context context;

    /**
     * ViewHolder structure
     */
    private static class ViewHolder {
        ImageView identificativeImageView;
        TextView labelTextView;
    }

    /**
     * Adapter constructor
     *
     * @param ingredientsAnalysis source list
     * @param ctx                 context
     */
    public IngredientsAnalysisSummaryAdapter(List<IngredientsAnalysis.IngredientAnalysis> ingredientsAnalysis, Context ctx) {
        super(ctx, R.layout.product_info_row_layout, ingredientsAnalysis);
        this.context = ctx;

    }

    /**
     * Create the view
     *
     * @param position    position in the source list
     * @param convertView view to inflate
     * @param parent      parent view
     * @return view
     */
    @Override
    public View getView(int position, View convertView, ViewGroup parent) {

        // Define source list
        IngredientsAnalysis.IngredientAnalysis ingredientAnalysis = getItem(position);

        // Check if an existing view is being reused, otherwise inflate the view
        ViewHolder viewHolder; // view lookup cache stored in tag

        if (convertView == null) {

            viewHolder = new ViewHolder();
            LayoutInflater inflater = LayoutInflater.from(getContext());
            convertView = inflater.inflate(R.layout.product_info_row_layout, parent, false);
            // Reference the views in the layout
            viewHolder.identificativeImageView = (ImageView) convertView.findViewById(R.id.identificative_icon);
            viewHolder.labelTextView = (TextView) convertView.findViewById(R.id.label);

            convertView.setTag(viewHolder);
        } else {
            viewHolder = (ViewHolder) convertView.getTag();
        }
        // Define the content of the layout views
        viewHolder.identificativeImageView.setImageDrawable(context.getDrawable(getIngredientAnalysisDrawable(ingredientAnalysis)));
        viewHolder.identificativeImageView.setImageTintList(ColorStateList.valueOf(context.getColor(R.color.backgroundColorPrimary)));
        viewHolder.labelTextView.setText(ingredientAnalysis.name);
        return convertView;
    }

    /**
     * Get the drawable related to the code of the item
     *
     * @param ingredientAnalysis source
     * @return drawable id
     */
    private int getIngredientAnalysisDrawable(IngredientsAnalysis.IngredientAnalysis ingredientAnalysis) {
        int drawableId = 0;
        if (ingredientAnalysis.id.contains("vegan")) {
            drawableId = R.drawable.ic_vegan;
        } else if (ingredientAnalysis.id.contains("vegetarian")) {
            drawableId = R.drawable.ic_vegetarian;
        } else {
            drawableId = R.drawable.ic_palm_oil;
        }
        return drawableId;
    }
}
