package edu.uoc.tfg.nutriknow.adapters;

import android.content.Context;
import android.content.res.ColorStateList;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import java.util.List;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.model.Labels;

/**
 * Adapter for managing the list of labels in views
 */
public class LabelsSummaryAdapter extends ArrayAdapter<Labels.Label> {

    private final Context context;

    /**
     * Viewholder structure
     */
    private static class ViewHolder {
        ImageView identificativeImageView;
        TextView labelTextView;
    }

    /**
     * Adapter constructor
     *
     * @param labels source list
     * @param ctx    context
     */
    public LabelsSummaryAdapter(List<Labels.Label> labels, Context ctx) {
        super(ctx, R.layout.product_info_row_layout, labels);
        this.context = ctx;

    }

    /**
     * Create the view
     *
     * @param position    position in the source list
     * @param convertView view to inflate
     * @param parent      parent view
     * @return view
     */
    @Override
    public View getView(int position, View convertView, ViewGroup parent) {

        // Define source list
        Labels.Label label = getItem(position);

        // Check if an existing view is being reused, otherwise inflate the view
        ViewHolder viewHolder; // view lookup cache stored in tag

        if (convertView == null) {

            viewHolder = new ViewHolder();
            LayoutInflater inflater = LayoutInflater.from(getContext());
            convertView = inflater.inflate(R.layout.product_info_row_layout, parent, false);
            // Reference the views in the layout
            viewHolder.identificativeImageView = (ImageView) convertView.findViewById(R.id.identificative_icon);
            viewHolder.labelTextView = (TextView) convertView.findViewById(R.id.label);

            convertView.setTag(viewHolder);
        } else {
            viewHolder = (ViewHolder) convertView.getTag();
        }
        // Define the content of the layout views
        viewHolder.identificativeImageView.setImageDrawable(context.getDrawable(R.drawable.ic_label));
        viewHolder.identificativeImageView.setImageTintList(ColorStateList.valueOf(context.getColor(R.color.backgroundColorPrimary)));
        viewHolder.labelTextView.setText(label.name);
        return convertView;
    }
}
