package edu.uoc.tfg.nutriknow.adapters;

import android.content.Context;
import android.content.res.ColorStateList;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import java.util.List;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.model.NutrientsLevel;

/**
 * Adapter for managing the list of nutrients levels in views
 */
public class NutrientsLevelsNutritionalAdapter extends ArrayAdapter<NutrientsLevel.NutrientLevel> {

    private static final String ES = "es";

    private final String language;
    private final Context context;

    /**
     * ViewHolder structure
     */
    private static class ViewHolder {
        ImageView identificativeImageView;
        TextView labelTextView;
        ImageView valueImageView;
    }

    /**
     * Adapter constructor
     *
     * @param nutrientLevels source list
     * @param ctx            context
     * @param language       selected locale
     */
    public NutrientsLevelsNutritionalAdapter(List<NutrientsLevel.NutrientLevel> nutrientLevels,
                                             Context ctx, String language) {
        super(ctx, R.layout.product_info_row_with_value_layout, nutrientLevels);
        this.language = language;
        this.context = ctx;

    }

    /**
     * Create the view
     *
     * @param position    position in the source list
     * @param convertView view to inflate
     * @param parent      parent view
     * @return view
     */
    @Override
    public View getView(int position, View convertView, ViewGroup parent) {

        // Define source list
        NutrientsLevel.NutrientLevel nutrientLevel = getItem(position);

        // Check if an existing view is being reused, otherwise inflate the view
        ViewHolder viewHolder; // view lookup cache stored in tag

        if (convertView == null) {

            viewHolder = new ViewHolder();
            LayoutInflater inflater = LayoutInflater.from(getContext());
            convertView = inflater.inflate(R.layout.product_info_row_with_value_layout, parent, false);
            // Reference the views in the layout
            viewHolder.identificativeImageView = (ImageView) convertView.findViewById(R.id.identificative_icon);
            viewHolder.labelTextView = (TextView) convertView.findViewById(R.id.label);
            viewHolder.valueImageView = (ImageView) convertView.findViewById(R.id.value_icon);

            convertView.setTag(viewHolder);
        } else {
            viewHolder = (ViewHolder) convertView.getTag();
        }
        // Define the content of the layout views
        viewHolder.identificativeImageView.setImageDrawable(context.getDrawable(getIdentificativeDrawable(nutrientLevel)));
        viewHolder.identificativeImageView.setImageTintList(ColorStateList.valueOf(context.getColor(R.color.backgroundColorPrimary)));
        viewHolder.valueImageView.setImageDrawable(context.getDrawable(getValueDrawable(nutrientLevel)));
        if (ES.equals(language)) {
            viewHolder.labelTextView.setText(nutrientLevel.name.es);
        } else {
            viewHolder.labelTextView.setText(nutrientLevel.name.en);
        }

        return convertView;
    }

    /**
     * Get the drawable related to the code of the item
     *
     * @param nutrientLevel source
     * @return drawable id
     */
    private int getIdentificativeDrawable(NutrientsLevel.NutrientLevel nutrientLevel) {
        int drawableId = 0;
        if (nutrientLevel.code.contains("sugar")) {
            drawableId = R.drawable.ic_sugar;
        } else if (nutrientLevel.code.contains("salt")) {
            drawableId = R.drawable.ic_salt;
        } else if (nutrientLevel.code.contains("saturated-fat")) {
            drawableId = R.drawable.ic_saturated_fat;
        } else {
            drawableId = R.drawable.ic_fat;
        }
        return drawableId;
    }

    /**
     * Get the drawable related to the value of the item
     *
     * @param nutrientLevel source
     * @return drawable id
     */
    private int getValueDrawable(NutrientsLevel.NutrientLevel nutrientLevel) {
        int drawableId = 0;
        if (nutrientLevel.code.contains("low")) {
            drawableId = R.drawable.low;
        } else if (nutrientLevel.code.contains("moderate")) {
            drawableId = R.drawable.moderate;
        } else {
            drawableId = R.drawable.high;
        }
        return drawableId;
    }
}
