package edu.uoc.tfg.nutriknow.adapters;

import android.content.Context;

import androidx.annotation.Nullable;
import androidx.annotation.StringRes;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentStatePagerAdapter;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.model.Product;
import edu.uoc.tfg.nutriknow.views.fragments.IngredientsInfoFragment;
import edu.uoc.tfg.nutriknow.views.fragments.NutritionalInfoFragment;
import edu.uoc.tfg.nutriknow.views.fragments.SummaryInfoFragment;

/**
 * Adapter for managing the tablayout
 */
public class TabsPagerAdapter extends FragmentStatePagerAdapter {

    @StringRes
    private static final int[] TAB_TITLES = new int[]{R.string.product_info_tab_one,
            R.string.product_info_tab_two, R.string.product_info_tab_three};
    private final Context mContext;
    private final Product product;

    /**
     * Adapter constructor
     *
     * @param context context
     * @param fm      fragment
     * @param prod    product object to use in the fragment
     */
    public TabsPagerAdapter(Context context, FragmentManager fm, Product prod) {
        super(fm, BEHAVIOR_RESUME_ONLY_CURRENT_FRAGMENT);
        mContext = context;
        product = prod;
    }

    /**
     * Get the fragment to locate in each position
     *
     * @param position tapped tab position
     * @return fragment
     */
    @Override
    public Fragment getItem(int position) {
        Fragment fragment = null;
        switch (position) {
            case 0:
                fragment = SummaryInfoFragment.newInstance(1, product);
                break;
            case 1:
                fragment = IngredientsInfoFragment.newInstance(2, product);
                break;
            case 2:
                fragment = NutritionalInfoFragment.newInstance(3, product);
                break;
        }
        // getItem is called to instantiate the fragment for the given page.
        // Return a PlaceholderFragment (defined as a static inner class below).
        return fragment;
    }

    /**
     * Set tab title
     *
     * @param position tab position
     * @return
     */
    @Nullable
    @Override
    public CharSequence getPageTitle(int position) {
        return mContext.getResources().getString(TAB_TITLES[position]);
    }

    /**
     * Get number of tabs
     *
     * @return size
     */
    @Override
    public int getCount() {
        return TAB_TITLES.length;
    }
}
