package edu.uoc.tfg.nutriknow.api.interfaces;

import com.google.gson.JsonObject;

import edu.uoc.tfg.nutriknow.model.AdditivesClasses;
import edu.uoc.tfg.nutriknow.model.Allergens;
import edu.uoc.tfg.nutriknow.model.Categories;
import edu.uoc.tfg.nutriknow.model.CompleteProductData;
import edu.uoc.tfg.nutriknow.model.IngredientsAnalysis;
import edu.uoc.tfg.nutriknow.model.Labels;
import edu.uoc.tfg.nutriknow.model.NovaGroups;
import edu.uoc.tfg.nutriknow.model.NutrientsLevel;
import retrofit2.Call;
import retrofit2.http.GET;
import retrofit2.http.Headers;
import retrofit2.http.Path;

/**
 * Interface to communicate with the Open Food Facts api
 */
public interface OpenFoodFactsService {

    /**
     * Get allergens catalog
     * @return allergens list
     */
    @Headers({
            "User-Agent: NutriKnow! - Android - Version 1.0"
    })
    @GET("/allergens.json")
    Call<Allergens> getAllAllergens();

    /**
     * Get labels catalog
     * @return labels list
     */
    @Headers({
            "User-Agent: NutriKnow! - Android - Version 1.0"
    })
    @GET("/labels.json")
    Call<Labels> getAllLabels();

    /**
     * Get categories catalog
     * @return categories list
     */
    @Headers({
            "User-Agent: NutriKnow! - Android - Version 1.0"
    })
    @GET("/categories.json")
    Call<Categories> getAllCategories();

    /**
     * Get nutrient levels catalog
     * @return nutrient levels list
     */
    @Headers({
            "User-Agent: NutriKnow! - Android - Version 1.0"
    })
    @GET("/data/taxonomies/nutrient_levels.json")
    Call<NutrientsLevel> getAllNutrientLevels();

    /**
     * Get nova groups catalog
     * @return nova groups list
     */
    @Headers({
            "User-Agent: NutriKnow! - Android - Version 1.0"
    })
    @GET("/data/taxonomies/nova_groups.json")
    Call<NovaGroups> getAllNovaGroups();

    /**
     * Get additives catalog
     * @return json object containing the whole catalog
     */
    @Headers({
            "User-Agent: NutriKnow! - Android - Version 1.0"
    })
    @GET("/data/taxonomies/additives.json")
    Call<JsonObject> getAllAdditives();

    /**
     * Get the additives classes catalog
     * @return additives classes list
     */
    @Headers({
            "User-Agent: NutriKnow! - Android - Version 1.0"
    })
    @GET("/data/taxonomies/additives_classes.json")
    Call<AdditivesClasses> getAllAdditivesClasses();

    /**
     * Get ingredients analysis catalog
     * @return ingredients analysis list
     */
    @Headers({
            "User-Agent: NutriKnow! - Android - Version 1.0"
    })
    @GET("/ingredients-analysis.json")
    Call<IngredientsAnalysis> getAllIngredientAnalysis();

    /**
     * Get product information
     * @param barcode barcode representing the product
     * @return encoded product information
     */
    @Headers({
            "User-Agent: NutriKnow! - Android - Version 1.0"
    })
    @GET("/api/v0/product/{barcode}")
    Call<CompleteProductData> getProductInformation(@Path("barcode") String barcode);

}
