package edu.uoc.tfg.nutriknow.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Product representation class grouped regarding the tab where the information is displayed
 */
public class Product implements Serializable, Comparable<Product> {

    private SummaryData summaryData;
    private IngredientsData ingredientsData;
    private NutritionalData nutritionalData;

    public Product() {
    }

    public Product(SummaryData summaryData, IngredientsData ingredientsData, NutritionalData nutritionalData) {
        this.summaryData = summaryData;
        this.ingredientsData = ingredientsData;
        this.nutritionalData = nutritionalData;
    }

    public SummaryData getSummaryData() {
        return summaryData;
    }

    public void setSummaryData(SummaryData summaryData) {
        this.summaryData = summaryData;
    }

    public IngredientsData getIngredientsData() {
        return ingredientsData;
    }

    public void setIngredientsData(IngredientsData ingredientsData) {
        this.ingredientsData = ingredientsData;
    }

    public NutritionalData getNutritionalData() {
        return nutritionalData;
    }

    public void setNutritionalData(NutritionalData nutritionalData) {
        this.nutritionalData = nutritionalData;
    }

    @Override
    public int compareTo(Product u) {
        if (getSummaryData() == null || u.getSummaryData().getCode() == null) {
            return 0;
        }
        return getSummaryData().getCode().compareTo(u.getSummaryData().getCode());
    }

    public static class SummaryData implements Serializable {
        private String code;
        private String productName_es;
        private String productName_en;
        private List<String> brands = new ArrayList<>();
        private List<String> stores = new ArrayList<>();
        private List<Labels.Label> labels_es = new ArrayList<>();
        private List<Labels.Label> labels_en = new ArrayList<>();
        private Date expirationDate;
        private List<Allergens.Allergen> allergens_es = new ArrayList<>();
        private List<Allergens.Allergen> allergens_en = new ArrayList<>();
        private List<Allergens.Allergen> traces_es = new ArrayList<>();
        private List<Allergens.Allergen> traces_en = new ArrayList<>();
        private List<String> countries;
        private float completeness;
        private List<IngredientsAnalysis.IngredientAnalysis> ingredientsAnalysis_es = new ArrayList<>();
        private List<IngredientsAnalysis.IngredientAnalysis> ingredientsAnalysis_en = new ArrayList<>();

        public SummaryData() {
        }

        public String getCode() {
            return code;
        }

        public void setCode(String code) {
            this.code = code;
        }

        public String getProductName_es() {
            return productName_es;
        }

        public void setProductName_es(String productName_es) {
            this.productName_es = productName_es;
        }

        public String getProductName_en() {
            return productName_en;
        }

        public void setProductName_en(String productName_en) {
            this.productName_en = productName_en;
        }

        public List<String> getBrands() {
            return brands;
        }

        public void setBrands(List<String> brands) {
            this.brands = brands;
        }

        public List<String> getStores() {
            return stores;
        }

        public void setStores(List<String> stores) {
            this.stores = stores;
        }

        public List<Labels.Label> getLabels_es() {
            return labels_es;
        }

        public void setLabels_es(List<Labels.Label> labels_es) {
            this.labels_es = labels_es;
        }

        public List<Labels.Label> getLabels_en() {
            return labels_en;
        }

        public void setLabels_en(List<Labels.Label> labels_en) {
            this.labels_en = labels_en;
        }

        public Date getExpirationDate() {
            return expirationDate;
        }

        public void setExpirationDate(Date expirationDate) {
            this.expirationDate = expirationDate;
        }

        public List<Allergens.Allergen> getAllergens_es() {
            return allergens_es;
        }

        public void setAllergens_es(List<Allergens.Allergen> allergens_es) {
            this.allergens_es = allergens_es;
        }

        public List<Allergens.Allergen> getAllergens_en() {
            return allergens_en;
        }

        public void setAllergens_en(List<Allergens.Allergen> allergens_en) {
            this.allergens_en = allergens_en;
        }

        public List<Allergens.Allergen> getTraces_es() {
            return traces_es;
        }

        public void setTraces_es(List<Allergens.Allergen> traces_es) {
            this.traces_es = traces_es;
        }

        public List<Allergens.Allergen> getTraces_en() {
            return traces_en;
        }

        public void setTraces_en(List<Allergens.Allergen> traces_en) {
            this.traces_en = traces_en;
        }

        public List<String> getCountries() {
            return countries;
        }

        public void setCountries(List<String> countries) {
            this.countries = countries;
        }

        public float getCompleteness() {
            return completeness;
        }

        public void setCompleteness(float completeness) {
            this.completeness = completeness;
        }

        public List<IngredientsAnalysis.IngredientAnalysis> getIngredientsAnalysis_es() {
            return ingredientsAnalysis_es;
        }

        public void setIngredientsAnalysis_es(List<IngredientsAnalysis.IngredientAnalysis> ingredientsAnalysis_es) {
            this.ingredientsAnalysis_es = ingredientsAnalysis_es;
        }

        public List<IngredientsAnalysis.IngredientAnalysis> getIngredientsAnalysis_en() {
            return ingredientsAnalysis_en;
        }

        public void setIngredientsAnalysis_en(List<IngredientsAnalysis.IngredientAnalysis> ingredientsAnalysis_en) {
            this.ingredientsAnalysis_en = ingredientsAnalysis_en;
        }
    }

    public static class IngredientsData implements Serializable {
        private int ingredients_count;
        private String ingredients_es;
        private String ingredients_en;
        private List<Categories.Category> categories_es = new ArrayList<>();
        private List<Categories.Category> categories_en = new ArrayList<>();
        private List<Allergens.Allergen> allergens_es = new ArrayList<>();
        private List<Allergens.Allergen> allergens_en = new ArrayList<>();
        private List<Allergens.Allergen> traces_es = new ArrayList<>();
        private List<Allergens.Allergen> traces_en = new ArrayList<>();
        private int novaGroup;
        private List<NovaGroups.NovaGroup> novaGroups = new ArrayList<>();
        private int additivesCount;
        private List<AdditivesClasses.AdditiveClass> additiveClasses = new ArrayList<>();
        private List<Additives> additives = new ArrayList<>();

        public IngredientsData() {
        }

        public int getIngredients_count() {
            return ingredients_count;
        }

        public void setIngredients_count(int ingredients_count) {
            this.ingredients_count = ingredients_count;
        }

        public String getIngredients_es() {
            return ingredients_es;
        }

        public void setIngredients_es(String ingredients_es) {
            this.ingredients_es = ingredients_es;
        }

        public String getIngredients_en() {
            return ingredients_en;
        }

        public void setIngredients_en(String ingredients_en) {
            this.ingredients_en = ingredients_en;
        }

        public List<Categories.Category> getCategories_es() {
            return categories_es;
        }

        public void setCategories_es(List<Categories.Category> categories_es) {
            this.categories_es = categories_es;
        }

        public List<Categories.Category> getCategories_en() {
            return categories_en;
        }

        public void setCategories_en(List<Categories.Category> categories_en) {
            this.categories_en = categories_en;
        }

        public List<NovaGroups.NovaGroup> getNovaGroups() {
            return novaGroups;
        }

        public void setNovaGroups(List<NovaGroups.NovaGroup> novaGroups) {
            this.novaGroups = novaGroups;
        }

        public List<AdditivesClasses.AdditiveClass> getAdditiveClasses() {
            return additiveClasses;
        }

        public void setAdditiveClasses(List<AdditivesClasses.AdditiveClass> additiveClasses) {
            this.additiveClasses = additiveClasses;
        }

        public List<Allergens.Allergen> getAllergens_es() {
            return allergens_es;
        }

        public void setAllergens_es(List<Allergens.Allergen> allergens_es) {
            this.allergens_es = allergens_es;
        }

        public List<Allergens.Allergen> getAllergens_en() {
            return allergens_en;
        }

        public void setAllergens_en(List<Allergens.Allergen> allergens_en) {
            this.allergens_en = allergens_en;
        }

        public List<Allergens.Allergen> getTraces_es() {
            return traces_es;
        }

        public void setTraces_es(List<Allergens.Allergen> traces_es) {
            this.traces_es = traces_es;
        }

        public List<Allergens.Allergen> getTraces_en() {
            return traces_en;
        }

        public void setTraces_en(List<Allergens.Allergen> traces_en) {
            this.traces_en = traces_en;
        }

        public int getAdditivesCount() {
            return additivesCount;
        }

        public void setAdditivesCount(int additivesCount) {
            this.additivesCount = additivesCount;
        }

        public int getNovaGroup() {
            return novaGroup;
        }

        public void setNovaGroup(int novaGroup) {
            this.novaGroup = novaGroup;
        }

        public List<Additives> getAdditives() {
            return additives;
        }

        public void setAdditives(List<Additives> additives) {
            this.additives = additives;
        }
    }

    public static class NutritionalData implements Serializable {
        private List<NutrientsLevel.NutrientLevel> nutrientLevels = new ArrayList<>();
        private List<Nutriments> nutriments = new ArrayList<>();
        private String servingSize;
        private String nutritionGrade;
        private Integer nutritionScore;

        public NutritionalData() {
        }


        public List<NutrientsLevel.NutrientLevel> getNutrientLevels() {
            return nutrientLevels;
        }

        public void setNutrientLevels(List<NutrientsLevel.NutrientLevel> nutrientLevels) {
            this.nutrientLevels = nutrientLevels;
        }

        public List<Nutriments> getNutriments() {
            return nutriments;
        }

        public void setNutriments(List<Nutriments> nutriments) {
            this.nutriments = nutriments;
        }

        public String getServingSize() {
            return servingSize;
        }

        public void setServingSize(String servingSize) {
            this.servingSize = servingSize;
        }

        public String getNutritionGrade() {
            return nutritionGrade;
        }

        public void setNutritionGrade(String nutritionGrade) {
            this.nutritionGrade = nutritionGrade;
        }

        public Integer getNutritionScore() {
            return nutritionScore;
        }

        public void setNutritionScore(Integer nutritionScore) {
            this.nutritionScore = nutritionScore;
        }


    }
}
