package edu.uoc.tfg.nutriknow.services;

import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.util.Log;

import androidx.core.app.NotificationCompat;

import com.google.firebase.messaging.FirebaseMessagingService;
import com.google.firebase.messaging.RemoteMessage;

import edu.uoc.tfg.nutriknow.NutriKnowApplication;
import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.views.activities.SplashActivity;

/**
 * Class that receives the push notifications when the app is in foreground and resend them
 */
public class PushNotificationService extends FirebaseMessagingService {

    private static final String TAG = "PushNotificationService";

    /**
     * Listener that receives the push notification
     *
     * @param remoteMessage notification
     */
    @Override
    public void onMessageReceived(RemoteMessage remoteMessage) {
        Log.i(TAG, "From: " + remoteMessage.getFrom());

        // Check if message contains a data payload.
        if (remoteMessage.getData().size() > 0) {
            Log.i(TAG, "Message data payload: " + remoteMessage.getData());
        }

        // Check if message contains a notification payload.
        if (remoteMessage.getNotification() != null) {
            Log.i(TAG, "Message Notification Body: " + remoteMessage.getNotification().getBody());
            if (NutriKnowApplication.isActivityVisible()) {
                sendNotification(remoteMessage.getNotification());
            }
        }
    }

    /**
     * Resend the notification for the device to display it
     *
     * @param messageBody notification content
     */
    public void sendNotification(RemoteMessage.Notification messageBody) {
        // Define which intent will display when notification is tapped
        Intent intent = new Intent(this, SplashActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP);
        PendingIntent pendingIntent = PendingIntent.getActivity(this, 0, intent, 0);
        // Create the notification
        NotificationCompat.Builder notificationBuilder =
                new NotificationCompat.Builder(this, "channelId")
                        .setSmallIcon(R.mipmap.ic_stat_push_notification)
                        .setColor(getApplicationContext().getColor(R.color.backgroundColorAccent))
                        .setContentTitle(messageBody.getTitle())
                        .setContentText(messageBody.getBody())
                        .setStyle(new NotificationCompat.BigTextStyle().bigText(messageBody.getBody()))
                        .setAutoCancel(true)
                        .setContentIntent(pendingIntent);
        //Define the notification manager
        NotificationManager notificationManager =
                (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);

        // Since android Oreo notification channel is needed.
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            String channelName = "ChannelName";
            notificationManager.createNotificationChannel(new NotificationChannel("channelId",
                    channelName, NotificationManager.IMPORTANCE_HIGH));
        }
        // Notify
        notificationManager.notify(0, notificationBuilder.build());
    }
}
