package edu.uoc.tfg.nutriknow.sqlite;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

import edu.uoc.tfg.nutriknow.model.WorkerData;
import edu.uoc.tfg.nutriknow.sqlite.adapters.AdditivesClassesDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.AdditivesDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.AllergensDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.CategoriesDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.IngredientAnalysisDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.LabelsDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.NovaGroupsDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.NutrientsLevelsDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.WorkerDataBaseAdapter;

/**
 * Helper for managing the creation and removal of the database
 */
public class DataBaseHelper extends SQLiteOpenHelper {

    /**
     * Helper constructor
     *
     * @param context context
     * @param name    database name
     * @param factory factory
     * @param version database version
     */
    public DataBaseHelper(Context context, String name, SQLiteDatabase.CursorFactory factory, int version) {
        super(context, name, factory, version);
    }

    /**
     * Create the database
     *
     * @param db database
     */
    @Override
    public void onCreate(SQLiteDatabase db) {
        try {
            db.execSQL(AllergensDataBaseAdapter.CREATE_TABLE);
            db.execSQL(CategoriesDataBaseAdapter.CREATE_TABLE);
            db.execSQL(IngredientAnalysisDataBaseAdapter.CREATE_TABLE);
            db.execSQL(LabelsDataBaseAdapter.CREATE_TABLE);
            db.execSQL(AdditivesClassesDataBaseAdapter.CREATE_TABLE);
            db.execSQL(NutrientsLevelsDataBaseAdapter.CREATE_TABLE);
            db.execSQL(NovaGroupsDataBaseAdapter.CREATE_TABLE);
            db.execSQL(AdditivesDataBaseAdapter.CREATE_TABLE);
            db.execSQL(WorkerDataBaseAdapter.CREATE_TABLE);
        } catch (Exception er) {
            Log.e("Error", "exception");
        }
    }

    /**
     * Upgrade database version
     *
     * @param db         database
     * @param oldVersion old database version
     * @param newVersion new database version
     */
    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        // Log the version upgrade.
        Log.w("TAG", "Upgrading from version " + oldVersion + " to " + newVersion + ", which will destroy all old data");

        // Drop all tables and create new database
        db.execSQL(AllergensDataBaseAdapter.DROP_TABLE);
        db.execSQL(CategoriesDataBaseAdapter.DROP_TABLE);
        db.execSQL(IngredientAnalysisDataBaseAdapter.DROP_TABLE);
        db.execSQL(LabelsDataBaseAdapter.DROP_TABLE);
        db.execSQL(AdditivesClassesDataBaseAdapter.DROP_TABLE);
        db.execSQL(NutrientsLevelsDataBaseAdapter.DROP_TABLE);
        db.execSQL(NovaGroupsDataBaseAdapter.DROP_TABLE);
        db.execSQL(AdditivesDataBaseAdapter.DROP_TABLE);
        db.execSQL(WorkerDataBaseAdapter.DROP_TABLE);

        onCreate(db);
    }
}
