package edu.uoc.tfg.nutriknow.sqlite.adapters;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

import edu.uoc.tfg.nutriknow.model.AdditivesClasses;
import edu.uoc.tfg.nutriknow.sqlite.DataBaseHelper;

/**
 * Database adapter for additives classes
 */
public class AdditivesClassesDataBaseAdapter extends DataBaseAdapter {

    private static final String TAG = "AdditivesAdapter";

    private static final String ES = "es";
    private static final String EN = "en";

    static final int DATABASE_VERSION = 1;

    // SQL Statement to create a new database.
    public static final String TABLE_NAME = "ADDITIVE_CLASS";
    public static final String CREATE_TABLE = "CREATE TABLE IF NOT EXISTS ADDITIVE_CLASS (ID integer primary key autoincrement, CODE text, NAME_ES text, DESCRIPTION_ES text, WIKIDATA_ES text, NAME_EN text, DESCRIPTION_EN text, WIKIDATA_EN text); ";
    public static final String DROP_TABLE = "DROP TABLE IF EXISTS ADDITIVE_CLASS";

    // SQL Columns
    static final String CODE_COLUMN = "CODE";
    static final String NAME_ES_COLUMN = "NAME_ES";
    static final String DESCRIPTION_ES_COLUMN = "DESCRIPTION_ES";
    static final String WIKIDATA_ES_COLUMN = "WIKIDATA_ES";
    static final String NAME_EN_COLUMN = "NAME_EN";
    static final String DESCRIPTION_EN_COLUMN = "DESCRIPTION_EN";
    static final String WIKIDATA_EN_COLUMN = "WIKIDATA_EN";

    /**
     * Adapter constructor
     *
     * @param _context context
     */
    public AdditivesClassesDataBaseAdapter(Context _context) {
        context = _context;
        dbHelper = new DataBaseHelper(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    /**
     * Method to open the Database
     *
     * @return adapter
     * @throws SQLException
     */
    public AdditivesClassesDataBaseAdapter open() throws SQLException {
        db = dbHelper.getWritableDatabase();
        return this;
    }

    /**
     * Method to close the Database
     */
    public void close() {
        db.close();
    }

    /**
     * Method to insert a record in Table
     *
     * @param additivesClasses source
     * @return boolean
     */
    public boolean insertAdditivesClasses(AdditivesClasses additivesClasses) {
        boolean result = true;
        long count = 0;
        try {

            // Assign values for each column.
            AdditivesClasses.AdditiveClass additiveClass = additivesClasses.getEnConcentratedPlantColour();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnBulkingAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnHumectant();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnAcid();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnColourRetentionAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnCarrier();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnVitamins();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnRaisingAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnNaturalFirmingAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnCookingEnhancer();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getFrAgentCremeux();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnGlazingAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnNaturalStabilizer();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnFoamingAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnFruitPreservative();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnEnhancer();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnFlavourEnhancer();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnSweetener();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnEmulsifier();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnGellingAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getFrDenreeAlimentaireColorante();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnNucleotides();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnOtherNutritionalSubstances();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnFlourTreatmentAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnPropellentGas();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnSequestrant();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnNaturalAntioxidant();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnStabiliser();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnThickener();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnPreservative();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getFrColorantDeLaCroute();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getFrAdditifsAlimentaires();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnAntioxidant();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnBreadImprover();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnColour();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnVitaminAndMineralBlend();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnCarbonatingAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnNaturalEmulsifier();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getFrConservateurDeSurface();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnNaturalColours();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnCoagulant();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnNaturalThickener();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnModifiedStarch();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnNaturalPreservative();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnBleachingAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnEmulsifyingSalts();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnPackagingGas();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnSaltSubstitute();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnAntiCakingAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnMinerals();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnAntiFoamingAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnAntiBulkingAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnFirmingAgent();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnAcidityRegulator();
            count += insertAdditiveClass(additiveClass);

            additiveClass = additivesClasses.getEnAminoAcids();
            count += insertAdditiveClass(additiveClass);

            Log.i(TAG, "Inserted additivesClasses -> " + String.valueOf(count));
        } catch (Exception ex) {
            result = false;
            ex.printStackTrace();
            Log.e(TAG, "Error in AdditivesClassesDataBaseAdapter.insertAdditives" + ex.getMessage());
        } finally {
            this.close();
        }
        return result;
    }

    /**
     * Method to get the contents
     *
     * @return additive classes content
     */
    public AdditivesClasses getAllAdditivesClasses() {
        Cursor cursor = null;
        try {
            db = dbHelper.getReadableDatabase();
            cursor = db.query(TABLE_NAME, null, null, null, null, null, null);

            // Move to first row
            if (!cursor.moveToFirst())
                return null;
            AdditivesClasses additivesClasses = new AdditivesClasses();
            do {
                AdditivesClasses.AdditiveClass additiveClass = new AdditivesClasses.AdditiveClass();
                additiveClass.code = cursor.getString(cursor.getColumnIndex(CODE_COLUMN));
                AdditivesClasses.AdditiveClass.Name name = new AdditivesClasses.AdditiveClass.Name();
                name.es = cursor.getString(cursor.getColumnIndex(NAME_ES_COLUMN));
                name.en = cursor.getString(cursor.getColumnIndex(NAME_EN_COLUMN));
                additiveClass.name = name;
                AdditivesClasses.AdditiveClass.Name description = new AdditivesClasses.AdditiveClass.Name();
                description.es = cursor.getString(cursor.getColumnIndex(DESCRIPTION_ES_COLUMN));
                description.en = cursor.getString(cursor.getColumnIndex(DESCRIPTION_EN_COLUMN));
                additiveClass.description = description;
                AdditivesClasses.AdditiveClass.Name wikidata = new AdditivesClasses.AdditiveClass.Name();
                wikidata.es = cursor.getString(cursor.getColumnIndex(WIKIDATA_ES_COLUMN));
                wikidata.en = cursor.getString(cursor.getColumnIndex(WIKIDATA_EN_COLUMN));
                additiveClass.wikidata = wikidata;

                getAdditive(additivesClasses, additiveClass);

            } while (cursor.moveToNext());
            return additivesClasses;
        } finally {
            cursor.close();
            this.close();
        }
    }

    /**
     * Get additive from database results
     *
     * @param additivesClasses source to inform
     * @param additiveClass    source to evaluate
     */
    private void getAdditive(AdditivesClasses additivesClasses, AdditivesClasses.AdditiveClass additiveClass) {

        switch (additiveClass.code) {
            case "en:bulking-agent":
                additivesClasses.enBulkingAgent = additiveClass;
                break;
            case "en:humectant":
                additivesClasses.enHumectant = additiveClass;
                break;
            case "en:acid":
                additivesClasses.enAcid = additiveClass;
                break;
            case "en:colour-retention-agent":
                additivesClasses.enColourRetentionAgent = additiveClass;
                break;
            case "en:carrier":
                additivesClasses.enCarrier = additiveClass;
                break;
            case "en:vitamins":
                additivesClasses.enVitamins = additiveClass;
                break;
            case "en:raising-agent":
                additivesClasses.enRaisingAgent = additiveClass;
                break;
            case "en:natural-firming-agent":
                additivesClasses.enNaturalFirmingAgent = additiveClass;
                break;
            case "en:cooking-enhancer":
                additivesClasses.enCookingEnhancer = additiveClass;
                break;
            case "fr:agent-cremeux":
                additivesClasses.frAgentCremeux = additiveClass;
                break;
            case "en:glazing-agent":
                additivesClasses.enGlazingAgent = additiveClass;
                break;
            case "en:natural-stabilizer":
                additivesClasses.enNaturalStabilizer = additiveClass;
                break;
            case "en:foaming-agent":
                additivesClasses.enFoamingAgent = additiveClass;
                break;
            case "en:fruit-preservative":
                additivesClasses.enFruitPreservative = additiveClass;
                break;
            case "en:enhancer":
                additivesClasses.enEnhancer = additiveClass;
                break;
            case "en:flavour-enhancer":
                additivesClasses.enFlavourEnhancer = additiveClass;
                break;
            case "en:sweetener":
                additivesClasses.enSweetener = additiveClass;
                break;
            case "en:emulsifier":
                additivesClasses.enEmulsifier = additiveClass;
                break;
            case "en:gelling-agent":
                additivesClasses.enGellingAgent = additiveClass;
                break;
            case "fr:denree-alimentaire-colorante":
                additivesClasses.frDenreeAlimentaireColorante = additiveClass;
                break;
            case "en:nucleotides":
                additivesClasses.enNucleotides = additiveClass;
                break;
            case "en:other-nutritional-substances":
                additivesClasses.enOtherNutritionalSubstances = additiveClass;
                break;
            case "en:flour-treatment-agent":
                additivesClasses.enFlourTreatmentAgent = additiveClass;
                break;
            case "en:propellent-gas":
                additivesClasses.enPropellentGas = additiveClass;
                break;
            case "en:sequestrant":
                additivesClasses.enSequestrant = additiveClass;
                break;
            case "en:natural-antioxidant":
                additivesClasses.enNaturalAntioxidant = additiveClass;
                break;
            case "en:stabiliser":
                additivesClasses.enStabiliser = additiveClass;
                break;
            case "en:thickener":
                additivesClasses.enThickener = additiveClass;
                break;
            case "en:preservative":
                additivesClasses.enPreservative = additiveClass;
                break;
            case "fr:colorant-de-la-croute":
                additivesClasses.frColorantDeLaCroute = additiveClass;
                break;
            case "fr:additifs-alimentaires":
                additivesClasses.frAdditifsAlimentaires = additiveClass;
                break;
            case "en:antioxidant":
                additivesClasses.enAntioxidant = additiveClass;
                break;
            case "en:bread-improver":
                additivesClasses.enBreadImprover = additiveClass;
                break;
            case "en:colour":
                additivesClasses.enColour = additiveClass;
                break;
            case "en:vitamin-and-mineral-blend":
                additivesClasses.enVitaminAndMineralBlend = additiveClass;
                break;
            case "en:carbonating-agent":
                additivesClasses.enCarbonatingAgent = additiveClass;
                break;
            case "en:natural-emulsifier":
                additivesClasses.enNaturalEmulsifier = additiveClass;
                break;
            case "fr:conservateur-de-surface":
                additivesClasses.frConservateurDeSurface = additiveClass;
                break;
            case "en:natural-colours":
                additivesClasses.enNaturalColours = additiveClass;
                break;
            case "en:coagulant":
                additivesClasses.enCoagulant = additiveClass;
                break;
            case "en:natural-thickener":
                additivesClasses.enNaturalThickener = additiveClass;
                break;
            case "en:modified-starch":
                additivesClasses.enModifiedStarch = additiveClass;
                break;
            case "en:natural-preservative":
                additivesClasses.enNaturalPreservative = additiveClass;
                break;
            case "en:bleaching-agent":
                additivesClasses.enBleachingAgent = additiveClass;
                break;
            case "en:emulsifying-salts":
                additivesClasses.enEmulsifyingSalts = additiveClass;
                break;
            case "en:packaging-gas":
                additivesClasses.enPackagingGas = additiveClass;
                break;
            case "en:salt-substitute":
                additivesClasses.enSaltSubstitute = additiveClass;
                break;
            case "en:anti-caking-agent":
                additivesClasses.enAntiCakingAgent = additiveClass;
                break;
            case "en:minerals":
                additivesClasses.enMinerals = additiveClass;
                break;
            case "en:anti-foaming-agent":
                additivesClasses.enAntiFoamingAgent = additiveClass;
                break;
            case "en:anti-bulking-agent":
                additivesClasses.enAntiBulkingAgent = additiveClass;
                break;
            case "en:firming-agent":
                additivesClasses.enFirmingAgent = additiveClass;
                break;
            case "en:acidity-regulator":
                additivesClasses.enAcidityRegulator = additiveClass;
                break;
            case "en:amino-acids":
                additivesClasses.enAminoAcids = additiveClass;
                break;
        }
    }

    /**
     * Insert additive in database
     *
     * @param additiveClass source to insert
     * @return count of rows inserted
     */
    private long insertAdditiveClass(AdditivesClasses.AdditiveClass additiveClass) {
        long count = 0;
        if (additiveClass.code != null && !additiveClass.code.isEmpty()) {
            ContentValues newValues = new ContentValues();
            newValues.put(CODE_COLUMN, additiveClass.code);
            if (additiveClass.name != null) {
                newValues.put(NAME_EN_COLUMN, additiveClass.name.en);
                if (additiveClass.name.es != null) {
                    newValues.put(NAME_ES_COLUMN, additiveClass.name.es);
                }
            }
            if (additiveClass.description != null) {
                newValues.put(DESCRIPTION_ES_COLUMN, additiveClass.description.es);
                newValues.put(DESCRIPTION_EN_COLUMN, additiveClass.description.en);
            }
            if (additiveClass.wikidata != null) {
                newValues.put(WIKIDATA_ES_COLUMN, additiveClass.wikidata.es);
                newValues.put(WIKIDATA_EN_COLUMN, additiveClass.wikidata.en);
            }
            db = dbHelper.getWritableDatabase();
            count = db.insert(TABLE_NAME, null, newValues);
        }

        return count;
    }
}
