package edu.uoc.tfg.nutriknow.sqlite.adapters;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

import edu.uoc.tfg.nutriknow.model.NovaGroups;
import edu.uoc.tfg.nutriknow.sqlite.DataBaseHelper;

/**
 * Database adapter for nova groups
 */
public class NovaGroupsDataBaseAdapter extends DataBaseAdapter {

    private static final String TAG = "NovaGroupsAdapter";

    private static final String ES = "es";
    private static final String EN = "en";

    static final int DATABASE_VERSION = 1;

    // SQL Statement to create a new database.
    public static final String TABLE_NAME = "NOVA_GROUP";
    public static final String CREATE_TABLE = "CREATE TABLE IF NOT EXISTS NOVA_GROUP (ID integer primary key autoincrement, CODE text, NAME_ES text, NAME_EN text); ";
    public static final String DROP_TABLE = "DROP TABLE IF EXISTS NOVA_GROUP";

    // SQL Columns
    static final String CODE_COLUMN = "CODE";
    static final String NAME_ES_COLUMN = "NAME_ES";
    static final String NAME_EN_COLUMN = "NAME_EN";

    /**
     * Adapter constructor
     *
     * @param _context context
     */
    public NovaGroupsDataBaseAdapter(Context _context) {
        context = _context;
        dbHelper = new DataBaseHelper(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    /**
     * Method to open the Database
     *
     * @return adapter
     * @throws SQLException
     */
    public NovaGroupsDataBaseAdapter open() throws SQLException {
        db = dbHelper.getWritableDatabase();
        return this;
    }

    /**
     * Method to close the Database
     */
    public void close() {
        db.close();
    }

    /**
     * Method to insert a record in Table
     *
     * @param novaGroups source
     * @return boolean
     */
    public boolean insertNovaGroups(NovaGroups novaGroups) {
        boolean result = true;
        long count = 0;
        try {

            // Assign values for each column.
            NovaGroups.NovaGroup novaGroup = novaGroups.getEnNotApplicable();
            count += insertNovaGroup(novaGroup);

            novaGroup = novaGroups.getEn1UnprocessedOrMinimallyProcessedFoods();
            count += insertNovaGroup(novaGroup);

            novaGroup = novaGroups.getEn2ProcessedCulinaryIngredients();
            count += insertNovaGroup(novaGroup);

            novaGroup = novaGroups.getEn3ProcessedFoods();
            count += insertNovaGroup(novaGroup);

            novaGroup = novaGroups.getEn4UltraProcessedFoodAndDrinkProducts();
            count += insertNovaGroup(novaGroup);

            Log.i(TAG, "Inserted nova groups -> " + String.valueOf(count));
        } catch (Exception ex) {
            result = false;
            Log.e(TAG, "Error in NovaGroupsDataBaseAdapter.insertNovaGroups" + ex.getMessage());
        } finally {
            this.close();
        }
        return result;
    }

    /**
     * Method to get the contents
     *
     * @return nova groups content
     */
    public NovaGroups getAllNovaGropus() {

        Cursor cursor = null;
        try {
            db = dbHelper.getReadableDatabase();
            cursor = db.query(TABLE_NAME, null, null, null, null, null, null);

            // Move to first row
            if (!cursor.moveToFirst())
                return null;
            NovaGroups novaGroups = new NovaGroups();
            do {
                NovaGroups.NovaGroup novaGroup = new NovaGroups.NovaGroup();
                novaGroup.code = cursor.getString(cursor.getColumnIndex(CODE_COLUMN));
                NovaGroups.NovaGroup.Name name = new NovaGroups.NovaGroup.Name();
                name.es = cursor.getString(cursor.getColumnIndex(NAME_ES_COLUMN));
                name.en = cursor.getString(cursor.getColumnIndex(NAME_EN_COLUMN));
                novaGroup.name = name;

                getNovaGroup(novaGroups, novaGroup);

            } while (cursor.moveToNext());
            return novaGroups;
        } finally {
            cursor.close();
            this.close();
        }
    }

    private void getNovaGroup(NovaGroups novaGroups, NovaGroups.NovaGroup novaGroup) {

        switch (novaGroup.code) {
            case "en:4-ultra-processed-food-and-drink-products":
                novaGroups.en4UltraProcessedFoodAndDrinkProducts = novaGroup;
                break;
            case "en:3-processed-foods":
                novaGroups.en3ProcessedFoods = novaGroup;
                break;
            case "en:not-applicable":
                novaGroups.enNotApplicable = novaGroup;
                break;
            case "en:1-unprocessed-or-minimally-processed-foods":
                novaGroups.en1UnprocessedOrMinimallyProcessedFoods = novaGroup;
                break;
            case "en:2-processed-culinary-ingredients":
                novaGroups.en2ProcessedCulinaryIngredients = novaGroup;
                break;
        }
    }

    private long insertNovaGroup(NovaGroups.NovaGroup novaGroup) {
        long count = 0;
        if (novaGroup.code != null && !novaGroup.code.isEmpty()) {
            ContentValues newValues = new ContentValues();
            newValues.put(CODE_COLUMN, novaGroup.code);
            newValues.put(NAME_ES_COLUMN, novaGroup.name.es);
            newValues.put(NAME_EN_COLUMN, novaGroup.name.en);
            // Insert the row into your table
            db = dbHelper.getWritableDatabase();
            count = db.insert(TABLE_NAME, null, newValues);
        }

        return count;
    }
}
