package edu.uoc.tfg.nutriknow.sqlite.adapters;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

import edu.uoc.tfg.nutriknow.model.NutrientsLevel;
import edu.uoc.tfg.nutriknow.sqlite.DataBaseHelper;

/**
 * Database adapter for nutrients levels
 */
public class NutrientsLevelsDataBaseAdapter extends DataBaseAdapter {

    private static final String TAG = "NutrientsLevelsAdapter";

    private static final String ES = "es";
    private static final String EN = "en";

    static final int DATABASE_VERSION = 1;

    // SQL Statement to create a new database.
    public static final String TABLE_NAME = "NUTRIENT_LEVEL";
    public static final String CREATE_TABLE = "CREATE TABLE IF NOT EXISTS NUTRIENT_LEVEL (ID integer primary key autoincrement, CODE text, NAME_ES text, NAME_EN text); ";
    public static final String DROP_TABLE = "DROP TABLE IF EXISTS NUTRIENT_LEVEL";

    // SQL Columns
    static final String CODE_COLUMN = "CODE";
    static final String NAME_ES_COLUMN = "NAME_ES";
    static final String NAME_EN_COLUMN = "NAME_EN";

    /**
     * Method to open the Database
     *
     * @return adapter
     * @throws SQLException
     */
    public NutrientsLevelsDataBaseAdapter(Context _context) {
        context = _context;
        dbHelper = new DataBaseHelper(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    //

    /**
     * Method to open the Database
     *
     * @return adapter
     * @throws SQLException
     */
    public NutrientsLevelsDataBaseAdapter open() throws SQLException {
        db = dbHelper.getWritableDatabase();
        return this;
    }

    /**
     * Method to close the Database
     */
    public void close() {
        db.close();
    }

    /**
     * Method to insert a record in Table
     *
     * @param nutrientsLevel source
     * @return boolean
     */
    public boolean insertNutrientsLevels(NutrientsLevel nutrientsLevel) {
        boolean result = true;
        long count = 0;
        try {

            // Assign values for each column.
            NutrientsLevel.NutrientLevel nutrientLevel = nutrientsLevel.getEnFatInHighQuantity();
            count += insertNutrientLevel(nutrientLevel);

            nutrientLevel = nutrientsLevel.getEnFatInLowQuantity();
            count += insertNutrientLevel(nutrientLevel);

            nutrientLevel = nutrientsLevel.getEnFatInModerateQuantity();
            count += insertNutrientLevel(nutrientLevel);

            nutrientLevel = nutrientsLevel.getEnSaltInHighQuantity();
            count += insertNutrientLevel(nutrientLevel);

            nutrientLevel = nutrientsLevel.getEnSaltInLowQuantity();
            count += insertNutrientLevel(nutrientLevel);

            nutrientLevel = nutrientsLevel.getEnSaltInModerateQuantity();
            count += insertNutrientLevel(nutrientLevel);

            nutrientLevel = nutrientsLevel.getEnSaturatedFatInHighQuantity();
            count += insertNutrientLevel(nutrientLevel);

            nutrientLevel = nutrientsLevel.getEnSaturatedFatInLowQuantity();
            count += insertNutrientLevel(nutrientLevel);

            nutrientLevel = nutrientsLevel.getEnSaturatedFatInModerateQuantity();
            count += insertNutrientLevel(nutrientLevel);

            nutrientLevel = nutrientsLevel.getEnSugarsInHighQuantity();
            count += insertNutrientLevel(nutrientLevel);

            nutrientLevel = nutrientsLevel.getEnSugarsInLowQuantity();
            count += insertNutrientLevel(nutrientLevel);

            nutrientLevel = nutrientsLevel.getEnSugarsInModerateQuantity();
            count += insertNutrientLevel(nutrientLevel);

            Log.i(TAG, "Inserted nutrients levels -> " + String.valueOf(count));
        } catch (Exception ex) {
            result = false;
            Log.e(TAG, "Error in NutrientsLevelsDataBaseAdapter.insertNutrientsLevels" + ex.getMessage());
        } finally {
            this.close();
        }
        return result;
    }

    /**
     * Method to get the contents
     *
     * @return nutrients levels content
     */
    public NutrientsLevel getAllNutrientsLevel() {

        Cursor cursor = null;
        try {
            db = dbHelper.getReadableDatabase();
            cursor = db.query(TABLE_NAME, null, null, null, null, null, null);

            // Move to first row
            if (!cursor.moveToFirst())
                return null;
            NutrientsLevel nutrientsLevel = new NutrientsLevel();
            do {
                NutrientsLevel.NutrientLevel nutrientLevel = new NutrientsLevel.NutrientLevel();
                nutrientLevel.code = cursor.getString(cursor.getColumnIndex(CODE_COLUMN));
                NutrientsLevel.NutrientLevel.Name name = new NutrientsLevel.NutrientLevel.Name();
                name.es = cursor.getString(cursor.getColumnIndex(NAME_ES_COLUMN));
                name.en = cursor.getString(cursor.getColumnIndex(NAME_EN_COLUMN));
                nutrientLevel.name = name;

                getNutrientLevel(nutrientsLevel, nutrientLevel);

            } while (cursor.moveToNext());
            return nutrientsLevel;
        } finally {
            cursor.close();
            this.close();
        }
    }

    private void getNutrientLevel(NutrientsLevel nutrientsLevel, NutrientsLevel.NutrientLevel nutrientLevel) {

        switch (nutrientLevel.code) {
            case "en:fat-in-high-quantity":
                nutrientsLevel.enFatInHighQuantity = nutrientLevel;
                break;
            case "en:sugars-in-moderate-quantity":
                nutrientsLevel.enSugarsInModerateQuantity = nutrientLevel;
                break;
            case "en:fat-in-moderate-quantity":
                nutrientsLevel.enFatInModerateQuantity = nutrientLevel;
                break;
            case "en:sugars-in-high-quantity":
                nutrientsLevel.enSugarsInHighQuantity = nutrientLevel;
                break;
            case "en:fat-in-low-quantity":
                nutrientsLevel.enFatInLowQuantity = nutrientLevel;
                break;
            case "en:saturated-fat-in-high-quantity":
                nutrientsLevel.enSaturatedFatInHighQuantity = nutrientLevel;
                break;
            case "en:saturated-fat-in-low-quantity":
                nutrientsLevel.enSaturatedFatInLowQuantity = nutrientLevel;
                break;
            case "en:sugars-in-low-quantity":
                nutrientsLevel.enSugarsInLowQuantity = nutrientLevel;
                break;
            case "en:salt-in-moderate-quantity":
                nutrientsLevel.enSaltInModerateQuantity = nutrientLevel;
                break;
            case "en:salt-in-low-quantity":
                nutrientsLevel.enSaltInLowQuantity = nutrientLevel;
                break;
            case "en:salt-in-high-quantity":
                nutrientsLevel.enSaltInHighQuantity = nutrientLevel;
                break;
            case "en:saturated-fat-in-moderate-quantity":
                nutrientsLevel.enSaturatedFatInModerateQuantity = nutrientLevel;
                break;
        }
    }

    private long insertNutrientLevel(NutrientsLevel.NutrientLevel nutrientLevel) {
        long count = 0;
        if (nutrientLevel.code != null && !nutrientLevel.code.isEmpty()) {
            ContentValues newValues = new ContentValues();
            newValues.put(CODE_COLUMN, nutrientLevel.code);
            newValues.put(NAME_ES_COLUMN, nutrientLevel.name.es);
            newValues.put(NAME_EN_COLUMN, nutrientLevel.name.en);
            // Insert the row into your table
            db = dbHelper.getWritableDatabase();
            count = db.insert(TABLE_NAME, null, newValues);
        }

        return count;
    }
}
