package edu.uoc.tfg.nutriknow.tasks;

import android.content.Context;
import android.os.AsyncTask;
import android.util.Log;

import org.json.JSONObject;

import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;

import edu.uoc.tfg.nutriknow.R;

/**
 * Asynchronous class for sending push notifications from firebase
 */
public class SendNotificationAsyncTask extends AsyncTask<String, Void, Void> {

    private final static String TAG = "SendNotificationService";
    private final static String AUTH_KEY_FCM = "AAAAS1CiTTM:APA91bFMsrWS9vf5N44YYuojtKSzk9Nj8MJIzDsqVLFJo3IMSHjb_2wVrQ4Lc9EvAuidSkPFxZrct9atUb77HdBWJ7GMnVwybvcN_nsJ6Zm6EbKJ-iefA_HS59b6kCH4lxADxWbH60vX";
    private final static String API_URL_FCM = "https://fcm.googleapis.com/fcm/send";

    private final Context context;

    /**
     * Class constructor
     *
     * @param ctx context
     */
    public SendNotificationAsyncTask(Context ctx) {
        this.context = ctx;
    }

    /**
     * Method for performing the shipment in background
     *
     * @param parameters token, productCode and productName
     * @return void
     */
    protected Void doInBackground(String... parameters) {
        try {
            pushFCMNotification(parameters[0], parameters[1], parameters[2]);
        } catch (Exception e) {
            Log.i(TAG, "Error -> " + e.getMessage());
        }
        return null;
    }

    /**
     * Send push notification using firebase
     *
     * @param token       firebase token
     * @param productCode product barcode
     * @param productName product name
     * @throws Exception
     */
    private void pushFCMNotification(String token, String productCode, String productName) throws Exception {

        URL url = new URL(API_URL_FCM);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();

        conn.setUseCaches(false);
        conn.setDoInput(true);
        conn.setDoOutput(true);

        conn.setRequestMethod("POST");
        conn.setRequestProperty("Authorization", "key=" + AUTH_KEY_FCM);
        conn.setRequestProperty("Content-Type", "application/json");

        String body = context.getString(R.string.product_changed_message);
        body = body.replace("[PRODUCT]", productName);
        body = body.replace("[BARCODE]", productCode);

        JSONObject notification = new JSONObject();
        notification.put("to", token.trim());
        notification.put("priority", "high");
        JSONObject info = new JSONObject();
        info.put("title", "NutriKNow!");
        info.put("text", body);
        notification.put("notification", info);

        OutputStreamWriter wr = new OutputStreamWriter(conn.getOutputStream());
        wr.write(notification.toString());
        wr.flush();
        wr.close();

        int responseCode = conn.getResponseCode();
        Log.i(TAG, "Response Code : " + responseCode + " - " + conn.getResponseMessage());

    }
}
