package edu.uoc.tfg.nutriknow.utils;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.tfg.nutriknow.model.Additives;
import edu.uoc.tfg.nutriknow.model.AdditivesClasses;
import edu.uoc.tfg.nutriknow.model.Allergens;
import edu.uoc.tfg.nutriknow.model.Categories;
import edu.uoc.tfg.nutriknow.model.IngredientsAnalysis;
import edu.uoc.tfg.nutriknow.model.Labels;
import edu.uoc.tfg.nutriknow.model.NovaGroups;
import edu.uoc.tfg.nutriknow.model.NutrientsLevel;
import edu.uoc.tfg.nutriknow.model.WorkerData;
import edu.uoc.tfg.nutriknow.sqlite.adapters.AdditivesClassesDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.AdditivesDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.AllergensDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.CategoriesDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.IngredientAnalysisDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.LabelsDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.NovaGroupsDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.NutrientsLevelsDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.WorkerDataBaseAdapter;

/**
 * Utils class for managing database catalogs
 */
public class DatabaseCatalogUtils {

    /**
     * Save allergens in database
     *
     * @param adapter       adapter
     * @param allergensList source
     * @param language      selected locale
     */
    public static void saveAllergensInLocalDatabase(AllergensDataBaseAdapter adapter,
                                                    List<Allergens.Allergen> allergensList,
                                                    String language) {
        adapter.insertAllergens(allergensList, language);
    }

    /**
     * Retrieve allergens from database
     *
     * @param adapter  adapter
     * @param language selected locale
     * @return list of data
     */
    public static List<Allergens.Allergen> retrieveAllergensFromLocalDatabase(AllergensDataBaseAdapter adapter,
                                                                              String language) {
        List<Allergens.Allergen> allergens = new ArrayList<>();
        if (adapter.tableExists(adapter.TABLE_NAME, true)) {
            allergens = adapter.getAllergensByLanguage(language);
        }
        return allergens;
    }

    /**
     * Save categories in database
     *
     * @param adapter        adapter
     * @param categoriesList source
     * @param language       selected locale
     */
    public static void saveCategoriesInLocalDatabase(CategoriesDataBaseAdapter adapter,
                                                     List<Categories.Category> categoriesList,
                                                     String language) {
        adapter.insertCategories(categoriesList, language);
    }

    /**
     * Retrieve categories from database
     *
     * @param adapter  adapter
     * @param language selected locale
     * @return list of data
     */
    public static List<Categories.Category> retrieveCategoriesFromLocalDatabase(CategoriesDataBaseAdapter adapter,
                                                                                String language) {
        List<Categories.Category> categories = new ArrayList<>();
        if (adapter.tableExists(adapter.TABLE_NAME, true)) {
            categories = adapter.getCategoriesByLanguage(language);
        }
        return categories;
    }

    /**
     * Save labels in database
     *
     * @param adapter   adapter
     * @param labelList source
     * @param language  selected locale
     */
    public static void saveLabelsInLocalDatabase(LabelsDataBaseAdapter adapter, List<Labels.Label> labelList,
                                                 String language) {
        adapter.insertLabels(labelList, language);
    }

    /**
     * Retrieve labels from database
     *
     * @param adapter  adapter
     * @param language selected locale
     * @return list of data
     */
    public static List<Labels.Label> retrieveLabelsFromLocalDatabase(LabelsDataBaseAdapter adapter,
                                                                     String language) {
        List<Labels.Label> labelList = new ArrayList<>();
        if (adapter.tableExists(adapter.TABLE_NAME, true)) {
            labelList = adapter.getLabelsByLanguage(language);
        }
        return labelList;
    }

    /**
     * Save ingredients analysis in database
     *
     * @param adapter                 adapter
     * @param ingredientsAnalysisList source
     * @param language                selected locale
     */
    public static void saveIngredientsAnalysisInLocalDatabase(IngredientAnalysisDataBaseAdapter adapter,
                                                              List<IngredientsAnalysis.IngredientAnalysis> ingredientsAnalysisList,
                                                              String language) {
        adapter.insertIngredientsAnalysis(ingredientsAnalysisList, language);
    }

    /**
     * Retrieve ingredients analysis from database
     *
     * @param adapter  adapter
     * @param language selected locale
     * @return list of data
     */
    public static List<IngredientsAnalysis.IngredientAnalysis> retrieveIngredientsAnalysisFromLocalDatabase(
            IngredientAnalysisDataBaseAdapter adapter, String language) {
        List<IngredientsAnalysis.IngredientAnalysis> ingredientsAnalysisList = new ArrayList<>();
        if (adapter.tableExists(adapter.TABLE_NAME, true)) {
            ingredientsAnalysisList = adapter.getIngredientsAnalysisByLanguage(language);
        }
        return ingredientsAnalysisList;
    }

    /**
     * Save additives clasess in database
     *
     * @param adapter          adapter
     * @param additivesClasses source
     */
    public static void saveAdditivesClassesInLocalDatabase(AdditivesClassesDataBaseAdapter adapter, AdditivesClasses additivesClasses) {
        adapter.insertAdditivesClasses(additivesClasses);
    }

    /**
     * Retrieve additives classes from database
     *
     * @param adapter adapter
     * @return additives classes
     */
    public static AdditivesClasses retrieveAdditivesClassesFromLocalDatabase(AdditivesClassesDataBaseAdapter adapter) {
        AdditivesClasses additivesClasses = null;
        if (adapter.tableExists(adapter.TABLE_NAME, true)) {
            additivesClasses = adapter.getAllAdditivesClasses();
        }
        return additivesClasses;
    }

    /**
     * Save nutrients levels in database
     *
     * @param adapter        adapter
     * @param nutrientsLevel source
     */
    public static void saveNutrientsLevelsInLocalDatabase(NutrientsLevelsDataBaseAdapter adapter,
                                                          NutrientsLevel nutrientsLevel) {
        adapter.insertNutrientsLevels(nutrientsLevel);
    }

    /**
     * Retrieve nutrients levels from database
     *
     * @param adapter adapter
     * @return nutrients level
     */
    public static NutrientsLevel retrieveNutrientsLevelsFromLocalDatabase(NutrientsLevelsDataBaseAdapter adapter) {
        NutrientsLevel nutrientsLevel = null;
        if (adapter.tableExists(adapter.TABLE_NAME, true)) {
            nutrientsLevel = adapter.getAllNutrientsLevel();
        }
        return nutrientsLevel;
    }

    /**
     * Save nova groups in database
     *
     * @param adapter    adapter
     * @param novaGroups source
     */
    public static void saveNovaGroupsInLocalDatabase(NovaGroupsDataBaseAdapter adapter, NovaGroups novaGroups) {
        adapter.insertNovaGroups(novaGroups);
    }

    /**
     * Retrieve nova groups from database
     *
     * @param adapter adapter
     * @return nova groups
     */
    public static NovaGroups retrieveNovaGroupsFromLocalDatabase(NovaGroupsDataBaseAdapter adapter) {
        NovaGroups novaGroups = null;
        if (adapter.tableExists(adapter.TABLE_NAME, true)) {
            novaGroups = adapter.getAllNovaGropus();
        }
        return novaGroups;
    }

    /**
     * Save additives in database
     *
     * @param adapter       adapter
     * @param additivesList source
     */
    public static void saveAdditivesInLocalDatabase(AdditivesDataBaseAdapter adapter,
                                                    List<Additives> additivesList) {
        adapter.insertAdditives(additivesList);
    }

    /**
     * Retrieve additives from database
     *
     * @param adapter adapter
     * @return list of data
     */
    public static List<Additives> retrieveAdditivesFromLocalDatabase(AdditivesDataBaseAdapter adapter) {
        List<Additives> additives = new ArrayList<>();
        if (adapter.tableExists(adapter.TABLE_NAME, true)) {
            additives = adapter.getAllAdditives();
        }
        return additives;
    }

    /**
     * Save worker data in database
     *
     * @param adapter       adapter
     * @param userUuid      user uuid
     * @param firebaseToken firebase token
     */
    public static void saveWorkerDataInLocalDatabase(WorkerDataBaseAdapter adapter,
                                                     String userUuid, String firebaseToken) {
        adapter.insertWorkerData(userUuid, firebaseToken);
    }

    /**
     * Retrieve worker data from database
     *
     * @param adapter adapter
     * @return worker data
     */
    public static WorkerData retrieveWorkerDataLocalDatabase(WorkerDataBaseAdapter adapter) {
        WorkerData workerData = null;
        if (adapter.tableExists(adapter.TABLE_NAME, true)) {
            workerData = adapter.getWorkerData();
        }
        return workerData;
    }

}
