package edu.uoc.tfg.nutriknow.utils;

import android.content.Context;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ListAdapter;
import android.widget.ListView;

import androidx.fragment.app.Fragment;

import java.util.Locale;

import edu.uoc.tfg.nutriknow.views.fragments.NutritionalInfoFragment;
import edu.uoc.tfg.nutriknow.views.fragments.UserAccountFragment;

/**
 * Utils class for managing listview
 */
public class ListViewUtils {

    /**
     * Calculated ListView height, in the adaptive layout of the parent
     *
     * @param listView list to resize
     * @param fragment fragment
     * @param ctx      context
     */
    public static void setListViewHeightBasedOnChildren(ListView listView, Fragment fragment, Context... ctx) {
        //  ListView obtain the corresponding Adapter
        ListAdapter listAdapter = listView.getAdapter();
        if (listAdapter == null) {
            return;
        }

        int totalHeight = 0;
        double rowHeight = 0;
        for (int i = 0, len = listAdapter.getCount(); i < len; i++) {
            //  listAdapter.getCount () returns the number of data items
            View listItem = listAdapter.getView(i, null, listView);
            //  Calculating the width and height of the child View
            listItem.measure(0, 0);
            //  All statistics the total height of the child
            if (fragment instanceof UserAccountFragment) {
                totalHeight += listItem.getMeasuredHeight();
            } else {
                totalHeight += listItem.getMeasuredHeight() * 0.7;
                rowHeight = listItem.getMeasuredHeight() * 0.7;
            }
        }

        ViewGroup.LayoutParams params = listView.getLayoutParams();

        if (fragment instanceof NutritionalInfoFragment) {
            Locale currentLocale = ctx[0].getResources().getConfiguration().locale;
            if ("es".equals(currentLocale.getLanguage())) {
                totalHeight += rowHeight * 0.5;
            }
        }

        params.height = totalHeight + (listView.getDividerHeight() * (listAdapter.getCount() - 1));
        //  listView.getDividerHeight () Gets children between the height occupied by the delimiter
        //  params.height finally get the desired height ListView complete the entire display
        listView.setLayoutParams(params);
    }
}
