package edu.uoc.tfg.nutriknow.utils;

import android.app.ProgressDialog;
import android.content.Context;
import android.util.Log;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.ValueEventListener;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.adapters.FavoritesRecyclerAdapter;
import edu.uoc.tfg.nutriknow.adapters.SearchesRecyclerAdapter;
import edu.uoc.tfg.nutriknow.model.Additives;
import edu.uoc.tfg.nutriknow.model.AdditivesClasses;
import edu.uoc.tfg.nutriknow.model.Allergens;
import edu.uoc.tfg.nutriknow.model.Categories;
import edu.uoc.tfg.nutriknow.model.CompleteProductData;
import edu.uoc.tfg.nutriknow.model.Favorite;
import edu.uoc.tfg.nutriknow.model.IngredientsAnalysis;
import edu.uoc.tfg.nutriknow.model.Labels;
import edu.uoc.tfg.nutriknow.model.NovaGroups;
import edu.uoc.tfg.nutriknow.model.NutrientsLevel;
import edu.uoc.tfg.nutriknow.model.Nutriments;
import edu.uoc.tfg.nutriknow.model.Product;
import edu.uoc.tfg.nutriknow.model.Search;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;
import edu.uoc.tfg.nutriknow.views.fragments.FavoritesFragment;
import edu.uoc.tfg.nutriknow.views.fragments.SearchesFragment;

/**
 * Utils class for managing product information
 */
public class ProductInfoUtils {

    private final static String TAG = "ProductInfoUtils";
    private static final String ES = "es";
    private static final String EN = "en";

    private static final String PERIODICITY_D = "D";
    private static final String PERIODICITY_W = "W";
    private static final String PERIODICITY_N = "N";

    /**
     * Check if a string is null of empty
     *
     * @param resource input data
     * @return string
     */
    public static String checkIfNullOrEmptyString(String resource) {
        String resourceToReturn = "";
        if (resource != null && !resource.isEmpty()) {
            resourceToReturn = resource;
        }
        return resourceToReturn;
    }

    /**
     * Check if a integer is null of empty
     *
     * @param resource input data
     * @return integer
     */
    public static Integer checkIfNullOrEmptyInteger(Integer resource) {
        Integer resourceToReturn = 0;
        if (resource != null) {
            resourceToReturn = resource;
        }
        return resourceToReturn;
    }

    /**
     * Convert string to capital letter
     *
     * @param resource input data
     * @return string
     */
    public static String convertStringToCapitalLetter(String resource) {
        String resourcesToReturn = "";
        if (resource != null && !resource.isEmpty()) {
            resourcesToReturn = resource.substring(0, 1).toUpperCase() + resource.substring(1).toLowerCase();
        }
        return resourcesToReturn;
    }

    /**
     * Convert list content to capital letter
     *
     * @param resources input data
     * @return list of strings
     */
    public static List<String> convertListToCapitalLetter(List<String> resources) {
        List<String> resourcesToReturn = new ArrayList<>();
        if (resources != null && !resources.isEmpty()) {
            for (String resource : resources) {
                resource = resource.substring(0, 1).toUpperCase() + resource.substring(1).toLowerCase();
                resourcesToReturn.add(resource);
            }
            Collections.sort(resourcesToReturn);
        }
        return resourcesToReturn;
    }

    /**
     * Convert string to list
     *
     * @param resource input data
     * @return list of strings
     */
    public static List<String> convertStringToList(String resource) {
        List<String> resourcesToReturn = new ArrayList<>();
        if (resource != null && !resource.isEmpty()) {
            String[] resourceArray = resource.split(",");
            resourcesToReturn = Arrays.asList(resourceArray);
        }
        return resourcesToReturn;
    }

    /**
     * Convert list content to string
     *
     * @param resources input data
     * @return string
     */
    public static String convertListToString(List<String> resources) {
        String resourcesToReturn = "";
        if (resources != null && !resources.isEmpty()) {
            for (String resource : resources) {
                resourcesToReturn = resourcesToReturn + resource + ", ";
            }
            resourcesToReturn = resourcesToReturn.substring(0, resourcesToReturn.length() - 2);
        }
        return resourcesToReturn;
    }

    /**
     * Convert string to date
     *
     * @param resource input data
     * @return date
     */
    public static Date convertStringToDate(String resource) {
        Date date = null;
        if (resource != null && !resource.isEmpty()) {
            SimpleDateFormat format = new SimpleDateFormat("dd/MM/yyyy");
            try {
                String[] resourceArray = resource.split("/");
                if (resourceArray.length > 1) {
                    String month = resourceArray[1];
                    String day = resourceArray[0] + "/";
                    if (resourceArray.length == 2) {
                        month = resourceArray[0];
                        day = "01/";
                    }
                    if (Integer.valueOf(month) < 10 && !month.startsWith("0")) {
                        month = "0" + month;
                    }
                    if (resourceArray.length == 2) {
                        resource = day + month + "/" + resourceArray[1];
                    } else {
                        resource = day + month + "/" + resourceArray[2];
                    }
                    date = format.parse(resource);
                }
            } catch (ParseException e) {
                Log.e(TAG, e.getMessage());
            }
        }
        return date;
    }

    /**
     * Round float value
     *
     * @param resource input data
     * @param places   number of positions
     * @return float
     */
    public static float roundFloat(float resource, int places) {
        BigDecimal bd = new BigDecimal(Float.toString(resource));
        bd = bd.setScale(places, BigDecimal.ROUND_HALF_UP);
        return bd.floatValue();
    }

    /**
     * Parse label from tag
     *
     * @param resources input data
     * @param catalog   required catalog
     * @return list of labels
     */
    public static List<Labels.Label> parseLabelFromTag(List<String> resources, List<Labels.Label> catalog) {
        List<Labels.Label> resourcesToReturn = new ArrayList<>();
        if (resources != null && !resources.isEmpty()) {
            for (String code : resources) {
                for (Labels.Label label : catalog) {
                    if (code.equals(label.id)) {
                        resourcesToReturn.add(label);
                        Log.i(TAG, "label -> " + label.name);
                        break;
                    }
                }
            }
        }
        return resourcesToReturn;
    }

    /**
     * Parse allergen from tag
     *
     * @param resources input data
     * @param catalog   required catalog
     * @return list of allergens
     */
    public static List<Allergens.Allergen> parseAllergenFromTag(List<String> resources, List<Allergens.Allergen> catalog) {
        List<Allergens.Allergen> resourcesToReturn = new ArrayList<>();
        if (resources != null && !resources.isEmpty()) {
            for (String code : resources) {
                for (Allergens.Allergen allergen : catalog) {
                    if (code.equals(allergen.id)) {
                        resourcesToReturn.add(allergen);
                        Log.i(TAG, "allergen -> " + allergen.name);
                        break;
                    }
                }
            }
        }
        return resourcesToReturn;
    }

    /**
     * Parse ingredients analysis from tag
     *
     * @param resources input data
     * @param catalog   required catalog
     * @return list of ingredient analysis
     */
    public static List<IngredientsAnalysis.IngredientAnalysis> parseIngredientAnalysisFromTag(List<String> resources,
                                                                                              List<IngredientsAnalysis.IngredientAnalysis> catalog) {
        List<IngredientsAnalysis.IngredientAnalysis> resourcesToReturn = new ArrayList<>();
        if (resources != null && !resources.isEmpty()) {
            for (String code : resources) {
                for (IngredientsAnalysis.IngredientAnalysis ingredientsAnalysis : catalog) {
                    if (code.equals(ingredientsAnalysis.id)) {
                        resourcesToReturn.add(ingredientsAnalysis);
                        Log.i(TAG, "ingredientsAnalysis -> " + ingredientsAnalysis.name);
                        break;
                    }
                }
            }
        }

        return resourcesToReturn;
    }

    /**
     * Parse category from tag
     *
     * @param resources input data
     * @param catalog   required catalog
     * @return list of categories
     */
    public static List<Categories.Category> parseCategoryFromTag(List<String> resources, List<Categories.Category> catalog) {
        List<Categories.Category> resourcesToReturn = new ArrayList<>();
        if (resources != null && !resources.isEmpty()) {
            for (String code : resources) {
                for (Categories.Category category : catalog) {
                    if (code.equals(category.id)) {
                        resourcesToReturn.add(category);
                        Log.i(TAG, "category -> " + category.name);
                        break;
                    }
                }
            }
        }
        return resourcesToReturn;
    }

    /**
     * Parse nutrients levels from tag
     *
     * @param resources input data
     * @param catalog   required catalog
     * @return list of nutrients levels
     */
    public static List<NutrientsLevel.NutrientLevel> parseNutrientLevelFromTag(List<String> resources, NutrientsLevel catalog) {
        List<NutrientsLevel.NutrientLevel> resourcesToReturn = new ArrayList<>();
        if (resources != null && !resources.isEmpty()) {
            for (String code : resources) {
                String encoded = code.replace(":", " ").replace("-", " ");
                String[] encodedArray = encoded.split(" ");
                String finalCode = "";
                for (int i = 0; i < encodedArray.length; i++) {
                    encodedArray[i] = encodedArray[i].substring(0, 1).toUpperCase() + encodedArray[i].substring(1).toLowerCase();
                    finalCode = finalCode + encodedArray[i];
                }
                finalCode = "get" + finalCode;
                try {
                    Method method = catalog.getClass().getDeclaredMethod(finalCode);
                    NutrientsLevel.NutrientLevel nutrientLevel = (NutrientsLevel.NutrientLevel) method.invoke(catalog);
                    Log.i(TAG, "nutrientLevel -> " + nutrientLevel.code);
                    resourcesToReturn.add(nutrientLevel);
                } catch (SecurityException | NoSuchMethodException | IllegalAccessException | InvocationTargetException e) {
                    Log.e(TAG, e.getMessage());
                }
            }
        }
        return resourcesToReturn;
    }

    /**
     * Parse nova groups from tag
     *
     * @param resources input data
     * @param catalog   required catalog
     * @return list of nova groups
     */
    public static List<NovaGroups.NovaGroup> parseNovaGroupsFromTag(List<String> resources, NovaGroups catalog) {
        List<NovaGroups.NovaGroup> resourcesToReturn = new ArrayList<>();
        if (resources != null && !resources.isEmpty()) {
            for (String code : resources) {
                String encoded = code.replace(":", " ").replace("-", " ");
                String[] encodedArray = encoded.split(" ");
                String finalCode = "";
                for (int i = 0; i < encodedArray.length; i++) {
                    encodedArray[i] = encodedArray[i].substring(0, 1).toUpperCase() + encodedArray[i].substring(1).toLowerCase();
                    finalCode = finalCode + encodedArray[i];
                }
                finalCode = "get" + finalCode;
                try {
                    Method method = catalog.getClass().getDeclaredMethod(finalCode);
                    NovaGroups.NovaGroup novaGroup = (NovaGroups.NovaGroup) method.invoke(catalog);
                    Log.i(TAG, "novaGroup -> " + novaGroup.code);
                    resourcesToReturn.add(novaGroup);
                } catch (SecurityException | NoSuchMethodException | IllegalAccessException | InvocationTargetException e) {
                    Log.e(TAG, e.getMessage());
                }
            }
        }
        return resourcesToReturn;
    }

    /**
     * Parse additives classes from tag
     *
     * @param resources input data
     * @param catalog   required catalog
     * @return list of additives classes
     */
    public static List<AdditivesClasses.AdditiveClass> parseAdditivesClassesFromTag(List<String> resources, AdditivesClasses catalog) {
        List<AdditivesClasses.AdditiveClass> resourcesToReturn = new ArrayList<>();
        if (resources != null && !resources.isEmpty()) {
            for (String code : resources) {
                if (!"".equals(code)) {
                    String encoded = code.replace(":", " ").replace("-", " ");
                    String[] encodedArray = encoded.split(" ");
                    String finalCode = "";
                    for (int i = 0; i < encodedArray.length; i++) {
                        encodedArray[i] = encodedArray[i].substring(0, 1).toUpperCase() + encodedArray[i].substring(1).toLowerCase();
                        finalCode = finalCode + encodedArray[i];
                    }
                    finalCode = "get" + finalCode;
                    try {
                        Method method = catalog.getClass().getDeclaredMethod(finalCode);
                        AdditivesClasses.AdditiveClass additiveClass = (AdditivesClasses.AdditiveClass) method.invoke(catalog);
                        Log.i(TAG, "additive class -> " + additiveClass.code);
                        resourcesToReturn.add(additiveClass);
                    } catch (SecurityException | NoSuchMethodException | IllegalAccessException | InvocationTargetException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
        return resourcesToReturn;
    }

    /**
     * Parse additives from tag
     *
     * @param resources input data
     * @param catalog   required catalog
     * @param classes   required additives classes catalog
     * @return list of addities
     */
    public static List<Additives> parseAdditivesFromTag(List<String> resources, List<Additives> catalog,
                                                        AdditivesClasses classes) {
        List<Additives> resourcesToReturn = new ArrayList<>();
        if (resources != null && !resources.isEmpty()) {
            for (String code : resources) {
                for (Additives additive : catalog) {
                    if (code.equals(additive.code)) {
                        if (additive.additives_classes_tags != null && !additive.additives_classes_tags.isEmpty()) {
                            additive.additives_classes = parseAdditivesClassesFromTag(additive.additives_classes_tags, classes);
                            additive.additives_classes_tags = null;
                        }
                        resourcesToReturn.add(additive);
                        Log.i(TAG, "additive -> " + additive.name_es);
                        break;
                    }
                }
            }
        }
        return resourcesToReturn;
    }

    /**
     * Parse nutriments from data
     *
     * @param resource input data
     * @param ctx      context
     * @return list of labels
     */
    public static List<Nutriments> parseNutrimentsFromData(CompleteProductData.ProductData.Nutriments resource, Context ctx) {
        List<Nutriments> resourcesToReturn = new ArrayList<>();
        if (resource != null) {
            Nutriments nutriment = new Nutriments();
            nutriment.setCode("energy-kcal");
            nutriment.setUnit(
                    ProductInfoUtils.checkIfNullOrEmptyString(resource.energy_kcal_unit));
            nutriment.setValue100(resource.energy_kcal_100g);
            nutriment.setValueServing(resource.energy_kcal_serving);
            resourcesToReturn.add(nutriment);

            nutriment = new Nutriments();
            nutriment.setCode("fat");
            nutriment.setUnit(
                    ProductInfoUtils.checkIfNullOrEmptyString(resource.fat_unit));
            nutriment.setValue100(resource.fat_100g);
            nutriment.setValueServing(resource.fat_serving);
            resourcesToReturn.add(nutriment);

            nutriment = new Nutriments();
            nutriment.setCode("saturated-fat");
            nutriment.setUnit(
                    ProductInfoUtils.checkIfNullOrEmptyString(resource.saturated_fat_unit));
            nutriment.setValue100(resource.saturated_fat_100g);
            nutriment.setValueServing(resource.saturated_fat_serving);
            resourcesToReturn.add(nutriment);

            nutriment = new Nutriments();
            nutriment.setCode("carbohydrates");
            nutriment.setUnit(
                    ProductInfoUtils.checkIfNullOrEmptyString(resource.carbohydrates_unit));
            nutriment.setValue100(resource.carbohydrates_100g);
            nutriment.setValueServing(resource.carbohydrates_serving);
            resourcesToReturn.add(nutriment);

            nutriment = new Nutriments();
            nutriment.setCode("sugars");
            nutriment.setUnit(
                    ProductInfoUtils.checkIfNullOrEmptyString(resource.sugars_unit));
            nutriment.setValue100(resource.sugars_100g);
            nutriment.setValueServing(resource.sugars_serving);
            resourcesToReturn.add(nutriment);

            nutriment = new Nutriments();
            nutriment.setCode("sodium");
            nutriment.setUnit(
                    ProductInfoUtils.checkIfNullOrEmptyString(resource.sodium_unit));
            nutriment.setValue100(resource.sodium_100g);
            nutriment.setValueServing(resource.sodium_serving);
            resourcesToReturn.add(nutriment);

            nutriment = new Nutriments();
            nutriment.setCode("salt");
            nutriment.setUnit(
                    ProductInfoUtils.checkIfNullOrEmptyString(resource.salt_unit));
            nutriment.setValue100(resource.salt_100g);
            nutriment.setValueServing(resource.salt_serving);
            resourcesToReturn.add(nutriment);

            nutriment = new Nutriments();
            nutriment.setCode("proteins");
            nutriment.setUnit(
                    ProductInfoUtils.checkIfNullOrEmptyString(resource.proteins_unit));
            nutriment.setValue100(resource.proteins_100g);
            nutriment.setValueServing(resource.proteins_serving);
            resourcesToReturn.add(nutriment);
        }
        return resourcesToReturn;
    }

    /**
     * Retrieve data from database
     *
     * @param welcomeActivity activity
     */
    public static void retrieveDataFromLocalDatabase(WelcomeActivity welcomeActivity) {
        // Allergens
        if (welcomeActivity.dataHolder.getAllergens().get(welcomeActivity.dataHolder.getCurrentLocale().getLanguage()) == null ||
                welcomeActivity.dataHolder.getAllergens().get(welcomeActivity.dataHolder.getCurrentLocale().getLanguage()).isEmpty()) {
            welcomeActivity.dataHolder.getAllergens().put(ES,
                    DatabaseCatalogUtils.retrieveAllergensFromLocalDatabase(welcomeActivity.allergensDataBaseAdapter, ES));
            welcomeActivity.dataHolder.getAllergens().put(EN,
                    DatabaseCatalogUtils.retrieveAllergensFromLocalDatabase(welcomeActivity.allergensDataBaseAdapter, EN));
        }
        //Categories
        if (welcomeActivity.dataHolder.getCategories().get(welcomeActivity.dataHolder.getCurrentLocale().getLanguage()) == null ||
                welcomeActivity.dataHolder.getCategories().get(welcomeActivity.dataHolder.getCurrentLocale().getLanguage()).isEmpty()) {
            welcomeActivity.dataHolder.getCategories().put(ES,
                    DatabaseCatalogUtils.retrieveCategoriesFromLocalDatabase(welcomeActivity.categoriesDataBaseAdapter, ES));
            welcomeActivity.dataHolder.getCategories().put(EN,
                    DatabaseCatalogUtils.retrieveCategoriesFromLocalDatabase(welcomeActivity.categoriesDataBaseAdapter, EN));
        }
        // Labels
        if (welcomeActivity.dataHolder.getLabels().get(welcomeActivity.dataHolder.getCurrentLocale().getLanguage()) == null ||
                welcomeActivity.dataHolder.getLabels().get(welcomeActivity.dataHolder.getCurrentLocale().getLanguage()).isEmpty()) {
            welcomeActivity.dataHolder.getLabels().put(ES,
                    DatabaseCatalogUtils.retrieveLabelsFromLocalDatabase(welcomeActivity.labelsDataBaseAdapter, ES));
            welcomeActivity.dataHolder.getLabels().put(EN,
                    DatabaseCatalogUtils.retrieveLabelsFromLocalDatabase(welcomeActivity.labelsDataBaseAdapter, EN));
        }
        // Ingredients analysis
        if (welcomeActivity.dataHolder.getIngredientsAnalysis().get(welcomeActivity.dataHolder.getCurrentLocale().getLanguage()) == null ||
                welcomeActivity.dataHolder.getIngredientsAnalysis().get(welcomeActivity.dataHolder.getCurrentLocale().getLanguage()).isEmpty()) {
            welcomeActivity.dataHolder.getIngredientsAnalysis().put(ES,
                    DatabaseCatalogUtils.retrieveIngredientsAnalysisFromLocalDatabase(welcomeActivity.ingredientAnalysisDataBaseAdapter, ES));
            welcomeActivity.dataHolder.getIngredientsAnalysis().put(EN,
                    DatabaseCatalogUtils.retrieveIngredientsAnalysisFromLocalDatabase(welcomeActivity.ingredientAnalysisDataBaseAdapter, EN));
        }
        // Additives
        if (welcomeActivity.dataHolder.getAdditivesClasses() == null) {
            welcomeActivity.dataHolder.setAdditivesClasses(
                    DatabaseCatalogUtils.retrieveAdditivesClassesFromLocalDatabase(welcomeActivity.additivesClassesDataBaseAdapter));
        }
        // Nova groups
        if (welcomeActivity.dataHolder.getNovaGroups() == null) {
            welcomeActivity.dataHolder.setNovaGroups(
                    DatabaseCatalogUtils.retrieveNovaGroupsFromLocalDatabase(welcomeActivity.novaGroupsDataBaseAdapter));
        }
        // Nutrients Levels
        if (welcomeActivity.dataHolder.getNutrientLevels() == null) {
            welcomeActivity.dataHolder.setNutrientLevels(
                    DatabaseCatalogUtils.retrieveNutrientsLevelsFromLocalDatabase(welcomeActivity.nutrientsLevelsDataBaseAdapter));
        }
        // Additives
        if (welcomeActivity.dataHolder.getAdditives() == null ||
                welcomeActivity.dataHolder.getAdditives().isEmpty()) {
            welcomeActivity.dataHolder.setAdditives(
                    DatabaseCatalogUtils.retrieveAdditivesFromLocalDatabase(welcomeActivity.additivesDataBaseAdapter));
        }
    }

    /**
     * Add product to favorites
     *
     * @param product          selected product
     * @param context          context
     * @param welcomeActivity  activity
     * @param relativeLayout   layout where the view is displayed
     * @param textView         textview for getting its size
     * @param searchesFragment fragment
     */
    public static void addProductToFavorites(Product product, Context context, WelcomeActivity welcomeActivity,
                                             final RelativeLayout relativeLayout, TextView textView, SearchesFragment searchesFragment) {
        final ProgressDialog progressDialog =
                ProgressDialogUtils.createProgressDialog(context, R.string.add_favorite_search_loading_message);
        progressDialog.show();

        Search search = null;
        for (Search srch : welcomeActivity.dataHolder.getSearches()) {
            if (srch.getProductCode().equals(product.getSummaryData().getCode())) {
                search = srch;
                break;
            }
        }
        // Create favorite
        Favorite favorite = new Favorite();
        favorite.setProductCode(product.getSummaryData().getCode());
        favorite.setProductName_en(product.getSummaryData().getProductName_en());
        favorite.setProductName_en(product.getSummaryData().getProductName_es());
        favorite.setNew(search.isNew());

        Date date = new Date();
        Calendar c = Calendar.getInstance();
        String periodicity = welcomeActivity.dataHolder.getProfile().getPeriodicity();
        switch (periodicity) {
            case PERIODICITY_D:
                c.add(Calendar.DATE, 1);
                date = new Date(c.getTimeInMillis());
                break;
            case PERIODICITY_W:
                c.add(Calendar.DATE, 7);
                date = new Date(c.getTimeInMillis());
                break;
            case PERIODICITY_N:
                date = null;
                break;
        }
        favorite.setNextDateToCheckProductInfo(date);
        // Add product to favorites
        welcomeActivity.dbReference.child(welcomeActivity.dataHolder.getUser().getUuid()).child("/favorites")
                .child(favorite.getProductCode()).setValue(favorite,
                new DatabaseReference.CompletionListener() {
                    @Override
                    public void onComplete(DatabaseError databaseError,
                                           DatabaseReference databaseReference) {
                        if (databaseError == null) {

                            boolean found = false;
                            for (Favorite fav : welcomeActivity.dataHolder.getFavorites()) {
                                if (favorite.getProductCode().equals(fav.getProductCode())) {
                                    found = true;
                                    break;
                                }
                            }
                            // Add favorite to list if it's a new one
                            if (!found) {
                                welcomeActivity.dataHolder.getFavorites().add(favorite);
                                Collections.sort(welcomeActivity.dataHolder.getFavorites());
                            }
                            // If it was called form searches screen, update list
                            if (searchesFragment != null) {
                                searchesFragment.adapter = new SearchesRecyclerAdapter(welcomeActivity.dataHolder.getSearches(),
                                        welcomeActivity.dataHolder.getProducts(), welcomeActivity, searchesFragment,
                                        welcomeActivity.dataHolder.getFavorites());
                                searchesFragment.recyclerView.setAdapter(searchesFragment.adapter);
                                searchesFragment.adapter.notifyDataSetChanged();

                                searchesFragment.evaluateViewToDisplay();
                            }
                            // Show snackbar
                            SnackBarUtils
                                    .showSnackBar(context, relativeLayout,
                                            welcomeActivity.getString(R.string.product_add_to_favorites_message),
                                            (int) textView.getTextSize(), false);
                        } else {
                            Log.e(TAG, "Database error: " + databaseError.toException());
                            SnackBarUtils
                                    .showSnackBar(context, relativeLayout,
                                            welcomeActivity.getString(R.string.product_add_to_Favorites_error),
                                            (int) textView.getTextSize(), true);
                        }
                        progressDialog.dismiss();
                    }
                });
    }

    /**
     * Delete product from favorites
     *
     * @param barcode         barcode of product
     * @param context         context
     * @param welcomeActivity activity
     * @param relativeLayout  layout where it is displayed
     * @param textView        textview for getting its size
     * @param linearLayout    layout where it is displayed
     * @param fragment        fragment
     */
    public static void deleteProductFromFavorites(String barcode, Context context, WelcomeActivity welcomeActivity,
                                                  final RelativeLayout relativeLayout, TextView textView,
                                                  LinearLayout linearLayout, Fragment fragment) {
        final ProgressDialog progressDialog =
                ProgressDialogUtils.createProgressDialog(context, R.string.remove_favorite_loading_message);
        progressDialog.show();

        // Get favorite data and remove it
        welcomeActivity.dbReference = welcomeActivity.database.getReference("/users");
        welcomeActivity.dbReference.child(welcomeActivity.dataHolder.getUser().getUuid())
                .child("favorites").child(barcode).addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    child.getRef().removeValue();
                }
                dataSnapshot.getRef().removeValue();
                // Remove favorite from list
                for (Favorite favorite : welcomeActivity.dataHolder.getFavorites()) {
                    if (barcode.equals(favorite.getProductCode())) {
                        welcomeActivity.dataHolder.getFavorites().remove(favorite);
                        break;
                    }
                }
                // Change scrren bacground if list is empty
                if (linearLayout != null) {
                    if (welcomeActivity.dataHolder.getFavorites().isEmpty()) {
                        linearLayout.setBackgroundColor(welcomeActivity.getColor(R.color.backgroundColorWindow));
                    }
                }
                // If it was called form favorites screen, update list
                if (fragment != null) {
                    if (fragment instanceof FavoritesFragment) {
                        FavoritesFragment favoritesFragment = (FavoritesFragment) fragment;
                        favoritesFragment.adapter = new FavoritesRecyclerAdapter(welcomeActivity.dataHolder.getFavorites(),
                                welcomeActivity.dataHolder.getProducts(), welcomeActivity, favoritesFragment);
                        favoritesFragment.recyclerView.setAdapter(favoritesFragment.adapter);
                        favoritesFragment.adapter.notifyDataSetChanged();

                        favoritesFragment.evaluateViewToDisplay();
                    } else {
                        SearchesFragment searchesFragment = (SearchesFragment) fragment;
                        searchesFragment.adapter = new SearchesRecyclerAdapter(welcomeActivity.dataHolder.getSearches(),
                                welcomeActivity.dataHolder.getProducts(), welcomeActivity, searchesFragment,
                                welcomeActivity.dataHolder.getFavorites());
                        searchesFragment.recyclerView.setAdapter(searchesFragment.adapter);
                        searchesFragment.adapter.notifyDataSetChanged();

                        searchesFragment.evaluateViewToDisplay();
                    }
                }
                // Show snackbar
                SnackBarUtils
                        .showSnackBar(context, relativeLayout,
                                welcomeActivity.getString(R.string.favorite_deleted_message),
                                (int) textView.getTextSize(), false);

                progressDialog.dismiss();
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {
                Log.e(TAG, "Database error: " + databaseError.toException());
                SnackBarUtils
                        .showSnackBar(context, relativeLayout,
                                welcomeActivity.getString(R.string.favorite_removal_error),
                                (int) textView.getTextSize(), true);
                progressDialog.dismiss();
            }
        });
    }
}
