package edu.uoc.tfg.nutriknow.utils;

import android.content.Context;
import android.view.View;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.google.android.material.snackbar.Snackbar;
import com.joanzapata.iconify.IconDrawable;
import com.joanzapata.iconify.fonts.FontAwesomeIcons;

import edu.uoc.tfg.nutriknow.R;

/**
 * Utils class to managing snacbars
 */
public class SnackBarUtils {
    /**
     * Show snackbar
     *
     * @param ctx            context
     * @param relativeLayout layout where it is displayed
     * @param message        message to show
     * @param size           font size
     * @param isError        determine if the snackbar should show an error
     */
    public static void showSnackBar(Context ctx, RelativeLayout relativeLayout,
                                    String message, int size, boolean isError) {
        Snackbar snackbar = Snackbar
                .make(relativeLayout, message, Snackbar.LENGTH_LONG);
        View snackbarLayout = snackbar.getView();
        TextView textView = (TextView) snackbarLayout
                .findViewById(com.google.android.material.R.id.snackbar_text);
        // Set snackbar icon
        IconDrawable icon;
        if (isError) {
            icon = IconDrawableUtils.createIconDrawable(ctx, FontAwesomeIcons.fa_times, size,
                    R.color.textColorPrimary, true);
            snackbar.setBackgroundTint(ctx.getResources().getColor(R.color.backgroundColorError));
        } else {
            icon = IconDrawableUtils.createIconDrawable(ctx, FontAwesomeIcons.fa_check, size,
                    R.color.textColorPrimary, true);
        }
        // Allow to hide snackbar when tapping on it
        snackbarLayout.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                snackbar.dismiss();
            }
        });
        textView.setCompoundDrawablesWithIntrinsicBounds(null, null, icon, null);

        snackbar.show();
    }
}
