package edu.uoc.tfg.nutriknow.views.activities;

import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;

import androidx.annotation.NonNull;

import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.material.textfield.TextInputLayout;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.joanzapata.iconify.IconDrawable;
import com.joanzapata.iconify.fonts.FontAwesomeIcons;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.model.User;
import edu.uoc.tfg.nutriknow.utils.IconDrawableUtils;
import edu.uoc.tfg.nutriknow.utils.InputValidationUtils;
import edu.uoc.tfg.nutriknow.utils.KeyboardUtils;
import edu.uoc.tfg.nutriknow.utils.ProgressDialogUtils;
import edu.uoc.tfg.nutriknow.utils.SnackBarUtils;
import edu.uoc.tfg.nutriknow.views.CommonActivity;

/**
 * Register user activity class, extends from Common activity
 */
public class RegisterActivity extends CommonActivity {

    private static final String TAG = "RegisterActivity";

    private EditText userText;
    private TextInputLayout userInput;
    private EditText emailText;
    private TextInputLayout emailInput;
    private EditText passwordText;
    private TextInputLayout passwordInput;
    private EditText repeatPasswordText;
    private TextInputLayout repeatPasswordInput;
    private Button signupButton;

    private boolean validUsername = false;
    private boolean validPassword = false;
    private boolean validRepeatPassword = false;
    private boolean validEmail = false;
    private boolean passwordsMatch = false;

    private User user;

    /**
     * Create view method
     *
     * @param savedInstanceState bundle
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        view = R.layout.register_activity;
        postAuth = false;
        super.onCreate(savedInstanceState);
        // Configure toolbar
        toolbar.setTitle(getString(R.string.register_activity_label));
        // Reference views from layout
        userInput = (TextInputLayout) findViewById(R.id.input_user);
        userText = (EditText) findViewById(R.id.edit_user);
        // Add icon to input
        IconDrawable userIcon = IconDrawableUtils
                .createIconDrawable(getApplicationContext(), FontAwesomeIcons.fa_user,
                        (int) userText.getTextSize(), R.color.textColorPrimary, false);
        userText.setCompoundDrawablesWithIntrinsicBounds(null, null, userIcon, null);
        // Add content validation functionality
        userText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                validUsername = InputValidationUtils
                        .validateUserName(getApplicationContext(), userInput, String.valueOf(s));
            }

            @Override
            public void afterTextChanged(Editable s) {
                if (validate()) {
                    signupButton.setEnabled(true);
                } else {
                    signupButton.setEnabled(false);
                }
            }
        });

        emailInput = (TextInputLayout) findViewById(R.id.input_email);
        emailText = (EditText) findViewById(R.id.edit_email);
        // Add icon to input
        IconDrawable emailIcon = IconDrawableUtils
                .createIconDrawable(getApplicationContext(), FontAwesomeIcons.fa_envelope,
                        (int) userText.getTextSize(), R.color.textColorPrimary, false);
        emailText.setCompoundDrawablesWithIntrinsicBounds(null, null, emailIcon, null);
        // Add content validation functionality
        emailText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                validEmail = InputValidationUtils
                        .validateEmailAddress(getApplicationContext(), emailInput, String.valueOf(s));
            }

            @Override
            public void afterTextChanged(Editable s) {
                if (validate()) {
                    signupButton.setEnabled(true);
                } else {
                    signupButton.setEnabled(false);
                }
            }
        });
        passwordInput = (TextInputLayout) findViewById(R.id.input_password);
        passwordText = (EditText) findViewById(R.id.edit_password);
        // Add content validation functionality
        passwordText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                validPassword = InputValidationUtils
                        .validatePassword(getApplicationContext(), passwordInput, String.valueOf(s));
            }

            @Override
            public void afterTextChanged(Editable s) {
                if (validate()) {
                    String password = passwordText.getText().toString();
                    String repeatPassword = repeatPasswordText.getText().toString();
                    boolean passwordsMatch = InputValidationUtils
                            .validateRepeatPassword(getApplicationContext(), passwordInput, repeatPasswordInput, password, repeatPassword);
                    signupButton.setEnabled(passwordsMatch);
                } else {
                    signupButton.setEnabled(false);
                }
            }
        });
        repeatPasswordInput = (TextInputLayout) findViewById(R.id.input_repeat_password);
        repeatPasswordText = (EditText) findViewById(R.id.edit_repeat_password);
        // Add content validation functionality
        repeatPasswordText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                validRepeatPassword = InputValidationUtils
                        .validatePassword(getApplicationContext(), repeatPasswordInput, String.valueOf(s));
            }

            @Override
            public void afterTextChanged(Editable s) {
                if (validate()) {
                    String password = passwordText.getText().toString();
                    String repeatPassword = repeatPasswordText.getText().toString();
                    passwordsMatch = InputValidationUtils
                            .validateRepeatPassword(getApplicationContext(), passwordInput, repeatPasswordInput, password, repeatPassword);
                    signupButton.setEnabled(passwordsMatch);
                } else {
                    signupButton.setEnabled(false);
                }
            }
        });
        // When enter is pressed in the keyboard, submit or pass to next field
        repeatPasswordText.setOnKeyListener(new View.OnKeyListener() {
            public boolean onKey(View v, int keyCode, KeyEvent event) {
                if (validate() && passwordsMatch) {
                    if (event.getAction() == KeyEvent.ACTION_DOWN) {
                        switch (keyCode) {
                            case KeyEvent.KEYCODE_DPAD_CENTER:
                            case KeyEvent.KEYCODE_ENTER:
                                signUp();
                                return true;
                            default:
                                break;
                        }
                    }
                }
                return false;
            }
        });
        // Add signup functionality
        signupButton = (Button) findViewById(R.id.btn_signup);
        signupButton.setEnabled(false);
        signupButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                signUp();
            }
        });
    }

    /**
     * Perform user registration
     */
    private void signUp() {
        // Validate form content
        if (!validate()) {
            onSignupFailed();
            return;
        }
        // Hide keyboard and clear inputs
        KeyboardUtils.hideKeyBoard(getApplicationContext(), RegisterActivity.this);
        userText.clearFocus();
        emailText.clearFocus();
        passwordText.clearFocus();
        repeatPasswordText.clearFocus();

        signupButton.setEnabled(false);

        progressDialog =
                ProgressDialogUtils.createProgressDialog(this, R.string.register_signingup_message);
        progressDialog.show();

        String userName = userText.getText().toString();
        String email = emailText.getText().toString();
        String password = passwordText.getText().toString();

        user = new User(userName, email);
        user.setHasProfile(false);
        // Sign up in firebase
        firebaseAuth.createUserWithEmailAndPassword(email, password)
                .addOnSuccessListener(new OnSuccessListener<AuthResult>() {
                    @Override
                    public void onSuccess(@NonNull AuthResult result) {
                        // Create user in database
                        FirebaseUser currentUser = firebaseAuth.getCurrentUser();
                        user.setUuid(currentUser.getUid());
                        dbReference = database.getReference("/users");
                        dbReference.child(currentUser.getUid()).child("/user").setValue(user,
                                new DatabaseReference.CompletionListener() {
                                    @Override
                                    public void onComplete(DatabaseError databaseError,
                                                           DatabaseReference databaseReference) {
                                        if (databaseError == null) {
                                            // User has signed up successfully
                                            onSignupSuccess();
                                        } else {
                                            onSignupFailed();
                                        }
                                        progressDialog.dismiss();
                                    }
                                });
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception exception) {
                        // User cannot sign in and an exception is thrown
                        Log.e(TAG, exception.getMessage());
                        onSignupFailed();
                        progressDialog.dismiss();
                    }
                });
    }

    /**
     * If the process was successful, go back to login activity
     */
    private void onSignupSuccess() {
        signupButton.setEnabled(true);
        SnackBarUtils
                .showSnackBar(this, relativeLayout, getString(R.string.register_successful_message),
                        (int) userText.getTextSize(), false);
        new android.os.Handler().postDelayed(
                new Runnable() {
                    public void run() {
                        // Start the login activity
                        Intent intent = new Intent(getApplicationContext(), LoginActivity.class);
                        startActivity(intent);
                        overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);
                        finish();
                    }
                }, 1000);
    }

    /**
     * If the process failed, show snackbar
     */
    private void onSignupFailed() {
        SnackBarUtils
                .showSnackBar(this, relativeLayout, getString(R.string.register_failed_error),
                        (int) userText.getTextSize(), true);
        signupButton.setEnabled(true);
    }

    /**
     * Validate form values
     *
     * @return boolean
     */
    private boolean validate() {
        return validUsername && validPassword && validRepeatPassword && validEmail;
    }

}