package edu.uoc.tfg.nutriknow.views.activities;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;

import androidx.appcompat.app.AppCompatActivity;

import edu.uoc.tfg.nutriknow.R;

/**
 * Splash activity class
 */
public class SplashActivity extends AppCompatActivity {

    private final int SPLASH_DISPLAY_LENGTH = 5000;

    /**
     * Create view method
     *
     * @param savedInstanceState bundle
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.splash_activity);

        // Close splash screen after some seconds
        new Handler().postDelayed(new Runnable() {
            @Override
            public void run() {
                // Create an Intent that will start the login activity
                Intent mainIntent = new Intent(getApplicationContext(), LoginActivity.class);
                SplashActivity.this.startActivity(mainIntent);
                SplashActivity.this.finish();
            }
        }, SPLASH_DISPLAY_LENGTH);
    }

    /**
     * If the window has the focus, hide system bars
     *
     * @param hasFocus boolean
     */
    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);
        if (hasFocus) {
            hideSystemUIAndNavigation(this);
        }
    }

    /**
     * Hide system bars
     *
     * @param activity activity
     */
    private void hideSystemUIAndNavigation(Activity activity) {
        View decorView = activity.getWindow().getDecorView();
        // Set the content to appear under the system bars so that the
        // content doesn't resize when the system bars hide and show.
        decorView.setSystemUiVisibility(View.SYSTEM_UI_FLAG_IMMERSIVE);
    }
}