package edu.uoc.tfg.nutriknow.views.activities;

import android.app.ProgressDialog;
import android.content.Intent;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.os.Bundle;
import android.os.Handler;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.view.GravityCompat;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;

import com.google.android.material.bottomnavigation.BottomNavigationView;
import com.google.android.material.navigation.NavigationView;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;

import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.Locale;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.api.OpenFoodFactsClient;
import edu.uoc.tfg.nutriknow.api.interfaces.OpenFoodFactsService;
import edu.uoc.tfg.nutriknow.model.CompleteProductData;
import edu.uoc.tfg.nutriknow.model.Favorite;
import edu.uoc.tfg.nutriknow.model.Product;
import edu.uoc.tfg.nutriknow.model.Search;
import edu.uoc.tfg.nutriknow.sqlite.adapters.AdditivesClassesDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.AdditivesDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.AllergensDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.CategoriesDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.IngredientAnalysisDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.LabelsDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.NovaGroupsDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.NutrientsLevelsDataBaseAdapter;
import edu.uoc.tfg.nutriknow.utils.ProductInfoUtils;
import edu.uoc.tfg.nutriknow.utils.ProgressDialogUtils;
import edu.uoc.tfg.nutriknow.utils.SnackBarUtils;
import edu.uoc.tfg.nutriknow.views.CommonActivity;
import edu.uoc.tfg.nutriknow.views.EndDrawerToggle;
import edu.uoc.tfg.nutriknow.views.fragments.FavoritesFragment;
import edu.uoc.tfg.nutriknow.views.fragments.HomeFragment;
import edu.uoc.tfg.nutriknow.views.fragments.ProductInfoFragment;
import edu.uoc.tfg.nutriknow.views.fragments.ProfileSetupFragment;
import edu.uoc.tfg.nutriknow.views.fragments.ScanFragment;
import edu.uoc.tfg.nutriknow.views.fragments.SearchesFragment;
import edu.uoc.tfg.nutriknow.views.fragments.SettingsFragment;
import edu.uoc.tfg.nutriknow.views.fragments.UserAccountFragment;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * Welcome activity class, extends from Common activity and implements navigation view interface
 */
public class WelcomeActivity extends CommonActivity implements
        NavigationView.OnNavigationItemSelectedListener {

    private static final String TAG = "WelcomeActivity";
    private static final String PRODUCT_FOUND = "product found";
    private static final String ES = "es";
    private static final String EN = "en";
    private static final String PERIODICITY_D = "D";
    private static final String PERIODICITY_W = "W";
    private static final String PERIODICITY_N = "N";

    public TextView userText;
    public TextView emailText;

    public BottomNavigationView bottomNavigationView;

    private Product product;

    public AllergensDataBaseAdapter allergensDataBaseAdapter;
    public CategoriesDataBaseAdapter categoriesDataBaseAdapter;
    public LabelsDataBaseAdapter labelsDataBaseAdapter;
    public NovaGroupsDataBaseAdapter novaGroupsDataBaseAdapter;
    public NutrientsLevelsDataBaseAdapter nutrientsLevelsDataBaseAdapter;
    public IngredientAnalysisDataBaseAdapter ingredientAnalysisDataBaseAdapter;
    public AdditivesClassesDataBaseAdapter additivesClassesDataBaseAdapter;
    public AdditivesDataBaseAdapter additivesDataBaseAdapter;

    /**
     * Create view method
     *
     * @param savedInstanceState bundle
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        view = R.layout.welcome_activity;
        postAuth = true;
        backAllowed = false;
        super.onCreate(savedInstanceState);

        // Initialize database adapters
        allergensDataBaseAdapter = new AllergensDataBaseAdapter(getApplicationContext());
        categoriesDataBaseAdapter = new CategoriesDataBaseAdapter(getApplicationContext());
        labelsDataBaseAdapter = new LabelsDataBaseAdapter(getApplicationContext());
        novaGroupsDataBaseAdapter = new NovaGroupsDataBaseAdapter(getApplicationContext());
        nutrientsLevelsDataBaseAdapter = new NutrientsLevelsDataBaseAdapter(getApplicationContext());
        ingredientAnalysisDataBaseAdapter = new IngredientAnalysisDataBaseAdapter(getApplicationContext());
        additivesClassesDataBaseAdapter = new AdditivesClassesDataBaseAdapter(getApplicationContext());
        additivesDataBaseAdapter = new AdditivesDataBaseAdapter(getApplicationContext());

        // Configure toolbar
        toolbar.setNavigationIcon(R.mipmap.toolbar_logo);

        // Reference side drawer
        drawerLayout = findViewById(R.id.drawer_layout);
        drawerToggle = new EndDrawerToggle(
                this, drawerLayout, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawerLayout.addDrawerListener(drawerToggle);

        // Reference navigation view in the drawer
        navigationView = findViewById(R.id.navigation_view);
        // Add drawer functionality
        navigationView.setNavigationItemSelectedListener(this);
        navigationView.getMenu().getItem(0).setVisible(false);

        headerView = navigationView.getHeaderView(0);
        headerView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (drawerLayout.isDrawerOpen(GravityCompat.END)) {
                    drawerLayout.closeDrawer(GravityCompat.END);
                }
            }
        });
        // Reference views from layout
        userText = (TextView) headerView.findViewById(R.id.user_text);
        userText.setText(dataHolder.getUser().getUserName());

        emailText = (TextView) headerView.findViewById(R.id.email_text);
        emailText.setText(dataHolder.getUser().getEmail());

        // Add bottom bar functionality
        bottomNavigationView = findViewById(R.id.bottom_bar);
        bottomNavigationView.setOnNavigationItemSelectedListener(new BottomNavigationView.OnNavigationItemSelectedListener() {
            @Override
            public boolean onNavigationItemSelected(@NonNull MenuItem menuItem) {
                int title = 0;
                Fragment fragment = null;
                if (!menuItem.isChecked()) {
                    switch (menuItem.getItemId()) {
                        // Bottom bar options
                        case R.id.scan:
                            title = R.string.scan_activity_label;
                            fragment = ScanFragment.newInstance(getString(title));
                            navigationView.setCheckedItem(R.id.nav_item0);
                            break;
                        case R.id.searches:
                            title = R.string.menu_my_searches;
                            fragment = SearchesFragment.newInstance(getString(title));
                            navigationView.setCheckedItem(R.id.nav_my_searches);
                            break;
                        case R.id.favorites:
                            title = R.string.favorites_activity_label;
                            fragment = FavoritesFragment.newInstance(getString(title));
                            navigationView.setCheckedItem(R.id.nav_my_favorites);
                            break;
                    }
                    if (fragment != null) {
                        FragmentManager fragmentManager = getSupportFragmentManager();
                        fragmentManager.beginTransaction().replace(R.id.fragment_layout, fragment).commit();
                        setTitle(getString(title));
                    }
                }
                return true;
            }
        });
        bottomNavigationView.getMenu().getItem(0).setVisible(false);

        // Get selected locale
        Locale currentLocale = getResources().getConfiguration().locale;
        dataHolder.setCurrentLocale(currentLocale);

        // Check if user has profile
        if (dataHolder.getUser().isHasProfile()) {
            MenuItem menuItem = navigationView.getMenu().getItem(1);
            onNavigationItemSelected(menuItem);
            menuItem.setChecked(true);
            // Use the stored locale in the user profile
            if (dataHolder.getProfile().getSelectedLanguage() != null &&
                    !dataHolder.getProfile().getSelectedLanguage().isEmpty()) {
                Resources resources = getResources();
                DisplayMetrics dm = resources.getDisplayMetrics();
                Configuration config = resources.getConfiguration();
                Locale locale = new Locale(dataHolder.getProfile().getSelectedLanguage().toLowerCase());
                config.setLocale(locale);
                resources.updateConfiguration(config, dm);
                dataHolder.setCurrentLocale(locale);
            }
        } else {
            // If not, redirect to profile setup fragment
            Fragment fragment = ProfileSetupFragment.newInstance(getString(R.string.profile_setup_activity_label));
            FragmentManager fragmentManager = getSupportFragmentManager();
            fragmentManager.beginTransaction().replace(R.id.fragment_layout, fragment).commit();
            setTitle(getString(R.string.profile_setup_activity_label));
        }
    }

    /**
     * On post create
     *
     * @param savedInstanceState bundle
     */
    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        // Sync drawer
        drawerToggle.syncState();
    }

    /**
     * on back pressed
     */
    @Override
    public void onBackPressed() {
        // Close drawer if open
        if (drawerLayout.isDrawerOpen(GravityCompat.END)) {
            drawerLayout.closeDrawer(GravityCompat.END);
        }
    }

    /**
     * Functionality for drawer options
     *
     * @param menuItem selected menu item
     * @return boolean
     */
    @Override
    public boolean onNavigationItemSelected(@NonNull MenuItem menuItem) {
        int title = 0;
        Fragment fragment = null;
        if (!menuItem.isChecked()) {
            bottomNavigationView.setSelectedItemId(R.id.item0);
            switch (menuItem.getItemId()) {
                // Drawer options
                case R.id.nav_home:
                    title = R.string.menu_home;
                    fragment = HomeFragment.newInstance(getString(title));
                    break;
                case R.id.nav_my_account:
                    title = R.string.menu_my_account;
                    fragment = UserAccountFragment.newInstance(getString(title));
                    break;
                case R.id.nav_my_searches:
                    title = R.string.menu_my_searches;
                    fragment = SearchesFragment.newInstance(getString(title));
                    bottomNavigationView.getMenu().getItem(2).setChecked(true);
                    break;
                case R.id.nav_my_favorites:
                    title = R.string.favorites_activity_label;
                    fragment = FavoritesFragment.newInstance(getString(title));
                    bottomNavigationView.getMenu().getItem(3).setChecked(true);
                    break;
                case R.id.nav_settings:
                    title = R.string.menu_settings;
                    fragment = SettingsFragment.newInstance(getString(title));
                    break;
                case R.id.nav_log_out:
                    title = R.string.menu_log_out;
                    logout();
                    break;
            }
            if (fragment != null) {
                FragmentManager fragmentManager = getSupportFragmentManager();
                fragmentManager.beginTransaction().replace(R.id.fragment_layout, fragment).commit();
                setTitle(getString(title));
            }
        }
        drawerLayout.closeDrawer(GravityCompat.END);
        return true;
    }

    /**
     * Method for preparing the toolbar for fragments
     */
    public void prepareToolbarForFragments() {
        toolbar.setNavigationIcon(defaultNavigationIcon);
        toolbar.getNavigationIcon().setTint(getColor(R.color.textColorPrimary));
        toolbar.setNavigationOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                toolbar.getNavigationIcon().setTint(getColor(R.color.textColorSecondary));
                WelcomeActivity.this.reloadActivity();
            }
        });
    }

    /**
     * Reload welcome activity
     */
    public void reloadActivity() {
        Intent intent = new Intent(getApplicationContext(), WelcomeActivity.class);
        startActivity(intent);
        overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);
        finish();
    }

    /**
     * Retrieve product information
     *
     * @param barcodeData    product barcode
     * @param progressDialog progress dialog
     */
    public void retrieveProductInformation(String barcodeData, ProgressDialog progressDialog) {
        if (progressDialog == null) {
            progressDialog =
                    ProgressDialogUtils.createProgressDialog(this, R.string.product_info_loading_message);
            progressDialog.show();
        }
        super.progressDialog = progressDialog;
        // Get open food facts endpoint
        OpenFoodFactsService apiService = OpenFoodFactsClient.getEsClient().create(OpenFoodFactsService.class);
        retrieveProductInformationFromApi(barcodeData, apiService, progressDialog);
    }

    /**
     * Retrieve product information from api by barcode
     *
     * @param barcodeData    product barcode
     * @param apiService     api client
     * @param progressDialog progress dialog
     */
    private void retrieveProductInformationFromApi(String barcodeData, OpenFoodFactsService apiService, ProgressDialog progressDialog) {
        Call<CompleteProductData> call = apiService.getProductInformation(barcodeData);
        call.enqueue(new Callback<CompleteProductData>() {
            @Override
            public void onResponse(Call<CompleteProductData> call, Response<CompleteProductData> response) {
                // Get service response
                CompleteProductData resource = response.body();
                // If product was found
                if (PRODUCT_FOUND.equals(resource.status_verbose)) {
                    product = new Product();
                    // Parse product data using Product object and methods in ProductInfoUtils
                    Product.SummaryData summaryData = new Product.SummaryData();
                    summaryData.setCode(resource.code);
                    if (resource.product.product_name_es != null &&
                            !resource.product.product_name_es.isEmpty()) {
                        summaryData.setProductName_es(resource.product.product_name_es);
                    } else {
                        summaryData.setProductName_es(resource.product.product_name);
                    }
                    summaryData.setProductName_en(
                            ProductInfoUtils.checkIfNullOrEmptyString(resource.product.product_name));
                    summaryData.setBrands(
                            ProductInfoUtils.convertListToCapitalLetter(resource.product.brands_tags));
                    summaryData.setStores(
                            ProductInfoUtils.convertStringToList(resource.product.stores));
                    summaryData.setLabels_es(
                            ProductInfoUtils.parseLabelFromTag(resource.product.labels_tags,
                                    dataHolder.getLabels().get(ES)));
                    summaryData.setLabels_en(
                            ProductInfoUtils.parseLabelFromTag(resource.product.labels_tags,
                                    dataHolder.getLabels().get(EN)));
                    summaryData.setExpirationDate(
                            ProductInfoUtils.convertStringToDate(
                                    ProductInfoUtils.checkIfNullOrEmptyString(resource.product.expiration_date)));
                    summaryData.setAllergens_es(
                            ProductInfoUtils.parseAllergenFromTag(resource.product.allergens_tags,
                                    dataHolder.getProfile().getAllergens_es()));
                    summaryData.setAllergens_en(
                            ProductInfoUtils.parseAllergenFromTag(resource.product.allergens_tags,
                                    dataHolder.getProfile().getAllergens_en()));
                    summaryData.setTraces_es(
                            ProductInfoUtils.parseAllergenFromTag(resource.product.traces_tags,
                                    dataHolder.getProfile().getAllergens_es()));
                    summaryData.setTraces_en(
                            ProductInfoUtils.parseAllergenFromTag(resource.product.traces_tags,
                                    dataHolder.getProfile().getAllergens_en()));
                    summaryData.setCountries(
                            ProductInfoUtils.convertStringToList(resource.product.countries));
                    summaryData.setCompleteness(resource.product.completeness);
                    summaryData.setIngredientsAnalysis_es(
                            ProductInfoUtils.parseIngredientAnalysisFromTag(resource.product.ingredients_analysis_tags,
                                    dataHolder.getIngredientsAnalysis().get(ES)));
                    summaryData.setIngredientsAnalysis_en(
                            ProductInfoUtils.parseIngredientAnalysisFromTag(resource.product.ingredients_analysis_tags,
                                    dataHolder.getIngredientsAnalysis().get(EN)));

                    product.setSummaryData(summaryData);

                    Product.IngredientsData ingredientsData = new Product.IngredientsData();

                    if (resource.product.ingredients_text_es != null &&
                            !resource.product.ingredients_text_es.isEmpty()) {
                        ingredientsData.setIngredients_es(
                                ProductInfoUtils.convertStringToCapitalLetter(
                                        ProductInfoUtils.checkIfNullOrEmptyString(resource.product.ingredients_text_es)));
                    } else {
                        ingredientsData.setIngredients_es(
                                ProductInfoUtils.convertStringToCapitalLetter(
                                        ProductInfoUtils.checkIfNullOrEmptyString(resource.product.ingredients_text)));
                    }
                    if (resource.product.ingredients_text_en != null &&
                            !resource.product.ingredients_text_en.isEmpty()) {
                        ingredientsData.setIngredients_en(
                                ProductInfoUtils.convertStringToCapitalLetter(
                                        ProductInfoUtils.checkIfNullOrEmptyString(resource.product.ingredients_text_en)));
                    } else {
                        ingredientsData.setIngredients_en(
                                ProductInfoUtils.convertStringToCapitalLetter(
                                        ProductInfoUtils.checkIfNullOrEmptyString(resource.product.ingredients_text)));
                    }
                    ingredientsData.setIngredients_count(
                            ProductInfoUtils.checkIfNullOrEmptyInteger(resource.product.ingredients_n));

                    ingredientsData.setCategories_es(
                            ProductInfoUtils.parseCategoryFromTag(resource.product.categories_hierarchy,
                                    dataHolder.getCategories().get(ES)));
                    ingredientsData.setCategories_en(
                            ProductInfoUtils.parseCategoryFromTag(resource.product.categories_hierarchy,
                                    dataHolder.getCategories().get(EN)));
                    ingredientsData.setAllergens_es(
                            ProductInfoUtils.parseAllergenFromTag(resource.product.allergens_tags,
                                    dataHolder.getAllergens().get(ES)));
                    ingredientsData.setAllergens_en(
                            ProductInfoUtils.parseAllergenFromTag(resource.product.allergens_tags,
                                    dataHolder.getAllergens().get(EN)));
                    ingredientsData.setTraces_es(
                            ProductInfoUtils.parseAllergenFromTag(resource.product.traces_tags,
                                    dataHolder.getAllergens().get(ES)));
                    ingredientsData.setTraces_en(
                            ProductInfoUtils.parseAllergenFromTag(resource.product.traces_tags,
                                    dataHolder.getAllergens().get(EN)));
                    ingredientsData.setAdditivesCount(
                            ProductInfoUtils.checkIfNullOrEmptyInteger(resource.product.additives_n));
                    ingredientsData.setAdditives(
                            ProductInfoUtils.parseAdditivesFromTag(resource.product.additives_tags,
                                    dataHolder.getAdditives(), dataHolder.getAdditivesClasses()));
                    ingredientsData.setNovaGroup(
                            ProductInfoUtils.checkIfNullOrEmptyInteger(resource.product.nova_group));
                    ingredientsData.setNovaGroups(
                            ProductInfoUtils.parseNovaGroupsFromTag(resource.product.nova_groups_tags,
                                    dataHolder.getNovaGroups()));

                    product.setIngredientsData(ingredientsData);

                    Product.NutritionalData nutritionalData = new Product.NutritionalData();
                    nutritionalData.setNutrientLevels(
                            ProductInfoUtils.parseNutrientLevelFromTag(resource.product.nutrient_levels_tags,
                                    dataHolder.getNutrientLevels()));
                    nutritionalData.setServingSize(
                            ProductInfoUtils.checkIfNullOrEmptyString(resource.product.serving_size));
                    nutritionalData.setNutriments(
                            ProductInfoUtils.parseNutrimentsFromData(resource.product.nutriments, getApplicationContext()));
                    nutritionalData.setNutritionGrade(
                            ProductInfoUtils.checkIfNullOrEmptyString(resource.product.nutrition_grades));
                    nutritionalData.setNutritionScore(
                            ProductInfoUtils.checkIfNullOrEmptyInteger(resource.product.nutriscore_score));

                    product.setNutritionalData(nutritionalData);
                    // Save product in database
                    dbReference.child(dataHolder.getUser().getUuid())
                            .child("products").child(resource.code).setValue(product,
                            new DatabaseReference.CompletionListener() {
                                @Override
                                public void onComplete(DatabaseError databaseError,
                                                       DatabaseReference databaseReference) {
                                    if (databaseError == null) {
                                        boolean isNewProduct = false;
                                        boolean found = false;
                                        for (Product prod : dataHolder.getProducts()) {
                                            if (product.getSummaryData().getCode().equals(prod.getSummaryData().getCode())) {
                                                found = true;
                                                break;
                                            }
                                        }
                                        if (!found) {
                                            isNewProduct = true;
                                            dataHolder.getProducts().add(product);
                                            Collections.sort(dataHolder.getProducts());
                                        }
                                        // Save a search item related with the product
                                        Search search = new Search();
                                        search.setProductCode(product.getSummaryData().getCode());
                                        search.setProductName_es(product.getSummaryData().getProductName_es());
                                        search.setProductName_en(product.getSummaryData().getProductName_en());
                                        search.setNew(isNewProduct);
                                        boolean finalIsNewProduct = isNewProduct;
                                        dbReference.child(dataHolder.getUser().getUuid())
                                                .child("searches").child(search.getProductCode()).setValue(search,
                                                new DatabaseReference.CompletionListener() {
                                                    @Override
                                                    public void onComplete(DatabaseError databaseError,
                                                                           DatabaseReference databaseReference) {
                                                        if (databaseError == null) {

                                                            boolean found = false;
                                                            for (Search srch : dataHolder.getSearches()) {
                                                                if (search.getProductCode().equals(srch.getProductCode())) {
                                                                    found = true;
                                                                    break;
                                                                }
                                                            }
                                                            if (!found) {
                                                                dataHolder.getSearches().add(search);
                                                                Collections.sort(dataHolder.getSearches());
                                                            } else {
                                                                found = false;
                                                                // Check if exists a favorite row related to this product
                                                                for (Favorite fav : dataHolder.getFavorites()) {
                                                                    if (search.getProductCode().equals(fav.getProductCode())) {
                                                                        found = true;
                                                                        break;
                                                                    }
                                                                }
                                                                // If does so, update the next date for looking for changes
                                                                if (found) {
                                                                    Date date = new Date();
                                                                    Calendar c = Calendar.getInstance();
                                                                    String periodicity = dataHolder.getProfile().getPeriodicity();
                                                                    switch (periodicity) {
                                                                        case PERIODICITY_D:
                                                                            c.add(Calendar.DATE, 1);
                                                                            date = new Date(c.getTimeInMillis());
                                                                            break;
                                                                        case PERIODICITY_W:
                                                                            c.add(Calendar.DATE, 7);
                                                                            date = new Date(c.getTimeInMillis());
                                                                            break;
                                                                        case PERIODICITY_N:
                                                                            date = null;
                                                                            break;
                                                                    }
                                                                    Favorite favorite = new Favorite();
                                                                    favorite.setProductCode(search.getProductCode());
                                                                    favorite.setProductName_es(search.getProductName_es());
                                                                    favorite.setProductName_en(search.getProductName_en());
                                                                    favorite.setNew(finalIsNewProduct);
                                                                    favorite.setNextDateToCheckProductInfo(date);
                                                                    // Store favorite in database
                                                                    dbReference.child(dataHolder.getUser().getUuid()).child("/favorites")
                                                                            .child(favorite.getProductCode()).setValue(favorite,
                                                                            new DatabaseReference.CompletionListener() {
                                                                                @Override
                                                                                public void onComplete(DatabaseError databaseError,
                                                                                                       DatabaseReference databaseReference) {
                                                                                    if (databaseError == null) {

                                                                                    } else {
                                                                                        Log.e(TAG, "Database error: " + databaseError.toException());
                                                                                        onDatabaseError(progressDialog);
                                                                                    }
                                                                                }
                                                                            });
                                                                }
                                                            }
                                                            // Establish time to wait to redirect
                                                            int timeToWait = 0;
                                                            if (resource.product.additives_tags.size() >= 5) {
                                                                timeToWait = 6500;
                                                            } else if (resource.product.additives_tags.size() > 0) {
                                                                timeToWait = 2500;
                                                            } else {
                                                                timeToWait = 1500;
                                                            }
                                                            new Handler().postDelayed(new Runnable() {
                                                                public void run() {
                                                                    progressDialog.dismiss();
                                                                }
                                                            }, timeToWait);
                                                            if (WelcomeActivity.this != null) {
                                                                WelcomeActivity.this.runOnUiThread(new Runnable() {
                                                                    @Override
                                                                    public void run() {
                                                                        goToProductInfoScreen();
                                                                    }
                                                                });
                                                            }
                                                        } else {
                                                            Log.e(TAG, "Database error: " + databaseError.toException());
                                                            onDatabaseError(progressDialog);
                                                        }
                                                    }
                                                });
                                    } else {
                                        Log.e(TAG, "Database error: " + databaseError.toException());
                                        onDatabaseError(progressDialog);
                                    }
                                }
                            });

                } else {

                    relativeLayout = (RelativeLayout) callingFragment.getActivity()
                            .findViewById(R.id.mainLayout);
                    progressDialog.dismiss();
                    SnackBarUtils
                            .showSnackBar(getApplicationContext(), relativeLayout,
                                    getString(R.string.product_not_found_openfacts_message),
                                    (int) userText.getTextSize(), true);
                    if (WelcomeActivity.this != null) {
                        WelcomeActivity.this.runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                // reload fragment to be able to show scanner again
                                reloadFragment();
                            }
                        });
                    }
                }
            }

            @Override
            public void onFailure(Call<CompleteProductData> call, Throwable t) {
                call.cancel();
                Log.e(TAG, t.getMessage() + "");

                relativeLayout = (RelativeLayout) callingFragment.getActivity()
                        .findViewById(R.id.mainLayout);
                progressDialog.dismiss();
                SnackBarUtils
                        .showSnackBar(getApplicationContext(), relativeLayout,
                                getString(R.string.openfacts_get_product_info_error),
                                (int) userText.getTextSize(), true);
            }
        });
    }

    /**
     * On database error, show snackbar
     *
     * @param progressDialog progress dialog
     */
    private void onDatabaseError(ProgressDialog progressDialog) {

        relativeLayout = (RelativeLayout) callingFragment.getActivity()
                .findViewById(R.id.mainLayout);
        SnackBarUtils
                .showSnackBar(getApplicationContext(), relativeLayout,
                        getString(R.string.openfacts_get_product_info_error),
                        (int) userText.getTextSize(), true);
        progressDialog.dismiss();
    }

    /**
     * Redirect to product info fragment
     */
    private void goToProductInfoScreen() {
        navigationView.setCheckedItem(R.id.nav_item0);
        bottomNavigationView.getMenu().getItem(0).setChecked(true);

        Fragment fragment = ProductInfoFragment.newInstance(getString(R.string.product_info_activity_label), product);
        getSupportFragmentManager().beginTransaction().replace(R.id.fragment_layout, fragment).commit();
        setTitle(getString(R.string.product_info_activity_label));
    }

    /**
     * Reload scan fragment
     */
    private void reloadFragment() {
        Fragment fragment = ScanFragment.newInstance(getString(R.string.scan_activity_label));
        getSupportFragmentManager().beginTransaction().replace(R.id.fragment_layout, fragment).commit();
        setTitle(getString(R.string.scan_activity_label));
    }

}