package edu.uoc.tfg.nutriknow.views.fragments;

import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Parcelable;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.ScrollView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.adapters.FavoritesRecyclerAdapter;
import edu.uoc.tfg.nutriknow.model.Favorite;
import edu.uoc.tfg.nutriknow.utils.ProductInfoUtils;
import edu.uoc.tfg.nutriknow.utils.ProgressDialogUtils;
import edu.uoc.tfg.nutriknow.views.LoadData;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;

/**
 * Favorites fragment class
 */
public class FavoritesFragment extends Fragment implements LoadData {

    private static final String TITLE = "title";

    private static final String TAG = "FavoritesFragment";

    private static WelcomeActivity welcomeActivity;
    private FavoritesFragment favoritesFragment;
    private RelativeLayout relativeLayout;
    private TextView textView;

    private ScrollView scrollView;
    public RecyclerView recyclerView;
    public RecyclerView.Adapter adapter;
    private LinearLayout linearLayout;

    private AlertDialog.Builder builder;

    private String barcode;

    private final String KEY_RECYCLER_STATE = "recycler_state";
    private static Bundle mBundleRecyclerViewState;
    private boolean scrollToBegin = false;

    /**
     * Class constructor
     */
    public FavoritesFragment() {
    }

    /**
     * Create a new instance
     * @param title fragment title
     * @return fragment
     */
    public static FavoritesFragment newInstance(String title) {
        FavoritesFragment fragment = new FavoritesFragment();
        Bundle args = new Bundle();
        args.putString(TITLE, title);
        fragment.setArguments(args);
        return fragment;
    }

    /**
     * Create view
     * @param inflater inflater
     * @param container container
     * @param savedInstanceState bundle
     * @return view
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        final View view = inflater.inflate(R.layout.favorites_fragment, container, false);

        welcomeActivity = (WelcomeActivity) getActivity();
        // Configure toolbar
        welcomeActivity.prepareToolbarForFragments();
        // Create progress dialog and show during favorites retrieval
        welcomeActivity.progressDialog =
                ProgressDialogUtils.createProgressDialog(getContext(), R.string.favorites_loading_message);
        welcomeActivity.progressDialog.show();
        // Select options in bottombar and drawer
        welcomeActivity.navigationView.setCheckedItem(R.id.nav_my_favorites);
        welcomeActivity.bottomNavigationView.getMenu().getItem(3).setChecked(true);

        favoritesFragment = (FavoritesFragment) getFragmentManager()
                .findFragmentById(R.id.fragment_layout);

        // Allow to scroll to begin
        if (!(welcomeActivity.callingFragment instanceof ProductInfoFragment)
                && !(welcomeActivity.callingFragment instanceof FavoritesFragment)
                && !(welcomeActivity.callingFragment instanceof SearchesFragment)) {
            scrollToBegin = true;
        }
        welcomeActivity.callingFragment = favoritesFragment;

        // Reference to layout views
        linearLayout = view.findViewById(R.id.main_container);

        relativeLayout = (RelativeLayout) favoritesFragment.getActivity()
                .findViewById(R.id.mainLayout);

        textView = (TextView) view.findViewById(R.id.item);

        recyclerView = view.findViewById(R.id.recycler_view);

        RecyclerView.LayoutManager layoutManager = new LinearLayoutManager(getContext());
        recyclerView.setLayoutManager(layoutManager);

        scrollView = (ScrollView) view.findViewById(R.id.scroll);

        // If empty, change screen background
        if (welcomeActivity.dataHolder.getFavorites().isEmpty()) {
            linearLayout.setBackgroundColor(welcomeActivity.getColor(R.color.backgroundColorWindow));
        }
        adapter = new FavoritesRecyclerAdapter(welcomeActivity.dataHolder.getFavorites(), welcomeActivity.dataHolder.getProducts(),
                welcomeActivity, favoritesFragment);
        recyclerView.setAdapter(adapter);

        evaluateViewToDisplay();
        // Create alert dialog for favorite removal
        builder = new AlertDialog.Builder(getContext(), R.style.AlertDialogTheme);
        builder.setMessage(R.string.favorite_delete_dialog_text);
        builder.setIcon(R.drawable.ic_edit);
        builder.setPositiveButton(R.string.favorite_delete_dialog_yes_text, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.dismiss();
                deleteFromFavorites();
            }
        });
        builder.setNegativeButton(R.string.favorite_delete_dialog_no_text, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                barcode = null;
                dialog.dismiss();
            }
        });

        return view;
    }

    /**
     * on view created
     * @param view view
     * @param savedInstanceState bundle
     */
    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        final WelcomeActivity welcomeActivity = (WelcomeActivity) getActivity();
        this.welcomeActivity = welcomeActivity;
        // Retrieve data from database
        this.retrieveDataFromLocalDatabase();
    }

    /**
     * On resume
     */
    @Override
    public void onResume() {
        super.onResume();
        // Allow scroll to begin when defined
        if (!scrollToBegin) {
            if (mBundleRecyclerViewState != null) {
                Parcelable listState = mBundleRecyclerViewState.getParcelable(KEY_RECYCLER_STATE);
                recyclerView.getLayoutManager().onRestoreInstanceState(listState);
            }
        } else {
            scrollView.scrollTo(0, 0);
        }
        evaluateViewToDisplay();
    }

    /**
     * Save recyclerview state
     */
    @Override
    public void onPause() {
        super.onPause();
        mBundleRecyclerViewState = new Bundle();
        Parcelable listState = recyclerView.getLayoutManager().onSaveInstanceState();
        mBundleRecyclerViewState.putParcelable(KEY_RECYCLER_STATE, listState);
    }

    /**
     * Retrieve product information
     * @param barcode product barcode
     */
    public void retrieveProductInformation(String barcode) {
        welcomeActivity.progressDialog =
                ProgressDialogUtils.createProgressDialog(getContext(), R.string.product_info_loading_message);
        welcomeActivity.progressDialog.show();
        welcomeActivity.retrieveProductInformation(barcode, welcomeActivity.progressDialog);
    }

    /**
     * Delete product from favorite list
     * @param barcode product barcode
     */
    public void deleteProductFromFavorites(String barcode) {
        AlertDialog alert = builder.create();
        alert.show();
        this.barcode = barcode;
    }

    /**
     * Delete product from favorite list
     */
    private void deleteFromFavorites() {
        favoritesFragment = (FavoritesFragment) getFragmentManager()
                .findFragmentById(R.id.fragment_layout);
        relativeLayout = (RelativeLayout) favoritesFragment.getActivity()
                .findViewById(R.id.mainLayout);

        ProductInfoUtils.deleteProductFromFavorites(barcode, getContext(), welcomeActivity,
                relativeLayout, textView, linearLayout, favoritesFragment);
    }

    /**
     * Retrieve data from database
     */
    @Override
    public void retrieveDataFromLocalDatabase() {
        // Get catalogs from sqlite database
        ProductInfoUtils.retrieveDataFromLocalDatabase(welcomeActivity);
        // Get favorite info from firebase database
        welcomeActivity.dbReference.child(welcomeActivity.dataHolder.getUser().getUuid()).addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                List<Favorite> favorites = new ArrayList<>();
                for (DataSnapshot productSnapshot : dataSnapshot.child("/favorites").getChildren()) {
                    favorites.add(productSnapshot.getValue(Favorite.class));
                }
                welcomeActivity.dataHolder.setFavorites(favorites);

                adapter.notifyDataSetChanged();
                welcomeActivity.progressDialog.dismiss();
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {
                Log.e(TAG, "Database error: " + databaseError.toException());
            }
        });

        evaluateViewToDisplay();
    }

    /**
     * Evaluate which view display if recyclerview is empty or not
     */
    public void evaluateViewToDisplay() {
        if (welcomeActivity.dataHolder.getFavorites().isEmpty()) {
            scrollView.setVisibility(View.VISIBLE);
            recyclerView.setVisibility(View.GONE);
        } else {
            scrollView.setVisibility(View.GONE);
            recyclerView.setVisibility(View.VISIBLE);
        }
    }
}