package edu.uoc.tfg.nutriknow.views.fragments;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.fragment.app.Fragment;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.utils.ProgressDialogUtils;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;

/**
 * Home fragment class
 */
public class HomeFragment extends Fragment {

    private static final String TITLE = "title";

    /**
     * Class constructor
     */
    public HomeFragment() {
    }

    /**
     * Generate new instance
     *
     * @param title fragment title
     * @return fragment
     */
    public static HomeFragment newInstance(String title) {
        HomeFragment fragment = new HomeFragment();
        Bundle args = new Bundle();
        args.putString(TITLE, title);
        fragment.setArguments(args);
        return fragment;
    }

    /**
     * Create view
     *
     * @param inflater           inflater
     * @param container          container
     * @param savedInstanceState bundle
     * @return view
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.home_fragment, container, false);

        final WelcomeActivity welcomeActivity = (WelcomeActivity) getActivity();

        HomeFragment homeFragment = (HomeFragment) getFragmentManager()
                .findFragmentById(R.id.fragment_layout);

        welcomeActivity.callingFragment = homeFragment;

        // Show progress dialog the first time using the app, until data is loaded
        welcomeActivity.progressDialog =
                ProgressDialogUtils.createProgressDialog(getContext(), R.string.welcome_activity_loading_message);
        welcomeActivity.progressDialog.show();
        welcomeActivity.checkIfDataIsLoaded(welcomeActivity.progressDialog);
        // Configure toolbar
        welcomeActivity.toolbar.setNavigationIcon(R.mipmap.toolbar_logo);
        welcomeActivity.toolbar.getNavigationIcon().setTint(welcomeActivity.getColor(R.color.textColorPrimary));
        welcomeActivity.toolbar.setNavigationOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                welcomeActivity.toolbar.getNavigationIcon().setTint(welcomeActivity.getColor(R.color.textColorSecondary));
                welcomeActivity.reloadActivity();
            }
        });
        return view;
    }
}