package edu.uoc.tfg.nutriknow.views.fragments;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ExpandableListView;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.ScrollView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import com.joanzapata.iconify.IconDrawable;
import com.joanzapata.iconify.fonts.FontAwesomeIcons;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.adapters.AdditivesAdapter;
import edu.uoc.tfg.nutriknow.adapters.AllergensSummaryAdapter;
import edu.uoc.tfg.nutriknow.adapters.CategoriesIngredientsAdapter;
import edu.uoc.tfg.nutriknow.model.PageViewModel;
import edu.uoc.tfg.nutriknow.model.NovaGroups;
import edu.uoc.tfg.nutriknow.model.Product;
import edu.uoc.tfg.nutriknow.utils.IconDrawableUtils;
import edu.uoc.tfg.nutriknow.utils.ListViewUtils;
import edu.uoc.tfg.nutriknow.utils.ProductInfoUtils;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;

/**
 * Ingredients info fragment class
 */
public class IngredientsInfoFragment extends Fragment {

    private static final String ARG_SECTION_NUMBER = "section_number";
    private static final String ARG_PRODUCT = "product";

    private static final String ES = "es";

    private static IngredientsInfoFragment ingredientsInfoFragment;
    private Product product;

    private ScrollView scrollView;

    /**
     * Generate new instance
     *
     * @param index tab index
     * @param prod  product
     * @return fragment
     */
    public static IngredientsInfoFragment newInstance(int index, Product prod) {
        ingredientsInfoFragment = new IngredientsInfoFragment(prod);
        Bundle bundle = new Bundle();
        bundle.putInt(ARG_SECTION_NUMBER, index);
        bundle.putSerializable(ARG_PRODUCT, prod);
        ingredientsInfoFragment.setArguments(bundle);
        return ingredientsInfoFragment;
    }

    /**
     * Class constructor
     */
    public IngredientsInfoFragment(Product prod) {
        product = prod;
    }

    /**
     * Create view
     *
     * @param savedInstanceState bundle
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // Get tablayout reference
        PageViewModel pageViewModel = new ViewModelProvider(this, new ViewModelProvider.NewInstanceFactory()).get(PageViewModel.class);
        int index = 2;
        if (getArguments() != null) {
            index = getArguments().getInt(ARG_SECTION_NUMBER);
            product = (Product) getArguments().getSerializable(ARG_PRODUCT);
        }
        pageViewModel.setProduct(product);
        pageViewModel.setIndex(index);
    }

    /**
     * Create view
     *
     * @param inflater           inflater
     * @param container          container
     * @param savedInstanceState bundle
     * @return view
     */
    @Override
    public View onCreateView(
            @NonNull LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        final View root = inflater.inflate(R.layout.ingredients_info_fragment, container, false);

        WelcomeActivity welcomeActivity = (WelcomeActivity) getActivity();

        scrollView = (ScrollView) root.findViewById(R.id.scroll);

        // Reference to layout views and hide or display them depending on their values
        TextView productNameTextView = (TextView) root.findViewById(R.id.product_name_text);
        if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
            if (product.getSummaryData().getProductName_es() != null && !product.getSummaryData().getProductName_es().isEmpty()) {
                productNameTextView.setText(product.getSummaryData().getProductName_es());
            } else {
                if (product.getSummaryData().getProductName_en() != null && !product.getSummaryData().getProductName_en().isEmpty()) {
                    productNameTextView.setText(product.getSummaryData().getProductName_en());
                } else {
                    productNameTextView.setText(getString(R.string.empty_product_name));
                }
            }
        } else {
            if (product.getSummaryData().getProductName_en() != null && !product.getSummaryData().getProductName_en().isEmpty()) {
                productNameTextView.setText(product.getSummaryData().getProductName_en());
            } else {
                if (product.getSummaryData().getProductName_es() != null && !product.getSummaryData().getProductName_es().isEmpty()) {
                    productNameTextView.setText(product.getSummaryData().getProductName_es());
                } else {
                    productNameTextView.setText(getString(R.string.empty_product_name));
                }
            }
        }

        TextView completenessTextLabelView = (TextView) root.findViewById(R.id.completeness_label_text);
        TextView completenessTextView = (TextView) root.findViewById(R.id.completeness_text);
        if (product.getSummaryData().getCompleteness() == 0) {
            completenessTextLabelView.setVisibility(View.GONE);
            completenessTextView.setVisibility(View.GONE);
        } else {
            float complete =
                    ProductInfoUtils.roundFloat(product.getSummaryData().getCompleteness() * 100f, 2);
            String completeness = complete + " %";
            completenessTextView.setText(completeness);
            // Add icon and change color depending on the field value
            int color = 0;
            if (complete >= 60) {
                color = R.color.textColorSecondary;
            } else if (complete < 60 && complete >= 40) {
                color = R.color.textColorWarning;
            } else {
                color = R.color.textColorError;
            }
            IconDrawable warning = IconDrawableUtils
                    .createIconDrawable(getContext(), FontAwesomeIcons.fa_warning,
                            (int) completenessTextView.getTextSize(), color, false);
            completenessTextView.setCompoundDrawablesWithIntrinsicBounds(warning, null, null, null);
        }

        RelativeLayout ingredientsLayout = (RelativeLayout) root.findViewById(R.id.ingredients_content_layout);
        TextView ingredientsTextView = (TextView) root.findViewById(R.id.ingredients_text);
        if ((product.getIngredientsData().getIngredients_es() == null || product.getIngredientsData().getIngredients_es().isEmpty()) &&
                (product.getIngredientsData().getIngredients_en() == null || product.getIngredientsData().getIngredients_en().isEmpty())) {
            ingredientsLayout.setVisibility(View.GONE);
        } else {
            if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
                if (product.getIngredientsData().getIngredients_es() != null && !product.getIngredientsData().getIngredients_es().isEmpty()) {
                    ingredientsTextView.setText(product.getIngredientsData().getIngredients_es());
                } else {
                    if (product.getIngredientsData().getIngredients_en() != null && !product.getIngredientsData().getIngredients_en().isEmpty()) {
                        ingredientsTextView.setText(product.getIngredientsData().getIngredients_en());
                    } else {
                        ingredientsLayout.setVisibility(View.GONE);
                    }
                }
            } else {
                if (product.getIngredientsData().getIngredients_en() != null && !product.getIngredientsData().getIngredients_en().isEmpty()) {
                    ingredientsTextView.setText(product.getIngredientsData().getIngredients_en());
                } else {
                    if (product.getIngredientsData().getIngredients_es() != null && !product.getIngredientsData().getIngredients_es().isEmpty()) {
                        ingredientsTextView.setText(product.getIngredientsData().getIngredients_es());
                    } else {
                        ingredientsLayout.setVisibility(View.GONE);
                    }
                }
            }
        }

        RelativeLayout allergensLayout = (RelativeLayout) root.findViewById(R.id.allergens_content_layout);
        ListView allergensListView = (ListView) root.findViewById(R.id.allergens_list);
        AllergensSummaryAdapter allergensSummaryAdapter = null;
        if (product.getIngredientsData().getAllergens_es().isEmpty() &&
                product.getIngredientsData().getAllergens_en().isEmpty()) {
            allergensLayout.setVisibility(View.GONE);
        } else {
            if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
                if (!product.getIngredientsData().getAllergens_es().isEmpty()) {
                    allergensSummaryAdapter = new AllergensSummaryAdapter(product.getIngredientsData().getAllergens_es(), getContext());
                } else {
                    if (!product.getIngredientsData().getAllergens_en().isEmpty()) {
                        allergensSummaryAdapter = new AllergensSummaryAdapter(product.getIngredientsData().getAllergens_en(), getContext());
                    } else {
                        allergensLayout.setVisibility(View.GONE);
                    }
                }
            } else {
                if (!product.getIngredientsData().getAllergens_en().isEmpty()) {
                    allergensSummaryAdapter = new AllergensSummaryAdapter(product.getIngredientsData().getAllergens_en(), getContext());
                } else {
                    if (!product.getIngredientsData().getAllergens_es().isEmpty()) {
                        allergensSummaryAdapter = new AllergensSummaryAdapter(product.getIngredientsData().getAllergens_es(), getContext());
                    } else {
                        allergensLayout.setVisibility(View.GONE);
                    }
                }
            }
        }
        if (allergensSummaryAdapter != null) {
            allergensListView.setAdapter(allergensSummaryAdapter);
            ListViewUtils.setListViewHeightBasedOnChildren(allergensListView, ingredientsInfoFragment);
        }

        RelativeLayout tracesLayout = (RelativeLayout) root.findViewById(R.id.traces_content_layout);
        ListView tracesListView = (ListView) root.findViewById(R.id.traces_list);
        AllergensSummaryAdapter tracesSummaryAdapter = null;
        if (product.getIngredientsData().getTraces_es().isEmpty() &&
                product.getIngredientsData().getTraces_en().isEmpty()) {
            tracesLayout.setVisibility(View.GONE);
        } else {
            if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
                if (!product.getIngredientsData().getTraces_es().isEmpty()) {
                    tracesSummaryAdapter = new AllergensSummaryAdapter(product.getIngredientsData().getTraces_es(), getContext());
                } else {
                    if (!product.getIngredientsData().getTraces_en().isEmpty()) {
                        tracesSummaryAdapter = new AllergensSummaryAdapter(product.getIngredientsData().getTraces_en(), getContext());
                    } else {
                        tracesLayout.setVisibility(View.GONE);
                    }
                }
            } else {
                if (!product.getIngredientsData().getTraces_en().isEmpty()) {
                    tracesSummaryAdapter = new AllergensSummaryAdapter(product.getIngredientsData().getTraces_en(), getContext());
                } else {
                    if (!product.getIngredientsData().getTraces_es().isEmpty()) {
                        tracesSummaryAdapter = new AllergensSummaryAdapter(product.getIngredientsData().getTraces_es(), getContext());
                    } else {
                        tracesLayout.setVisibility(View.GONE);
                    }
                }
            }
        }
        if (tracesSummaryAdapter != null) {
            tracesListView.setAdapter(tracesSummaryAdapter);
            ListViewUtils.setListViewHeightBasedOnChildren(tracesListView, ingredientsInfoFragment);
        }

        RelativeLayout categoriesLayout = (RelativeLayout) root.findViewById(R.id.categories_content_layout);
        ListView categoriesListView = (ListView) root.findViewById(R.id.categories_list);
        CategoriesIngredientsAdapter categoriesSummaryAdapter = null;
        if (product.getIngredientsData().getCategories_es().isEmpty() &&
                product.getIngredientsData().getCategories_en().isEmpty()) {
            categoriesLayout.setVisibility(View.GONE);
        } else {
            if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
                if (!product.getIngredientsData().getCategories_es().isEmpty()) {
                    categoriesSummaryAdapter = new CategoriesIngredientsAdapter(
                            product.getIngredientsData().getCategories_es(), getContext());
                } else {
                    if (!product.getIngredientsData().getCategories_en().isEmpty()) {
                        categoriesSummaryAdapter = new CategoriesIngredientsAdapter(
                                product.getIngredientsData().getCategories_en(), getContext());
                    } else {
                        categoriesLayout.setVisibility(View.GONE);
                    }
                }
            } else {
                if (!product.getIngredientsData().getCategories_en().isEmpty()) {
                    categoriesSummaryAdapter = new CategoriesIngredientsAdapter(
                            product.getIngredientsData().getCategories_en(), getContext());
                } else {
                    if (!product.getIngredientsData().getCategories_es().isEmpty()) {
                        categoriesSummaryAdapter = new CategoriesIngredientsAdapter(
                                product.getIngredientsData().getCategories_es(), getContext());
                    } else {
                        categoriesLayout.setVisibility(View.GONE);
                    }
                }
            }
        }
        if (categoriesSummaryAdapter != null) {
            categoriesListView.setAdapter(categoriesSummaryAdapter);
            ListViewUtils.setListViewHeightBasedOnChildren(categoriesListView, ingredientsInfoFragment);
        }

        RelativeLayout novaGroupAdditivesLayout = (RelativeLayout) root.findViewById(R.id.nova_group_additives_content_layout);

        RelativeLayout novaGroupLayout = (RelativeLayout) root.findViewById(R.id.nova_group_content_layout);
        TextView novaGroupTextView = (TextView) root.findViewById(R.id.nova_group_text);
        ImageView novaGroupImageView = (ImageView) root.findViewById(R.id.nova_group_icon);

        RelativeLayout additivesLayout = (RelativeLayout) root.findViewById(R.id.additives_content_layout);
        ExpandableListView additivesListView = (ExpandableListView) root.findViewById(R.id.additives_list);
        // Disable group click in expandable list view
        additivesListView.setOnGroupClickListener(new ExpandableListView.OnGroupClickListener() {
            @Override
            public boolean onGroupClick(ExpandableListView parent, View v, int groupPosition, long id) {
                return true;
            }
        });
        additivesListView.setOnChildClickListener(new ExpandableListView.OnChildClickListener() {
            @Override
            public boolean onChildClick(ExpandableListView parent, View v, int groupPosition, int childPosition, long id) {
                return true;
            }
        });
        AdditivesAdapter additivesSummaryAdapter = null;
        if (product.getIngredientsData().getNovaGroups().isEmpty() && product.getIngredientsData().getAdditives().isEmpty() &&
                (product.getIngredientsData().getNovaGroup() <= 0 || product.getIngredientsData().getNovaGroup() > 4)) {
            novaGroupAdditivesLayout.setVisibility(View.GONE);
        } else {
            if (product.getIngredientsData().getNovaGroups().isEmpty() &&
                    (product.getIngredientsData().getNovaGroup() <= 0 || product.getIngredientsData().getNovaGroup() > 4)) {
                novaGroupLayout.setVisibility(View.GONE);
            } else {
                if (!product.getIngredientsData().getNovaGroups().isEmpty()) {
                    NovaGroups.NovaGroup novaGroup = product.getIngredientsData().getNovaGroups().get(0);
                    if (novaGroup != null) {
                        if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
                            novaGroupTextView.setText(novaGroup.name.es);
                        } else {
                            novaGroupTextView.setText(novaGroup.name.en);
                        }
                    } else {
                        novaGroupTextView.setVisibility(View.GONE);
                    }
                } else {
                    novaGroupTextView.setVisibility(View.GONE);
                }
                if ((product.getIngredientsData().getNovaGroup() > 0 && product.getIngredientsData().getNovaGroup() <= 4)) {
                    // Assign drawable depending on the field value
                    int drawableId = 0;
                    switch (product.getIngredientsData().getNovaGroup()) {
                        case 1:
                            drawableId = R.drawable.ic_nova_group_1;
                            break;
                        case 2:
                            drawableId = R.drawable.ic_nova_group_2;
                            break;
                        case 3:
                            drawableId = R.drawable.ic_nova_group_3;
                            break;
                        case 4:
                            drawableId = R.drawable.ic_nova_group_4;
                            break;
                    }
                    novaGroupImageView.setImageDrawable(getContext().getDrawable(drawableId));
                } else {
                    novaGroupImageView.setVisibility(View.GONE);
                }
            }
            if (product.getIngredientsData().getAdditives().isEmpty()) {
                additivesLayout.setVisibility(View.GONE);
            } else {
                additivesSummaryAdapter = new AdditivesAdapter(
                        product.getIngredientsData().getAdditives(), getContext(),
                        welcomeActivity.dataHolder.getCurrentLocale().getLanguage(), additivesListView);
            }
            if (additivesSummaryAdapter != null) {
                additivesListView.setAdapter(additivesSummaryAdapter);
            }
        }
        return root;
    }

    /**
     * Scroll to begin when tab is invisible to the user
     * @param isVisibleToUser
     */
    @Override
    public void setUserVisibleHint(boolean isVisibleToUser) {
        super.setUserVisibleHint(isVisibleToUser);
        if (this.isVisible()) {
            if (!isVisibleToUser) {
                scrollView.scrollTo(0, 0);
            }
        }
    }
}