package edu.uoc.tfg.nutriknow.views.fragments;

import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.ScrollView;
import android.widget.TextView;

import com.joanzapata.iconify.IconDrawable;
import com.joanzapata.iconify.fonts.FontAwesomeIcons;

import java.util.List;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.adapters.NutrientsLevelsNutritionalAdapter;
import edu.uoc.tfg.nutriknow.model.PageViewModel;
import edu.uoc.tfg.nutriknow.model.Nutriments;
import edu.uoc.tfg.nutriknow.model.Product;
import edu.uoc.tfg.nutriknow.utils.IconDrawableUtils;
import edu.uoc.tfg.nutriknow.utils.ListViewUtils;
import edu.uoc.tfg.nutriknow.utils.ProductInfoUtils;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;

/**
 * Nutritional info fragment class
 */
public class NutritionalInfoFragment extends Fragment {

    private static final String ARG_SECTION_NUMBER = "section_number";
    private static final String ARG_PRODUCT = "product";

    private static final String ES = "es";

    private static NutritionalInfoFragment nutritionalInfoFragment;
    private Product product;

    private ScrollView scrollView;

    /**
     * Generate new instance
     *
     * @param index tab index
     * @param prod  product
     * @return fragment
     */
    public static NutritionalInfoFragment newInstance(int index, Product prod) {
        nutritionalInfoFragment = new NutritionalInfoFragment(prod);
        Bundle bundle = new Bundle();
        bundle.putInt(ARG_SECTION_NUMBER, index);
        bundle.putSerializable(ARG_PRODUCT, prod);
        nutritionalInfoFragment.setArguments(bundle);
        return nutritionalInfoFragment;
    }

    /**
     * Class constructor
     */
    public NutritionalInfoFragment(Product prod) {
        product = prod;
    }

    /**
     * Create view
     *
     * @param savedInstanceState bundle
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // Get tablayout reference
        PageViewModel pageViewModel = new ViewModelProvider(this, new ViewModelProvider.NewInstanceFactory()).get(PageViewModel.class);
        int index = 3;
        if (getArguments() != null) {
            index = getArguments().getInt(ARG_SECTION_NUMBER);
            product = (Product) getArguments().getSerializable(ARG_PRODUCT);
        }
        pageViewModel.setProduct(product);
        pageViewModel.setIndex(index);
    }

    /**
     * Create view
     *
     * @param inflater           inflater
     * @param container          container
     * @param savedInstanceState bundle
     * @return view
     */
    @Override
    public View onCreateView(
            @NonNull LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        final View root = inflater.inflate(R.layout.nutritional_info_fragment, container, false);

        WelcomeActivity welcomeActivity = (WelcomeActivity) getActivity();

        scrollView = (ScrollView) root.findViewById(R.id.scroll);

        // Reference to layout views and hide or display them depending on their values
        TextView productNameTextView = (TextView) root.findViewById(R.id.product_name_text);
        if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
            if (product.getSummaryData().getProductName_es() != null && !product.getSummaryData().getProductName_es().isEmpty()) {
                productNameTextView.setText(product.getSummaryData().getProductName_es());
            } else {
                if (product.getSummaryData().getProductName_en() != null && !product.getSummaryData().getProductName_en().isEmpty()) {
                    productNameTextView.setText(product.getSummaryData().getProductName_en());
                } else {
                    productNameTextView.setText(getString(R.string.empty_product_name));
                }
            }
        } else {
            if (product.getSummaryData().getProductName_en() != null && !product.getSummaryData().getProductName_en().isEmpty()) {
                productNameTextView.setText(product.getSummaryData().getProductName_en());
            } else {
                if (product.getSummaryData().getProductName_es() != null && !product.getSummaryData().getProductName_es().isEmpty()) {
                    productNameTextView.setText(product.getSummaryData().getProductName_es());
                } else {
                    productNameTextView.setText(getString(R.string.empty_product_name));
                }
            }
        }

        TextView completenessTextLabelView = (TextView) root.findViewById(R.id.completeness_label_text);
        TextView completenessTextView = (TextView) root.findViewById(R.id.completeness_text);
        if (product.getSummaryData().getCompleteness() == 0) {
            completenessTextLabelView.setVisibility(View.GONE);
            completenessTextView.setVisibility(View.GONE);
        } else {
            float complete =
                    ProductInfoUtils.roundFloat(product.getSummaryData().getCompleteness() * 100f, 2);
            String completeness = complete + " %";
            completenessTextView.setText(completeness);
            // Add icon and change color depending on the field value
            int color = 0;
            if (complete >= 60) {
                color = R.color.textColorSecondary;
            } else if (complete < 60 && complete >= 40) {
                color = R.color.textColorWarning;
            } else {
                color = R.color.textColorError;
            }
            IconDrawable warning = IconDrawableUtils
                    .createIconDrawable(getContext(), FontAwesomeIcons.fa_warning,
                            (int) completenessTextView.getTextSize(), color, false);
            completenessTextView.setCompoundDrawablesWithIntrinsicBounds(warning, null, null, null);
        }

        RelativeLayout nutriscoreLayout = (RelativeLayout) root.findViewById(R.id.nutriscore_content_layout);
        ImageView nutriscoreImageView = (ImageView) root.findViewById(R.id.nutriscore_icon);

        if (product.getNutritionalData().getNutritionGrade() == null ||
                product.getNutritionalData().getNutritionGrade().isEmpty()) {
            nutriscoreLayout.setVisibility(View.GONE);
        } else {
            // Assign drawable depending on the field value
            int drawableId = 0;
            switch (product.getNutritionalData().getNutritionGrade().toUpperCase().trim()) {
                case "A":
                    drawableId = R.drawable.ic_nutriscore_a;
                    break;
                case "B":
                    drawableId = R.drawable.ic_nutriscore_b;
                    break;
                case "C":
                    drawableId = R.drawable.ic_nutriscore_c;
                    break;
                case "D":
                    drawableId = R.drawable.ic_nutriscore_d;
                    break;
                case "E":
                    drawableId = R.drawable.ic_nutriscore_e;
                    break;
            }
            nutriscoreImageView.setImageDrawable(getContext().getDrawable(drawableId));
        }

        RelativeLayout nutrientsLevelsLayout = (RelativeLayout) root.findViewById(R.id.nutrient_levels_content_layout);
        ListView nutrientsLevelsListView = (ListView) root.findViewById(R.id.nutrient_levels_list);
        NutrientsLevelsNutritionalAdapter nutrientsLevelsNutritionalAdapter = null;
        if (product.getIngredientsData().getAdditives().isEmpty()) {
            nutrientsLevelsLayout.setVisibility(View.GONE);
        } else {
            nutrientsLevelsNutritionalAdapter = new NutrientsLevelsNutritionalAdapter(
                    product.getNutritionalData().getNutrientLevels(), getContext(),
                    welcomeActivity.dataHolder.getCurrentLocale().getLanguage());
        }
        if (nutrientsLevelsNutritionalAdapter != null) {
            nutrientsLevelsListView.setAdapter(nutrientsLevelsNutritionalAdapter);
            ListViewUtils.setListViewHeightBasedOnChildren(nutrientsLevelsListView, nutritionalInfoFragment, getContext());
        }

        RelativeLayout nutrimentsLayout = (RelativeLayout) root.findViewById(R.id.nutriments_layout);

        if (product.getNutritionalData().getNutriments() == null || product.getNutritionalData().getNutriments().isEmpty()) {
            nutrimentsLayout.setVisibility(View.GONE);
        } else {
            TextView servingSize = (TextView) root.findViewById(R.id.serving_text);
            servingSize.setText(product.getNutritionalData().getServingSize());

            RelativeLayout energyLayout = (RelativeLayout) root.findViewById(R.id.energy_content_layout);
            RelativeLayout fatLayout = (RelativeLayout) root.findViewById(R.id.fat_content_layout);
            RelativeLayout saturatedFatLayout = (RelativeLayout) root.findViewById(R.id.saturated_fat_content_layout);
            RelativeLayout carbohydratesLayout = (RelativeLayout) root.findViewById(R.id.carbohydrates_content_layout);
            RelativeLayout proteinsLayout = (RelativeLayout) root.findViewById(R.id.proteins_content_layout);
            RelativeLayout sugarsLayout = (RelativeLayout) root.findViewById(R.id.sugars_content_layout);
            RelativeLayout saltLayout = (RelativeLayout) root.findViewById(R.id.salts_content_layout);
            RelativeLayout sodiumLayout = (RelativeLayout) root.findViewById(R.id.sodium_content_layout);

            TextView energyServing = (TextView) root.findViewById(R.id.energy_serving_text);
            TextView energy100 = (TextView) root.findViewById(R.id.energy_100_text);
            TextView fatServing = (TextView) root.findViewById(R.id.fat_serving_text);
            TextView fat100 = (TextView) root.findViewById(R.id.fat_100_text);
            TextView saturatedFatServing = (TextView) root.findViewById(R.id.saturated_fat_serving_text);
            TextView saturatedFat100 = (TextView) root.findViewById(R.id.saturated_fat_100_text);
            TextView carbohydratesServing = (TextView) root.findViewById(R.id.carbohydrates_serving_text);
            TextView carbohydrates100 = (TextView) root.findViewById(R.id.carbohydrates_100_text);
            TextView proteinsServing = (TextView) root.findViewById(R.id.proteins_serving_text);
            TextView proteins100 = (TextView) root.findViewById(R.id.proteins_100_text);
            TextView sugarsServing = (TextView) root.findViewById(R.id.sugars_serving_text);
            TextView sugars100 = (TextView) root.findViewById(R.id.sugars_100_text);
            TextView saltServing = (TextView) root.findViewById(R.id.salts_serving_text);
            TextView salt100 = (TextView) root.findViewById(R.id.salts_100_text);
            TextView sodiumServing = (TextView) root.findViewById(R.id.sodium_serving_text);
            TextView sodium100 = (TextView) root.findViewById(R.id.sodium_100_text);

            List<Nutriments> nutriments = product.getNutritionalData().getNutriments();
            for (Nutriments nutriment : nutriments) {
                String servingValue = nutriment.getValueServing() + " " + nutriment.getUnit();
                String value100 = nutriment.getValue100() + " " + nutriment.getUnit();
                switch (nutriment.getCode()) {
                    case "energy-kcal":
                        if (product.getNutritionalData().getServingSize().isEmpty() &&
                                nutriment.getValue100() == 0) {
                            energyLayout.setVisibility(View.GONE);
                        } else if (product.getNutritionalData().getServingSize().isEmpty()) {
                            energyServing.setVisibility(View.GONE);
                        } else {
                            energyServing.setText(servingValue);
                        }
                        energy100.setText(value100);
                        break;
                    case "fat":
                        if (product.getNutritionalData().getServingSize().isEmpty() &&
                                nutriment.getValue100() == 0) {
                            fatLayout.setVisibility(View.GONE);
                        } else if (product.getNutritionalData().getServingSize().isEmpty()) {
                            fatServing.setVisibility(View.GONE);
                        } else {
                            fatServing.setText(servingValue);
                        }
                        fat100.setText(value100);
                        break;
                    case "saturated-fat":
                        if (product.getNutritionalData().getServingSize().isEmpty() &&
                                nutriment.getValue100() == 0) {
                            saturatedFatLayout.setVisibility(View.GONE);
                        } else if (product.getNutritionalData().getServingSize().isEmpty()) {
                            saturatedFatServing.setVisibility(View.GONE);
                        } else {
                            saturatedFatServing.setText(servingValue);
                        }
                        saturatedFat100.setText(value100);
                        break;
                    case "carbohydrates":
                        if (product.getNutritionalData().getServingSize().isEmpty() &&
                                nutriment.getValue100() == 0) {
                            carbohydratesLayout.setVisibility(View.GONE);
                        } else if (product.getNutritionalData().getServingSize().isEmpty()) {
                            carbohydratesServing.setVisibility(View.GONE);
                        } else {
                            carbohydratesServing.setText(servingValue);
                        }
                        carbohydrates100.setText(value100);
                        break;
                    case "sugars":
                        if (product.getNutritionalData().getServingSize().isEmpty() &&
                                nutriment.getValue100() == 0) {
                            sugarsLayout.setVisibility(View.GONE);
                        } else if (product.getNutritionalData().getServingSize().isEmpty()) {
                            sugarsServing.setVisibility(View.GONE);
                        } else {
                            sugarsServing.setText(servingValue);
                        }
                        sugars100.setText(value100);
                        break;
                    case "proteins":
                        if (product.getNutritionalData().getServingSize().isEmpty() &&
                                nutriment.getValue100() == 0) {
                            proteinsLayout.setVisibility(View.GONE);
                        } else if (product.getNutritionalData().getServingSize().isEmpty()) {
                            proteinsServing.setVisibility(View.GONE);
                        } else {
                            proteinsServing.setText(servingValue);
                        }
                        proteins100.setText(value100);
                        break;
                    case "salt":
                        if (product.getNutritionalData().getServingSize().isEmpty() &&
                                nutriment.getValue100() == 0) {
                            saltLayout.setVisibility(View.GONE);
                        } else if (product.getNutritionalData().getServingSize().isEmpty()) {
                            saltServing.setVisibility(View.GONE);
                        } else {
                            saltServing.setText(servingValue);
                        }
                        salt100.setText(value100);
                        break;
                    case "sodium":
                        if (product.getNutritionalData().getServingSize().isEmpty() &&
                                nutriment.getValue100() == 0) {
                            sodiumLayout.setVisibility(View.GONE);
                        } else if (product.getNutritionalData().getServingSize().isEmpty()) {
                            sodiumServing.setVisibility(View.GONE);
                        } else {
                            sodiumServing.setText(servingValue);
                        }
                        sodium100.setText(value100);
                        break;
                }
            }
        }
        return root;
    }

    /**
     * Scroll to begin when tab is invisible to the user
     * @param isVisibleToUser
     */
    @Override
    public void setUserVisibleHint(boolean isVisibleToUser) {
        super.setUserVisibleHint(isVisibleToUser);
        if (this.isVisible()) {
            if (!isVisibleToUser) {
                scrollView.scrollTo(0, 0);
            }
        }
    }
}