package edu.uoc.tfg.nutriknow.views.fragments;

import android.app.ProgressDialog;
import android.content.res.ColorStateList;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.EditorInfo;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.core.widget.ImageViewCompat;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;

import com.google.android.material.textfield.TextInputLayout;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.regex.Pattern;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.api.OpenFoodFactsClient;
import edu.uoc.tfg.nutriknow.api.interfaces.OpenFoodFactsService;
import edu.uoc.tfg.nutriknow.model.Allergens;
import edu.uoc.tfg.nutriknow.model.Favorite;
import edu.uoc.tfg.nutriknow.model.Profile;
import edu.uoc.tfg.nutriknow.model.User;
import edu.uoc.tfg.nutriknow.sqlite.adapters.AllergensDataBaseAdapter;
import edu.uoc.tfg.nutriknow.utils.DatabaseCatalogUtils;
import edu.uoc.tfg.nutriknow.utils.InputValidationUtils;
import edu.uoc.tfg.nutriknow.utils.KeyboardUtils;
import edu.uoc.tfg.nutriknow.utils.ListViewUtils;
import edu.uoc.tfg.nutriknow.utils.ProgressDialogUtils;
import edu.uoc.tfg.nutriknow.utils.SnackBarUtils;
import edu.uoc.tfg.nutriknow.views.LoadData;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * Profile setup fragment class, implements load data interface
 */
public class ProfileSetupFragment extends Fragment implements LoadData {

    private static final String TITLE = "title";
    private static final String TAG = "ProfileSetupFragment";
    private static final String ES = "es";
    private static final String EN = "en";
    private static final String PERIODICITY_D = "D";
    private static final String PERIODICITY_W = "W";
    private static final String PERIODICITY_N = "N";

    private final Map<String, List<Allergens.Allergen>> retrievedAllergens = new HashMap<>();
    private List<String> allergensToShow = new ArrayList<>();

    private ProfileSetupFragment profileSetupFragment;
    private TextView userText;
    private TextView emailText;
    private Button setupButton;
    private RelativeLayout relativeLayout;
    private WelcomeActivity welcomeActivity;
    private AutoCompleteTextView autoCompleteText;
    private TextInputLayout allergensTextInput;
    private ArrayAdapter<String> adapterForAutoComplete;
    private ImageView addButton;

    private boolean validAllergen = false;
    private static final Pattern pattern = Pattern.compile("\\b[A-Z].*?\\b");

    private ListView listView;
    private ArrayAdapter<String> adapterForListView;

    private String periodicity = PERIODICITY_D;
    private String selectedAllergen;

    private final Map<String, List<Allergens.Allergen>> allergensToSaveInProfile = new HashMap<>();

    /**
     * Class constructor
     */
    public ProfileSetupFragment() {
    }

    /**
     * Create a new instance
     *
     * @param title fragment title
     * @return fragment
     */
    public static ProfileSetupFragment newInstance(String title) {
        ProfileSetupFragment fragment = new ProfileSetupFragment();
        Bundle args = new Bundle();
        args.putString(TITLE, title);
        fragment.setArguments(args);
        return fragment;
    }

    /**
     * Create view
     *
     * @param savedInstanceState bundle
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        final View view = inflater.inflate(R.layout.profile_setup_fragment, container, false);

        final WelcomeActivity welcomeActivity = (WelcomeActivity) getActivity();
        this.welcomeActivity = welcomeActivity;
        setupButton = (Button) view.findViewById(R.id.btn_setup);
        // Configure toolbar depending on if the user has a defined profile or not
        if (welcomeActivity.dataHolder.getProfile() == null) {
            welcomeActivity.setupToolbar(true);
            welcomeActivity.toolbar.setNavigationOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    welcomeActivity.toolbar.getNavigationIcon().setTint(welcomeActivity.getColor(R.color.textColorSecondary));
                    welcomeActivity.logout();
                }
            });
            welcomeActivity.drawerToggle.hideDrawer();
            welcomeActivity.drawerLayout.setDrawerLockMode(DrawerLayout.LOCK_MODE_LOCKED_CLOSED);
            welcomeActivity.bottomNavigationView.setVisibility(View.GONE);
            setupButton.setText(R.string.setup_button);
        } else {
            welcomeActivity.toolbar.setNavigationIcon(welcomeActivity.defaultNavigationIcon);
            welcomeActivity.toolbar.getNavigationIcon().setTint(getActivity().getColor(R.color.textColorPrimary));
            welcomeActivity.toolbar.setNavigationOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    goToUserAccountScreen();
                }
            });
            setupButton.setText(R.string.save_button);
        }
        profileSetupFragment = (ProfileSetupFragment) getFragmentManager()
                .findFragmentById(R.id.fragment_layout);

        welcomeActivity.callingFragment = profileSetupFragment;

        // Reference layout views
        relativeLayout = (RelativeLayout) profileSetupFragment.getActivity()
                .findViewById(R.id.mainLayout);

        userText = (TextView) view.findViewById(R.id.user_profile_text);
        userText.setText(welcomeActivity.dataHolder.getUser().getUserName());

        emailText = (TextView) view.findViewById(R.id.email_profile_text);
        emailText.setText(welcomeActivity.dataHolder.getUser().getEmail());

        RadioGroup radioGroup = (RadioGroup) view.findViewById(R.id.radio_group);
        // Add radio group selection functionality
        radioGroup.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                // checkedId is the RadioButton selected
                switch (checkedId) {
                    case R.id.periodicity_daily:
                        periodicity = PERIODICITY_D;
                        break;
                    case R.id.periodicity_weekly:
                        periodicity = PERIODICITY_W;
                        break;
                    case R.id.periodicity_never:
                        periodicity = PERIODICITY_N;
                        break;
                }
            }
        });

        allergensTextInput = (TextInputLayout) view.findViewById(R.id.input_allergens);
        autoCompleteText = (AutoCompleteTextView) view.findViewById(R.id.allergens);
        // Add validation functionality
        autoCompleteText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                allergensTextInput.setHintEnabled(false);
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                validAllergen = InputValidationUtils
                        .validateAllergensContent(getContext(), allergensTextInput, String.valueOf(s),
                                allergensToShow);
            }

            @Override
            public void afterTextChanged(Editable s) {
                if (validAllergen) {
                    selectedAllergen = autoCompleteText.getText().toString();
                    setupButton.setEnabled(true);
                    addButton.setEnabled(true);
                } else {
                    selectedAllergen = "";
                    setupButton.setEnabled(false);
                    addButton.setEnabled(false);
                }
            }
        });

        final List<String> listForAdapter = new ArrayList<String>();

        autoCompleteText.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
                if ((event != null && (event.getKeyCode() == KeyEvent.KEYCODE_ENTER)) || (actionId == EditorInfo.IME_ACTION_DONE)) {
                    addAllergenToList(listForAdapter);
                }
                return false;
            }
        });

        listView = (ListView) view.findViewById(R.id.selectedList);
        adapterForListView = new ArrayAdapter<String>(getContext(), R.layout.allergens_editable_row_layout, R.id.label,
                listForAdapter);
        listView.setAdapter(adapterForListView);
        // Add list item on click functionality
        listView.setOnItemClickListener(
                new AdapterView.OnItemClickListener() {
                    @Override
                    public void onItemClick(AdapterView<?> arg0, View view,
                                            int position, long id) {
                        String item = (String) adapterForListView.getItem(position);
                        listForAdapter.remove(item);
                        adapterForListView.notifyDataSetChanged();
                        updateAllergensToSaveInProfile(listForAdapter, welcomeActivity.dataHolder.getCurrentLocale().getLanguage());
                        ListViewUtils.setListViewHeightBasedOnChildren(listView, profileSetupFragment);
                        if (listForAdapter.isEmpty()) {
                            listView.setVisibility(View.GONE);
                        }
                    }
                }
        );
        retrieveDataFromProfileIfAny(view, listForAdapter);

        addButton = (ImageView) view.findViewById(R.id.add_icon);
        // Add add icon functionality (add allergen to profile list)
        addButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                addAllergenToList(listForAdapter);
            }
        });
        // Add setup button functionality
        setupButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                // Create user in database
                setupButton.setEnabled(false);

                int messageId = (welcomeActivity.dataHolder.getUser().isHasProfile()) ?
                        R.string.profile_setup_saving_message : R.string.profile_setup_configuring_message;
                welcomeActivity.progressDialog =
                        ProgressDialogUtils.createProgressDialog(getContext(), messageId);
                welcomeActivity.progressDialog.show();
                User currentUser = welcomeActivity.dataHolder.getUser();

                Locale currentLocale = getResources().getConfiguration().locale;
                welcomeActivity.dataHolder.setCurrentLocale(currentLocale);

                Profile profile = new Profile(periodicity, new Date(), currentLocale.getLanguage(),
                        allergensToSaveInProfile.get(ES), allergensToSaveInProfile.get(EN));
                welcomeActivity.dbReference = welcomeActivity.database.getReference("/users");
                // If the user has not a configured profile, update flag
                if (!welcomeActivity.dataHolder.getUser().isHasProfile()) {
                    currentUser.setHasProfile(true);
                    welcomeActivity.dbReference.child(currentUser.getUuid()).child("/user").setValue(currentUser,
                            new DatabaseReference.CompletionListener() {
                                @Override
                                public void onComplete(DatabaseError databaseError,
                                                       DatabaseReference databaseReference) {
                                    if (databaseError == null) {

                                    } else {
                                        Log.e(TAG, "Database error: " + databaseError.toException());
                                        ProfileSetupFragment.this.onSetupFailed();
                                        welcomeActivity.progressDialog.dismiss();
                                    }
                                }
                            });
                }
                // Add profile to database
                welcomeActivity.dbReference.child(currentUser.getUuid()).child("profile").setValue(profile,
                        new DatabaseReference.CompletionListener() {
                            @Override
                            public void onComplete(DatabaseError databaseError,
                                                   DatabaseReference databaseReference) {
                                if (databaseError == null) {
                                    welcomeActivity.dataHolder.getUser().setHasProfile(true);
                                    welcomeActivity.dataHolder.setProfile(profile);
                                    welcomeActivity.progressDialog.dismiss();
                                    welcomeActivity.reloadActivity();
                                } else {
                                    Log.e(TAG, "Database error: " + databaseError.toException());
                                    ProfileSetupFragment.this.onSetupFailed();
                                    welcomeActivity.progressDialog.dismiss();
                                }
                            }
                        });
                // Get favorites, if any, and update date for looking for changes with the new periodicity
                welcomeActivity.dbReference.child(currentUser.getUuid()).child("/favorites").addListenerForSingleValueEvent(new ValueEventListener() {
                    @Override
                    public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                        for (DataSnapshot favoriteSnapshot : dataSnapshot.getChildren()) {
                            Favorite favorite = favoriteSnapshot.getValue(Favorite.class);
                            Date date = new Date();
                            Calendar c = Calendar.getInstance();
                            String periodicity = profile.getPeriodicity();
                            switch (periodicity) {
                                case PERIODICITY_D:
                                    c.add(Calendar.DATE, 1);
                                    date = new Date(c.getTimeInMillis());
                                    break;
                                case PERIODICITY_W:
                                    c.add(Calendar.DATE, 7);
                                    date = new Date(c.getTimeInMillis());
                                    break;
                                case PERIODICITY_N:
                                    date = null;
                                    break;
                            }
                            favorite.setNextDateToCheckProductInfo(date);

                            welcomeActivity.dbReference.child(currentUser.getUuid()).child("/favorites")
                                    .child(favorite.getProductCode()).setValue(favorite,
                                    new DatabaseReference.CompletionListener() {
                                        @Override
                                        public void onComplete(DatabaseError databaseError,
                                                               DatabaseReference databaseReference) {
                                            if (databaseError == null) {

                                            } else {
                                                Log.e(TAG, "Database error: " + databaseError.toException());
                                            }
                                        }
                                    });
                        }
                    }

                    @Override
                    public void onCancelled(@NonNull DatabaseError databaseError) {
                        Log.e(TAG, "Database error: " + databaseError.toException());
                    }
                });
            }
        });

        return view;
    }

    /**
     * Add allergens from autocompleteview to allergens list
     *
     * @param listForAdapter list for the adapter
     */
    private void addAllergenToList(List<String> listForAdapter) {
        ImageViewCompat.setImageTintList(addButton, ColorStateList.valueOf(ContextCompat.getColor(getContext(), R.color.textColorSecondary)));

        KeyboardUtils.hideKeyBoard(getContext(), getActivity());
        if (!InputValidationUtils.validateSelectedAllergen(getContext(), allergensTextInput, selectedAllergen)) {
            ImageViewCompat.setImageTintList(addButton, ColorStateList.valueOf(ContextCompat.getColor(getContext(), R.color.textColorPrimary)));
            return;
        }

        autoCompleteText.clearFocus();
        if (InputValidationUtils.validateSelectedAllergensList(getContext(), allergensTextInput, selectedAllergen, listForAdapter)) {
            listForAdapter.add(selectedAllergen);
            Collections.sort(listForAdapter);
            updateAllergensToSaveInProfile(listForAdapter, welcomeActivity.dataHolder.getCurrentLocale().getLanguage());
            adapterForListView.notifyDataSetChanged();
            ListViewUtils.setListViewHeightBasedOnChildren(listView, profileSetupFragment);
            if (listView.getVisibility() != View.VISIBLE) {
                listView.setVisibility(View.VISIBLE);
            }
            autoCompleteText.setText(null);
            selectedAllergen = null;
            setupButton.setEnabled(true);
        }
        ImageViewCompat.setImageTintList(addButton, ColorStateList.valueOf(ContextCompat.getColor(getContext(), R.color.textColorPrimary)));
    }

    /**
     * on destroy
     */
    @Override
    public void onDestroy() {
        super.onDestroy();
        // Initialize drawer and bottom bar
        welcomeActivity.drawerToggle.showDrawer();
        welcomeActivity.drawerLayout.setDrawerLockMode(DrawerLayout.LOCK_MODE_UNLOCKED);
        welcomeActivity.bottomNavigationView.setVisibility(View.VISIBLE);
    }

    /**
     * on view created
     *
     * @param view               view
     * @param savedInstanceState bundle
     */
    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        final WelcomeActivity welcomeActivity = (WelcomeActivity) getActivity();
        this.welcomeActivity = welcomeActivity;
        welcomeActivity.allergensDataBaseAdapter = new AllergensDataBaseAdapter(getContext());
        // Retrieve data from local database or wait for catalogs to load
        if (welcomeActivity.dataHolder.getUser().isHasProfile()) {
            this.retrieveDataFromLocalDatabase();
        } else {
            welcomeActivity.progressDialog =
                    ProgressDialogUtils.createProgressDialog(getContext(), R.string.welcome_activity_loading_message);
            welcomeActivity.progressDialog.show();
            welcomeActivity.checkIfDataIsLoaded(welcomeActivity.progressDialog);
        }
        retrieveDataFromProfileIfAny(view, new ArrayList<String>());
    }

    /**
     * Retrieve allergens data from rest api
     */
    private void retrieveAllergens() {
        welcomeActivity.progressDialog =
                ProgressDialogUtils.createProgressDialog(getContext(), R.string.profile_setup_loading_message);
        welcomeActivity.progressDialog.show();

        OpenFoodFactsService apiServiceEs = OpenFoodFactsClient.getEsClient().create(OpenFoodFactsService.class);
        apiRestOpenFoodFactsCall(apiServiceEs, ES, welcomeActivity.progressDialog);

        OpenFoodFactsService apiServiceEn = OpenFoodFactsClient.getEnClient().create(OpenFoodFactsService.class);
        apiRestOpenFoodFactsCall(apiServiceEn, EN, welcomeActivity.progressDialog);

    }

    /**
     * Convert allergen to string
     *
     * @param retrievedAllergens source
     * @param language           selected locale
     * @return list of strings
     */
    private List<String> convertAllergenToString(Map<String, List<Allergens.Allergen>> retrievedAllergens, String language) {
        List<String> allergens = new ArrayList<>();
        for (Allergens.Allergen allergen : retrievedAllergens.get(language)) {
            allergens.add(allergen.name);
        }
        return allergens;
    }

    /**
     * Method for calling open food facts api to retrieve allergens info
     *
     * @param apiService     endpoint
     * @param language       selected locale
     * @param progressDialog progress dialog
     */
    private void apiRestOpenFoodFactsCall(OpenFoodFactsService apiService, String language, ProgressDialog progressDialog) {
        /**
         GET List Allergens
         **/
        Call<Allergens> call = apiService.getAllAllergens();
        call.enqueue(new Callback<Allergens>() {
            @Override
            public void onResponse(Call<Allergens> call, Response<Allergens> response) {
                // Get service response
                Allergens resource = response.body();
                // Parse allergens information
                List<Allergens.Allergen> allergens = new ArrayList<>();
                for (Allergens.Allergen allergen : resource.tags) {
                    if (pattern.matcher(allergen.name).matches()) {
                        allergens.add(allergen);
                    }
                }
                Collections.sort(allergens, (s1, s2) -> s1.name.compareTo(s2.name));

                welcomeActivity.dataHolder.getAllergens().put(language, allergens);
                retrievedAllergens.put(language, allergens);
                // Store allergens in database
                DatabaseCatalogUtils.saveAllergensInLocalDatabase(welcomeActivity.allergensDataBaseAdapter, allergens, language);
                // Update adapter
                if (retrievedAllergens.get(welcomeActivity.dataHolder.getCurrentLocale().getLanguage()) != null
                        && !retrievedAllergens.get(welcomeActivity.dataHolder.getCurrentLocale().getLanguage()).isEmpty()) {
                    allergensToShow = convertAllergenToString(retrievedAllergens, welcomeActivity.dataHolder.getCurrentLocale().getLanguage());
                    adapterForAutoComplete = new ArrayAdapter<String>(getContext(), android.R.layout.simple_list_item_1,
                            allergensToShow.toArray(new String[allergensToShow.size()]));
                    autoCompleteText.setAdapter(adapterForAutoComplete);
                }
                if (retrievedAllergens.get(ES) != null && !retrievedAllergens.get(ES).isEmpty()
                        && retrievedAllergens.get(EN) != null && !retrievedAllergens.get(EN).isEmpty()) {
                    progressDialog.dismiss();
                }
            }

            @Override
            public void onFailure(Call<Allergens> call, Throwable t) {
                call.cancel();
                Log.e(TAG, t.getMessage() + "");
                profileSetupFragment = (ProfileSetupFragment) getFragmentManager()
                        .findFragmentById(R.id.fragment_layout);
                relativeLayout = (RelativeLayout) profileSetupFragment.getActivity()
                        .findViewById(R.id.mainLayout);
                progressDialog.dismiss();
                SnackBarUtils
                        .showSnackBar(getContext(), relativeLayout,
                                getString(R.string.openfacts_get_all_allergens_error),
                                (int) userText.getTextSize(), true);
            }
        });
    }

    /**
     * When setup fails, show snackbar
     */
    private void onSetupFailed() {

        relativeLayout = (RelativeLayout) profileSetupFragment.getActivity()
                .findViewById(R.id.mainLayout);
        SnackBarUtils
                .showSnackBar(getContext(), relativeLayout, getString(R.string.profile_setup_failed_error),
                        (int) emailText.getTextSize(), true);
        setupButton.setEnabled(true);
    }

    /**
     * Retrieve data from profile if exists any
     *
     * @param view           view to search
     * @param listForAdapter resource
     */
    private void retrieveDataFromProfileIfAny(View view, List<String> listForAdapter) {
        if (welcomeActivity.dataHolder.getProfile() != null) {
            periodicity = welcomeActivity.dataHolder.getProfile().getPeriodicity();
            RadioButton radioButton = null;
            switch (periodicity) {
                case PERIODICITY_D:
                    radioButton = (RadioButton) view.findViewById(R.id.periodicity_daily);
                    break;
                case PERIODICITY_W:
                    radioButton = (RadioButton) view.findViewById(R.id.periodicity_weekly);
                    break;
                case PERIODICITY_N:
                    radioButton = (RadioButton) view.findViewById(R.id.periodicity_never);
                    break;
            }
            radioButton.setChecked(true);

            boolean hasAllergens;
            List<Allergens.Allergen> listToShow;
            if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
                hasAllergens = (welcomeActivity.dataHolder.getProfile().getAllergens_es() != null &&
                        !welcomeActivity.dataHolder.getProfile().getAllergens_es().isEmpty());
                listToShow = welcomeActivity.dataHolder.getProfile().getAllergens_es();
            } else {
                hasAllergens = (welcomeActivity.dataHolder.getProfile().getAllergens_en() != null &&
                        !welcomeActivity.dataHolder.getProfile().getAllergens_en().isEmpty());
                listToShow = welcomeActivity.dataHolder.getProfile().getAllergens_en();
            }
            if (hasAllergens) {
                listView.setVisibility(View.VISIBLE);
                for (Allergens.Allergen allergen : listToShow) {
                    listForAdapter.add(allergen.name);
                }
                updateAllergensToSaveInProfile(listForAdapter, welcomeActivity.dataHolder.getCurrentLocale().getLanguage());
                adapterForListView.notifyDataSetChanged();
                ListViewUtils.setListViewHeightBasedOnChildren(listView, profileSetupFragment);
            }
        }
    }

    /**
     * Redirect back to user account
     */
    private void goToUserAccountScreen() {
        welcomeActivity.toolbar.getNavigationIcon().setTint(getActivity().getColor(R.color.textColorSecondary));
        Fragment fragment = UserAccountFragment.newInstance(getString(R.string.user_account_activity_label));
        FragmentManager fragmentManager = getFragmentManager();
        fragmentManager.beginTransaction().replace(R.id.fragment_layout, fragment).commit();
        getActivity().setTitle(getString(R.string.user_account_activity_label));
    }

    /**
     * Update allergens to save
     *
     * @param listForAdapter resource
     * @param language       selected locale
     */
    private void updateAllergensToSaveInProfile(List<String> listForAdapter, String language) {
        List<Allergens.Allergen> allergensCurrentLanguage = welcomeActivity.dataHolder.getAllergens().get(language);
        List<Allergens.Allergen> selectedAllergensCurrentLanguage = new ArrayList<>();
        if (allergensCurrentLanguage != null && !allergensCurrentLanguage.isEmpty()) {
            for (Allergens.Allergen allergen : allergensCurrentLanguage) {
                if (listForAdapter.contains(allergen.name)) {
                    selectedAllergensCurrentLanguage.add(allergen);
                }
            }
            allergensToSaveInProfile.put(language, selectedAllergensCurrentLanguage);

            String otherLanguage = (ES.equals(language)) ? EN : ES;
            List<Allergens.Allergen> selectedAllergensOtherLanguage = new ArrayList<>();
            List<Allergens.Allergen> allergensOtherLanguage = welcomeActivity.dataHolder.getAllergens().get(otherLanguage);
            if (allergensOtherLanguage != null && !allergensOtherLanguage.isEmpty()) {
                for (Allergens.Allergen currentAllergen : selectedAllergensCurrentLanguage) {
                    search:
                    for (Allergens.Allergen otherAllergen : allergensOtherLanguage) {
                        if (currentAllergen.id.equals(otherAllergen.id)) {
                            selectedAllergensOtherLanguage.add(otherAllergen);
                            break search;
                        }
                    }
                }
                allergensToSaveInProfile.put(otherLanguage, selectedAllergensOtherLanguage);
            }
        }
    }

    /**
     * Retrieve allergens from database
     */
    @Override
    public void retrieveDataFromLocalDatabase() {
        welcomeActivity.dataHolder.getAllergens().put(ES,
                DatabaseCatalogUtils.retrieveAllergensFromLocalDatabase(welcomeActivity.allergensDataBaseAdapter, ES));
        welcomeActivity.dataHolder.getAllergens().put(EN,
                DatabaseCatalogUtils.retrieveAllergensFromLocalDatabase(welcomeActivity.allergensDataBaseAdapter, EN));

        if (welcomeActivity.dataHolder.getAllergens().get(welcomeActivity.dataHolder.getCurrentLocale().getLanguage()).isEmpty()) {
            this.retrieveAllergens();
        } else {
            allergensToShow = new ArrayList<>();
            List<Allergens.Allergen> allergensInHolder =
                    welcomeActivity.dataHolder.getAllergens().get(welcomeActivity.dataHolder.getCurrentLocale().getLanguage());
            for (Allergens.Allergen allergen : allergensInHolder) {
                if (pattern.matcher(allergen.name).matches()) {
                    allergensToShow.add(allergen.name);
                }
            }
            Collections.sort(allergensToShow/*, (s1, s2) -> s1.name.compareTo(s2.name)*/);

            adapterForAutoComplete = new ArrayAdapter<String>(getContext(),
                    android.R.layout.simple_list_item_1, allergensToShow.toArray(new String[allergensToShow.size()]));
            autoCompleteText.setAdapter(adapterForAutoComplete);
        }
    }
}