package edu.uoc.tfg.nutriknow.views.fragments;

import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Parcelable;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.ScrollView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.adapters.SearchesRecyclerAdapter;
import edu.uoc.tfg.nutriknow.model.Favorite;
import edu.uoc.tfg.nutriknow.model.Product;
import edu.uoc.tfg.nutriknow.model.Search;
import edu.uoc.tfg.nutriknow.utils.ProductInfoUtils;
import edu.uoc.tfg.nutriknow.utils.ProgressDialogUtils;
import edu.uoc.tfg.nutriknow.utils.SnackBarUtils;
import edu.uoc.tfg.nutriknow.views.LoadData;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;

/**
 * Searches fragment class
 */
public class SearchesFragment extends Fragment implements LoadData {

    private static final String TITLE = "title";
    private static final String TAG = "SearchesFragment";

    private static WelcomeActivity welcomeActivity;
    private SearchesFragment searchesFragment;
    private RelativeLayout relativeLayout;
    private TextView textView;
    private ScrollView scrollView;

    public RecyclerView recyclerView;
    public RecyclerView.Adapter adapter;
    private LinearLayout linearLayout;

    private AlertDialog.Builder builder;

    private String barcode;

    private final String KEY_RECYCLER_STATE = "recycler_state";
    private static Bundle mBundleRecyclerViewState;
    private boolean scrollToBegin = false;

    /**
     * Class constructor
     */
    public SearchesFragment() {
    }

    /**
     * Create a new instance
     * @param title fragment title
     * @return fragment
     */
    public static SearchesFragment newInstance(String title) {
        SearchesFragment fragment = new SearchesFragment();
        Bundle args = new Bundle();
        args.putString(TITLE, title);
        fragment.setArguments(args);
        return fragment;
    }

    /**
     * Create view
     * @param inflater inflater
     * @param container container
     * @param savedInstanceState bundle
     * @return view
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        final View view = inflater.inflate(R.layout.searches_fragment, container, false);

        welcomeActivity = (WelcomeActivity) getActivity();
        // Configure toolbar
        welcomeActivity.prepareToolbarForFragments();
        // Create progress dialog and show during favorites retrieval
        welcomeActivity.progressDialog =
                ProgressDialogUtils.createProgressDialog(getContext(), R.string.searches_loading_message);
        welcomeActivity.progressDialog.show();

        searchesFragment = (SearchesFragment) getFragmentManager()
                .findFragmentById(R.id.fragment_layout);
        // Allow to scroll to begin
        if (!(welcomeActivity.callingFragment instanceof ProductInfoFragment)
                && !(welcomeActivity.callingFragment instanceof FavoritesFragment)
                && !(welcomeActivity.callingFragment instanceof SearchesFragment)) {
            scrollToBegin = true;
        }

        welcomeActivity.callingFragment = searchesFragment;
        // Select options in bottombar and drawer
        welcomeActivity.navigationView.setCheckedItem(R.id.nav_my_searches);
        welcomeActivity.bottomNavigationView.getMenu().getItem(2).setChecked(true);

        // Reference to layout views
        linearLayout = view.findViewById(R.id.main_container);

        relativeLayout = (RelativeLayout) searchesFragment.getActivity()
                .findViewById(R.id.mainLayout);

        textView = (TextView) view.findViewById(R.id.item);

        recyclerView = view.findViewById(R.id.recycler_view);

        RecyclerView.LayoutManager layoutManager = new LinearLayoutManager(getContext());
        recyclerView.setLayoutManager(layoutManager);

        scrollView = (ScrollView) view.findViewById(R.id.scroll);

        // If empty, change screen background
        if (welcomeActivity.dataHolder.getSearches().isEmpty()) {
            linearLayout.setBackgroundColor(welcomeActivity.getColor(R.color.backgroundColorWindow));
        }
        adapter = new SearchesRecyclerAdapter(welcomeActivity.dataHolder.getSearches(),
                welcomeActivity.dataHolder.getProducts(), welcomeActivity, searchesFragment,
                welcomeActivity.dataHolder.getFavorites());
        recyclerView.setAdapter(adapter);

        evaluateViewToDisplay();
        // Create alert dialog for search removal
        builder = new AlertDialog.Builder(getContext(), R.style.AlertDialogTheme);
        builder.setMessage(R.string.product_delete_dialog_text);
        builder.setIcon(R.drawable.ic_edit);
        builder.setPositiveButton(R.string.product_delete_dialog_yes_text, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.dismiss();
                deleteProduct();

            }
        });
        builder.setNegativeButton(R.string.product_delete_dialog_no_text, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                barcode = null;
                dialog.dismiss();
            }
        });

        return view;
    }

    /**
     * on view created
     * @param view view
     * @param savedInstanceState bundle
     */
    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        final WelcomeActivity welcomeActivity = (WelcomeActivity) getActivity();
        this.welcomeActivity = welcomeActivity;
        // Retrieve data from database
        this.retrieveDataFromLocalDatabase();
    }

    /**
     * On resume
     */
    @Override
    public void onResume() {
        super.onResume();
        // Allow scroll to begin when defined
        if (!scrollToBegin) {
            if (mBundleRecyclerViewState != null) {
                Parcelable listState = mBundleRecyclerViewState.getParcelable(KEY_RECYCLER_STATE);
                recyclerView.getLayoutManager().onRestoreInstanceState(listState);
            }
        } else {
            scrollView.scrollTo(0, 0);
        }
        evaluateViewToDisplay();
    }

    /**
     * Save recyclerview state
     */
    @Override
    public void onPause() {
        super.onPause();
        mBundleRecyclerViewState = new Bundle();
        Parcelable listState = recyclerView.getLayoutManager().onSaveInstanceState();
        mBundleRecyclerViewState.putParcelable(KEY_RECYCLER_STATE, listState);
    }

    /**
     * Retrieve product information
     * @param barcode product barcode
     */
    public void retrieveProductInformation(String barcode) {
        welcomeActivity.progressDialog =
                ProgressDialogUtils.createProgressDialog(getContext(), R.string.product_info_loading_message);
        welcomeActivity.progressDialog.show();
        welcomeActivity.retrieveProductInformation(barcode, welcomeActivity.progressDialog);
    }

    /**
     * Delete product information
     * @param barcode product barcode
     */
    public void deleteProductInformation(String barcode, int selectedIndex) {
        AlertDialog alert = builder.create();
        alert.show();
        this.barcode = barcode;
    }

    /**
     * Delete product
     */
    private void deleteProduct() {
        welcomeActivity.progressDialog =
                ProgressDialogUtils.createProgressDialog(getContext(), R.string.remove_search_loading_message);
        welcomeActivity.progressDialog.show();

        // Remove all the product information from database
        welcomeActivity.dbReference = welcomeActivity.database.getReference("/users");
        welcomeActivity.dbReference.child(welcomeActivity.dataHolder.getUser().getUuid())
                .child("products").child(barcode).addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    child.getRef().removeValue();
                }
                dataSnapshot.getRef().removeValue();

                for (Product product : welcomeActivity.dataHolder.getProducts()) {
                    if (barcode.equals(product.getSummaryData().getCode())) {
                        welcomeActivity.dataHolder.getProducts().remove(product);
                        break;
                    }
                }
                // Remove search
                welcomeActivity.dbReference.child(welcomeActivity.dataHolder.getUser().getUuid())
                        .child("searches").child(barcode).addListenerForSingleValueEvent(new ValueEventListener() {
                    @Override
                    public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                        for (DataSnapshot child : dataSnapshot.getChildren()) {
                            child.getRef().removeValue();
                        }
                        dataSnapshot.getRef().removeValue();

                        for (Search search : welcomeActivity.dataHolder.getSearches()) {
                            if (barcode.equals(search.getProductCode())) {
                                welcomeActivity.dataHolder.getSearches().remove(search);
                                break;
                            }
                        }
                        if (welcomeActivity.dataHolder.getSearches().isEmpty()) {
                            linearLayout.setBackgroundColor(welcomeActivity.getColor(R.color.backgroundColorWindow));
                        }
                        // Remove favorite, if any
                        welcomeActivity.dbReference.child(welcomeActivity.dataHolder.getUser().getUuid())
                                .child("favorites").child(barcode).addListenerForSingleValueEvent(new ValueEventListener() {
                            @Override
                            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                                for (DataSnapshot child : dataSnapshot.getChildren()) {
                                    child.getRef().removeValue();
                                }
                                dataSnapshot.getRef().removeValue();

                                for (Favorite favorite : welcomeActivity.dataHolder.getFavorites()) {
                                    if (barcode.equals(favorite.getProductCode())) {
                                        welcomeActivity.dataHolder.getFavorites().remove(favorite);
                                        break;
                                    }
                                }
                                adapter = new SearchesRecyclerAdapter(welcomeActivity.dataHolder.getSearches(),
                                            welcomeActivity.dataHolder.getProducts(), welcomeActivity, searchesFragment,
                                            welcomeActivity.dataHolder.getFavorites());
                                recyclerView.setAdapter(adapter);
                                adapter.notifyDataSetChanged();

                                searchesFragment.evaluateViewToDisplay();

                                searchesFragment = (SearchesFragment) getFragmentManager()
                                        .findFragmentById(R.id.fragment_layout);
                                relativeLayout = (RelativeLayout) searchesFragment.getActivity()
                                        .findViewById(R.id.mainLayout);
                                SnackBarUtils
                                        .showSnackBar(getContext(), relativeLayout,
                                                getString(R.string.product_deleted_message),
                                                (int) textView.getTextSize(), false);

                                welcomeActivity.progressDialog.dismiss();
                            }

                            @Override
                            public void onCancelled(@NonNull DatabaseError databaseError) {
                                Log.e(TAG, "Database error: " + databaseError.toException());
                                searchesFragment = (SearchesFragment) getFragmentManager()
                                        .findFragmentById(R.id.fragment_layout);
                                relativeLayout = (RelativeLayout) searchesFragment.getActivity()
                                        .findViewById(R.id.mainLayout);
                                SnackBarUtils
                                        .showSnackBar(getContext(), relativeLayout,
                                                getString(R.string.product_removal_error),
                                                (int) textView.getTextSize(), true);
                                welcomeActivity.progressDialog.dismiss();
                            }
                        });

                    }

                    @Override
                    public void onCancelled(@NonNull DatabaseError databaseError) {
                        Log.e(TAG, "Database error: " + databaseError.toException());
                        searchesFragment = (SearchesFragment) getFragmentManager()
                                .findFragmentById(R.id.fragment_layout);
                        relativeLayout = (RelativeLayout) searchesFragment.getActivity()
                                .findViewById(R.id.mainLayout);
                        SnackBarUtils
                                .showSnackBar(getContext(), relativeLayout,
                                        getString(R.string.product_removal_error),
                                        (int) textView.getTextSize(), true);
                        welcomeActivity.progressDialog.dismiss();
                    }
                });
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {
                Log.e(TAG, "Database error: " + databaseError.toException());
                searchesFragment = (SearchesFragment) getFragmentManager()
                        .findFragmentById(R.id.fragment_layout);
                relativeLayout = (RelativeLayout) searchesFragment.getActivity()
                        .findViewById(R.id.mainLayout);
                SnackBarUtils
                        .showSnackBar(getContext(), relativeLayout,
                                getString(R.string.product_removal_error),
                                (int) textView.getTextSize(), true);
                welcomeActivity.progressDialog.dismiss();
            }
        });
    }

    /**
     * Retrieve data from local database
     */
    @Override
    public void retrieveDataFromLocalDatabase() {
        ProductInfoUtils.retrieveDataFromLocalDatabase(welcomeActivity);

        welcomeActivity.dbReference.child(welcomeActivity.dataHolder.getUser().getUuid()).addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                List<Search> searches = new ArrayList<>();
                for (DataSnapshot productSnapshot : dataSnapshot.child("/searches").getChildren()) {
                    searches.add(productSnapshot.getValue(Search.class));
                }
                welcomeActivity.dataHolder.setSearches(searches);
                List<Favorite> favorites = new ArrayList<>();
                for (DataSnapshot productSnapshot : dataSnapshot.child("/favorites").getChildren()) {
                    favorites.add(productSnapshot.getValue(Favorite.class));
                }
                welcomeActivity.dataHolder.setFavorites(favorites);

                adapter.notifyDataSetChanged();
                welcomeActivity.progressDialog.dismiss();
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {
                Log.e(TAG, "Database error: " + databaseError.toException());
            }
        });

        evaluateViewToDisplay();
    }

    /**
     * Add product to favorites
     * @param product product to add
     */
    public void addProductToFavorites(Product product) {
        searchesFragment = (SearchesFragment) getFragmentManager()
                .findFragmentById(R.id.fragment_layout);
        relativeLayout = (RelativeLayout) searchesFragment.getActivity()
                .findViewById(R.id.mainLayout);

        ProductInfoUtils.addProductToFavorites(product, getContext(), welcomeActivity,
                relativeLayout, textView, searchesFragment);
    }

    /**
     * Delete product from favorite list
     */
    public void deleteProductFromFavorites(String barcode) {
        searchesFragment = (SearchesFragment) getFragmentManager()
                .findFragmentById(R.id.fragment_layout);
        relativeLayout = (RelativeLayout) searchesFragment.getActivity()
                .findViewById(R.id.mainLayout);

        ProductInfoUtils.deleteProductFromFavorites(barcode, getContext(), welcomeActivity,
                relativeLayout, textView, linearLayout, searchesFragment);
    }

    /**
     * Evaluate which view display if recyclerview is empty or not
     */
    public void evaluateViewToDisplay() {
        if (welcomeActivity.dataHolder.getSearches().isEmpty()) {
            scrollView.setVisibility(View.VISIBLE);
            recyclerView.setVisibility(View.GONE);
        } else {
            scrollView.setVisibility(View.GONE);
            recyclerView.setVisibility(View.VISIBLE);
        }
    }
}