package edu.uoc.tfg.nutriknow.views.fragments;

import android.content.res.Configuration;
import android.content.res.Resources;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;

import androidx.fragment.app.Fragment;

import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;

import java.util.Locale;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.model.Profile;
import edu.uoc.tfg.nutriknow.utils.ProgressDialogUtils;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;

/**
 * Settings fragment class
 */
public class SettingsFragment extends Fragment {

    private static final String TITLE = "title";
    private static final String TAG = "SettingsFragment";

    private static final String ES = "es";
    private static final String EN = "en";

    private WelcomeActivity welcomeActivity;
    private Button setupButton;
    private String selectedLanguage;

    /**
     * Class constructor
     */
    public SettingsFragment() {
    }

    /**
     * Create a new instance
     *
     * @param title fragment title
     * @return fragment
     */
    public static SettingsFragment newInstance(String title) {
        SettingsFragment fragment = new SettingsFragment();
        Bundle args = new Bundle();
        args.putString(TITLE, title);
        fragment.setArguments(args);
        return fragment;
    }

    /**
     * Create view
     *
     * @param inflater           inflater
     * @param container          container
     * @param savedInstanceState bundle
     * @return view
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        final View view = inflater.inflate(R.layout.settings_fragment, container, false);

        welcomeActivity = ((WelcomeActivity) getActivity());
        welcomeActivity.prepareToolbarForFragments();

        SettingsFragment settingsFragment = (SettingsFragment) getFragmentManager()
                .findFragmentById(R.id.fragment_layout);

        welcomeActivity.callingFragment = settingsFragment;

        // Get selected locale
        Locale currentLocale = getResources().getConfiguration().locale;
        selectedLanguage = currentLocale.getLanguage();
        // Reference to layout views
        TextView userText = (TextView) view.findViewById(R.id.user_settings_text);
        userText.setText(welcomeActivity.dataHolder.getUser().getUserName());

        TextView emailText = (TextView) view.findViewById(R.id.email_settings_text);
        emailText.setText(welcomeActivity.dataHolder.getUser().getEmail());

        RadioGroup radioGroup = (RadioGroup) view.findViewById(R.id.radio_group);
        // Add radio group functionality
        RadioButton radioButton = null;
        if (ES.equals(selectedLanguage)) {
            radioButton = (RadioButton) view.findViewById(R.id.language_es);
        } else {
            radioButton = (RadioButton) view.findViewById(R.id.language_en);
        }
        radioButton.setChecked(true);
        radioGroup.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                // checkedId is the RadioButton selected
                switch (checkedId) {
                    case R.id.language_es:
                        selectedLanguage = ES;
                        break;
                    case R.id.language_en:
                        selectedLanguage = EN;
                        break;
                }
            }
        });
        // Add setup button functionality
        setupButton = (Button) view.findViewById(R.id.btn_settings);
        setupButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (!selectedLanguage.equals(currentLocale.getLanguage())) {
                    // Create user in database
                    setupButton.setEnabled(false);

                    welcomeActivity.progressDialog =
                            ProgressDialogUtils.createProgressDialog(getContext(), R.string.settings_saving_message);
                    welcomeActivity.progressDialog.show();
                    // Change current locale
                    Resources resources = getResources();
                    DisplayMetrics dm = resources.getDisplayMetrics();
                    Configuration config = resources.getConfiguration();
                    Locale locale = new Locale(selectedLanguage.toLowerCase());
                    config.setLocale(locale);
                    resources.updateConfiguration(config, dm);
                    // Update current profile
                    Profile profile = welcomeActivity.dataHolder.getProfile();
                    profile.setSelectedLanguage(locale.getLanguage());
                    // Store profile in database
                    welcomeActivity.dbReference.child(welcomeActivity.dataHolder.getUser().getUuid()).child("profile").setValue(profile,
                            new DatabaseReference.CompletionListener() {
                                @Override
                                public void onComplete(DatabaseError databaseError,
                                                       DatabaseReference databaseReference) {
                                    if (databaseError == null) {

                                        welcomeActivity.dataHolder.setCurrentLocale(locale);
                                        welcomeActivity.dataHolder.setProfile(profile);

                                        welcomeActivity.progressDialog.dismiss();
                                        welcomeActivity.reloadActivity();
                                    } else {
                                        Log.e(TAG, "Database error: " + databaseError.toException());
                                        welcomeActivity.progressDialog.dismiss();
                                    }
                                }
                            });


                }
            }
        });
        return view;
    }
}