package edu.uoc.tfg.nutriknow.views.fragments;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.ScrollView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import com.joanzapata.iconify.IconDrawable;
import com.joanzapata.iconify.fonts.FontAwesomeIcons;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.adapters.AllergensSummaryAdapter;
import edu.uoc.tfg.nutriknow.adapters.IngredientsAnalysisSummaryAdapter;
import edu.uoc.tfg.nutriknow.adapters.LabelsSummaryAdapter;
import edu.uoc.tfg.nutriknow.model.PageViewModel;
import edu.uoc.tfg.nutriknow.model.Product;
import edu.uoc.tfg.nutriknow.utils.IconDrawableUtils;
import edu.uoc.tfg.nutriknow.utils.ListViewUtils;
import edu.uoc.tfg.nutriknow.utils.ProductInfoUtils;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;

/**
 * Summary info fragment class
 */
public class SummaryInfoFragment extends Fragment {

    private static final String ARG_SECTION_NUMBER = "section_number";
    private static final String ARG_PRODUCT = "product";

    private static final String ES = "es";

    private static SummaryInfoFragment summaryInfoFragment;
    private Product product;

    private ScrollView scrollView;

    /**
     * Generate new instance
     *
     * @param index tab index
     * @param prod  product
     * @return fragment
     */
    public static SummaryInfoFragment newInstance(int index, Product prod) {
        summaryInfoFragment = new SummaryInfoFragment(prod);
        Bundle bundle = new Bundle();
        bundle.putInt(ARG_SECTION_NUMBER, index);
        bundle.putSerializable(ARG_PRODUCT, prod);
        summaryInfoFragment.setArguments(bundle);
        return summaryInfoFragment;
    }

    /**
     * Class constructor
     */
    public SummaryInfoFragment(Product prod) {
        product = prod;
    }

    /**
     * Create view
     *
     * @param savedInstanceState bundle
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // Get tablayout reference
        PageViewModel pageViewModel = new ViewModelProvider(this, new ViewModelProvider.NewInstanceFactory()).get(PageViewModel.class);
        int index = 1;
        if (getArguments() != null) {
            index = getArguments().getInt(ARG_SECTION_NUMBER);
            product = (Product) getArguments().getSerializable(ARG_PRODUCT);
        }
        pageViewModel.setProduct(product);
        pageViewModel.setIndex(index);
    }

    /**
     * Create view
     *
     * @param inflater           inflater
     * @param container          container
     * @param savedInstanceState bundle
     * @return view
     */
    @Override
    public View onCreateView(
            @NonNull LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        final View root = inflater.inflate(R.layout.summary_info_fragment, container, false);

        WelcomeActivity welcomeActivity = (WelcomeActivity) getActivity();

        scrollView = (ScrollView) root.findViewById(R.id.scroll);

        // Reference to layout views and hide or display them depending on their values
        TextView productNameTextView = (TextView) root.findViewById(R.id.product_name_text);
        if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
            if (product.getSummaryData().getProductName_es() != null && !product.getSummaryData().getProductName_es().isEmpty()) {
                productNameTextView.setText(product.getSummaryData().getProductName_es());
            } else {
                if (product.getSummaryData().getProductName_en() != null && !product.getSummaryData().getProductName_en().isEmpty()) {
                    productNameTextView.setText(product.getSummaryData().getProductName_en());
                } else {
                    productNameTextView.setText(getString(R.string.empty_product_name));
                }
            }
        } else {
            if (product.getSummaryData().getProductName_en() != null && !product.getSummaryData().getProductName_en().isEmpty()) {
                productNameTextView.setText(product.getSummaryData().getProductName_en());
            } else {
                if (product.getSummaryData().getProductName_es() != null && !product.getSummaryData().getProductName_es().isEmpty()) {
                    productNameTextView.setText(product.getSummaryData().getProductName_es());
                } else {
                    productNameTextView.setText(getString(R.string.empty_product_name));
                }
            }
        }

        TextView completenessTextLabelView = (TextView) root.findViewById(R.id.completeness_label_text);
        TextView completenessTextView = (TextView) root.findViewById(R.id.completeness_text);
        if (product.getSummaryData().getCompleteness() == 0) {
            completenessTextLabelView.setVisibility(View.GONE);
            completenessTextView.setVisibility(View.GONE);
        } else {
            float complete =
                    ProductInfoUtils.roundFloat(product.getSummaryData().getCompleteness() * 100f, 2);
            String completeness = complete + " %";
            completenessTextView.setText(completeness);
            // Add icon and change color depending on the field value
            int color = 0;
            if (complete >= 60) {
                color = R.color.textColorSecondary;
            } else if (complete < 60 && complete >= 40) {
                color = R.color.textColorWarning;
            } else {
                color = R.color.textColorError;
            }
            IconDrawable warning = IconDrawableUtils
                    .createIconDrawable(getContext(), FontAwesomeIcons.fa_warning,
                            (int) completenessTextView.getTextSize(), color, false);
            completenessTextView.setCompoundDrawablesWithIntrinsicBounds(warning, null, null, null);
        }

        RelativeLayout brandsLayout = (RelativeLayout) root.findViewById(R.id.brands_content_layout);
        TextView brandsTextView = (TextView) root.findViewById(R.id.brands_text);
        if (product.getSummaryData().getBrands().isEmpty()) {
            brandsLayout.setVisibility(View.GONE);
        } else {
            if (product.getSummaryData().getBrands().isEmpty()) {
                brandsTextView.setVisibility(View.GONE);
            } else {
                brandsTextView.setText(
                        ProductInfoUtils.convertListToString(product.getSummaryData().getBrands()));
            }
        }

        RelativeLayout storesLayout = (RelativeLayout) root.findViewById(R.id.stores_layout);
        TextView storesTextView = (TextView) root.findViewById(R.id.stores_text);
        if (product.getSummaryData().getStores().isEmpty()) {
            storesLayout.setVisibility(View.GONE);
        } else {
            storesTextView.setText(
                    ProductInfoUtils.convertListToString(product.getSummaryData().getStores()));
        }

        RelativeLayout countriesLayout = (RelativeLayout) root.findViewById(R.id.countries_content_layout);
        TextView countriesTextView = (TextView) root.findViewById(R.id.countries_text);
        if (product.getSummaryData().getCountries().isEmpty()) {
            countriesLayout.setVisibility(View.GONE);
        } else {
            countriesTextView.setText(
                    ProductInfoUtils.convertListToString(product.getSummaryData().getCountries()).replace("en:", ""));
        }

        RelativeLayout highlightedAllergensLayout = (RelativeLayout) root.findViewById(R.id.highlighted_allergens_content_layout);
        ListView highlightedAllergensListView = (ListView) root.findViewById(R.id.highlighted_allergens_list);
        AllergensSummaryAdapter allergensSummaryAdapter = null;
        if (product.getSummaryData().getAllergens_es().isEmpty() &&
                product.getSummaryData().getAllergens_en().isEmpty()) {
            highlightedAllergensLayout.setVisibility(View.GONE);
        } else {
            if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
                if (!product.getSummaryData().getAllergens_es().isEmpty()) {
                    allergensSummaryAdapter = new AllergensSummaryAdapter(product.getSummaryData().getAllergens_es(),
                            welcomeActivity.dataHolder.getProfile().getAllergens_es(), getContext());
                } else {
                    if (!product.getSummaryData().getAllergens_en().isEmpty()) {
                        allergensSummaryAdapter = new AllergensSummaryAdapter(product.getSummaryData().getAllergens_en(),
                                welcomeActivity.dataHolder.getProfile().getAllergens_en(), getContext());
                    } else {
                        highlightedAllergensLayout.setVisibility(View.GONE);
                    }
                }
            } else {
                if (!product.getSummaryData().getAllergens_en().isEmpty()) {
                    allergensSummaryAdapter = new AllergensSummaryAdapter(product.getSummaryData().getAllergens_en(),
                            welcomeActivity.dataHolder.getProfile().getAllergens_en(), getContext());
                } else {
                    if (!product.getSummaryData().getAllergens_es().isEmpty()) {
                        allergensSummaryAdapter = new AllergensSummaryAdapter(product.getSummaryData().getAllergens_es(),
                                welcomeActivity.dataHolder.getProfile().getAllergens_es(), getContext());
                    } else {
                        highlightedAllergensLayout.setVisibility(View.GONE);
                    }
                }
            }
        }
        if (allergensSummaryAdapter != null) {
            highlightedAllergensListView.setAdapter(allergensSummaryAdapter);
            ListViewUtils.setListViewHeightBasedOnChildren(highlightedAllergensListView, summaryInfoFragment);
        }

        RelativeLayout highlightedTracesLayout = (RelativeLayout) root.findViewById(R.id.highlighted_traces_content_layout);
        ListView highlightedTracesListView = (ListView) root.findViewById(R.id.highlighted_traces_list);
        AllergensSummaryAdapter tracesSummaryAdapter = null;
        if (product.getSummaryData().getTraces_es().isEmpty() &&
                product.getSummaryData().getTraces_en().isEmpty()) {
            highlightedTracesLayout.setVisibility(View.GONE);
        } else {
            if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
                if (!product.getSummaryData().getTraces_es().isEmpty()) {
                    tracesSummaryAdapter = new AllergensSummaryAdapter(product.getSummaryData().getTraces_es(),
                            welcomeActivity.dataHolder.getProfile().getAllergens_es(), getContext());
                } else {
                    if (!product.getSummaryData().getTraces_en().isEmpty()) {
                        tracesSummaryAdapter = new AllergensSummaryAdapter(product.getSummaryData().getTraces_en(),
                                welcomeActivity.dataHolder.getProfile().getAllergens_en(), getContext());
                    } else {
                        highlightedTracesLayout.setVisibility(View.GONE);
                    }
                }
            } else {
                if (!product.getSummaryData().getTraces_en().isEmpty()) {
                    tracesSummaryAdapter = new AllergensSummaryAdapter(product.getSummaryData().getTraces_en(),
                            welcomeActivity.dataHolder.getProfile().getAllergens_en(), getContext());
                } else {
                    if (!product.getSummaryData().getTraces_es().isEmpty()) {
                        tracesSummaryAdapter = new AllergensSummaryAdapter(product.getSummaryData().getTraces_es(),
                                welcomeActivity.dataHolder.getProfile().getAllergens_es(), getContext());
                    } else {
                        highlightedTracesLayout.setVisibility(View.GONE);
                    }
                }
            }
        }
        if (tracesSummaryAdapter != null) {
            highlightedTracesListView.setAdapter(tracesSummaryAdapter);
            ListViewUtils.setListViewHeightBasedOnChildren(highlightedTracesListView, summaryInfoFragment);
        }

        RelativeLayout ingredientsAnalysisLayout = (RelativeLayout) root.findViewById(R.id.ingredients_analysis_content_layout);
        ListView ingredientAnalysisListView = (ListView) root.findViewById(R.id.ingredients_analysis_list);
        IngredientsAnalysisSummaryAdapter ingredientsAnalysisSummaryAdapter = null;
        if (product.getSummaryData().getIngredientsAnalysis_es().isEmpty() &&
                product.getSummaryData().getIngredientsAnalysis_en().isEmpty()) {
            ingredientsAnalysisLayout.setVisibility(View.GONE);
        } else {
            if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
                if (!product.getSummaryData().getIngredientsAnalysis_es().isEmpty()) {
                    ingredientsAnalysisSummaryAdapter = new IngredientsAnalysisSummaryAdapter(
                            product.getSummaryData().getIngredientsAnalysis_es(), getContext());
                } else {
                    if (!product.getSummaryData().getIngredientsAnalysis_en().isEmpty()) {
                        ingredientsAnalysisSummaryAdapter = new IngredientsAnalysisSummaryAdapter(
                                product.getSummaryData().getIngredientsAnalysis_en(), getContext());
                    } else {
                        ingredientsAnalysisLayout.setVisibility(View.GONE);
                    }
                }
            } else {
                if (!product.getSummaryData().getIngredientsAnalysis_en().isEmpty()) {
                    ingredientsAnalysisSummaryAdapter = new IngredientsAnalysisSummaryAdapter(
                            product.getSummaryData().getIngredientsAnalysis_en(), getContext());
                } else {
                    if (!product.getSummaryData().getIngredientsAnalysis_es().isEmpty()) {
                        ingredientsAnalysisSummaryAdapter = new IngredientsAnalysisSummaryAdapter(
                                product.getSummaryData().getIngredientsAnalysis_es(), getContext());
                    } else {
                        ingredientsAnalysisLayout.setVisibility(View.GONE);
                    }
                }
            }
        }
        if (ingredientsAnalysisSummaryAdapter != null) {
            ingredientAnalysisListView.setAdapter(ingredientsAnalysisSummaryAdapter);
            ListViewUtils.setListViewHeightBasedOnChildren(ingredientAnalysisListView, summaryInfoFragment);
        }

        RelativeLayout labelsLayout = (RelativeLayout) root.findViewById(R.id.labels_content_layout);
        ListView labelsListView = (ListView) root.findViewById(R.id.labels_list);
        LabelsSummaryAdapter labelsSummaryAdapter = null;
        if (product.getSummaryData().getLabels_es().isEmpty() &&
                product.getSummaryData().getLabels_en().isEmpty()) {
            labelsLayout.setVisibility(View.GONE);
        } else {
            if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
                if (!product.getSummaryData().getLabels_es().isEmpty()) {
                    labelsSummaryAdapter = new LabelsSummaryAdapter(
                            product.getSummaryData().getLabels_es(), getContext());
                } else {
                    if (!product.getSummaryData().getLabels_en().isEmpty()) {
                        labelsSummaryAdapter = new LabelsSummaryAdapter(
                                product.getSummaryData().getLabels_en(), getContext());
                    } else {
                        labelsLayout.setVisibility(View.GONE);
                    }
                }
            } else {
                if (!product.getSummaryData().getLabels_en().isEmpty()) {
                    labelsSummaryAdapter = new LabelsSummaryAdapter(
                            product.getSummaryData().getLabels_en(), getContext());
                } else {
                    if (!product.getSummaryData().getLabels_es().isEmpty()) {
                        labelsSummaryAdapter = new LabelsSummaryAdapter(
                                product.getSummaryData().getLabels_es(), getContext());
                    } else {
                        labelsLayout.setVisibility(View.GONE);
                    }
                }
            }
        }
        if (labelsSummaryAdapter != null) {
            labelsListView.setAdapter(labelsSummaryAdapter);
            ListViewUtils.setListViewHeightBasedOnChildren(labelsListView, summaryInfoFragment);
        }

        return root;
    }

    /**
     * Scroll to begin when tab is invisible to the user
     *
     * @param isVisibleToUser
     */
    @Override
    public void setUserVisibleHint(boolean isVisibleToUser) {
        super.setUserVisibleHint(isVisibleToUser);
        if (this.isVisible()) {
            if (!isVisibleToUser) {
                scrollView.scrollTo(0, 0);
            }
        }
    }
}