package edu.uoc.tfg.nutriknow.views.fragments;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;

import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.model.Allergens;
import edu.uoc.tfg.nutriknow.utils.ListViewUtils;
import edu.uoc.tfg.nutriknow.utils.ProgressDialogUtils;
import edu.uoc.tfg.nutriknow.utils.SnackBarUtils;
import edu.uoc.tfg.nutriknow.views.activities.LoginActivity;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;

/**
 * Settings fragment class
 */
public class UserAccountFragment extends Fragment implements View.OnClickListener {

    private static final String TITLE = "title";
    private static final String TAG = "UserAccount";
    private static final String ES = "es";

    private UserAccountFragment userAccountFragment;

    private WelcomeActivity welcomeActivity;
    private RelativeLayout relativeLayout;
    private TextView userText;
    private Button deleteButton;

    private AlertDialog.Builder builder;
    private AlertDialog alert;

    /**
     * Class constructor
     */
    public UserAccountFragment() {
    }

    /**
     * Create a new instance
     *
     * @param title fragment title
     * @return fragment
     */
    public static UserAccountFragment newInstance(String title) {
        UserAccountFragment fragment = new UserAccountFragment();
        Bundle args = new Bundle();
        args.putString(TITLE, title);
        fragment.setArguments(args);
        return fragment;
    }

    /**
     * Create view
     *
     * @param inflater           inflater
     * @param container          container
     * @param savedInstanceState bundle
     * @return view
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        final View view = inflater.inflate(R.layout.user_account_fragment, container, false);

        welcomeActivity = ((WelcomeActivity) getActivity());
        // Configure toolbar
        welcomeActivity.prepareToolbarForFragments();

        userAccountFragment = (UserAccountFragment) getFragmentManager()
                .findFragmentById(R.id.fragment_layout);

        welcomeActivity.callingFragment = userAccountFragment;
        // Reference layout views
        relativeLayout = (RelativeLayout) userAccountFragment.getActivity()
                .findViewById(R.id.mainLayout);

        userText = (TextView) view.findViewById(R.id.user_account_text);
        userText.setText(welcomeActivity.dataHolder.getUser().getUserName());

        TextView emailText = (TextView) view.findViewById(R.id.email_account_text);
        emailText.setText(welcomeActivity.dataHolder.getUser().getEmail());

        TextView periodicityText = (TextView) view.findViewById(R.id.periodicity_value_text);
        String periodicityValue = null;
        switch (welcomeActivity.dataHolder.getProfile().getPeriodicity()) {
            case "D":
                periodicityValue = getString(R.string.periodicity_daily_label);
                break;
            case "W":
                periodicityValue = getString(R.string.periodicity_weekly_label);
                break;
            case "N":
                periodicityValue = getString(R.string.periodicity_never_label);
                break;
        }
        periodicityText.setText(periodicityValue);

        ImageView editAllergensButton = (ImageView) view.findViewById(R.id.edit_allergens_icon);
        editAllergensButton.setOnClickListener(this);

        ImageView editPeriodicityButton = (ImageView) view.findViewById(R.id.edit_periodicity_icon);
        editPeriodicityButton.setOnClickListener(this);

        ListView listView = (ListView) view.findViewById(R.id.selectedList);

        boolean hasAllergens;
        List<Allergens.Allergen> listToShow;

        if (ES.equals(welcomeActivity.dataHolder.getCurrentLocale().getLanguage())) {
            hasAllergens = (welcomeActivity.dataHolder.getProfile().getAllergens_es() != null &&
                    !welcomeActivity.dataHolder.getProfile().getAllergens_es().isEmpty());
            listToShow = welcomeActivity.dataHolder.getProfile().getAllergens_es();
        } else {
            hasAllergens = (welcomeActivity.dataHolder.getProfile().getAllergens_en() != null &&
                    !welcomeActivity.dataHolder.getProfile().getAllergens_en().isEmpty());
            listToShow = welcomeActivity.dataHolder.getProfile().getAllergens_en();
        }
        final List<String> listForAdapter = new ArrayList<String>();
        if (hasAllergens) {
            listView.setVisibility(View.VISIBLE);
            for (Allergens.Allergen allergen : listToShow) {
                listForAdapter.add(allergen.name);
            }
        }
        if (listForAdapter.isEmpty()) {
            listForAdapter.add(getString(R.string.user_account_none_label));
        }
        Collections.sort(listForAdapter);
        ArrayAdapter<String> adapterForListView = new ArrayAdapter<String>(getContext(), R.layout.allergens_row_layout, R.id.label,
                listForAdapter);
        listView.setAdapter(adapterForListView);
        adapterForListView.notifyDataSetChanged();
        ListViewUtils.setListViewHeightBasedOnChildren(listView, userAccountFragment);
        // Create alert dialog for account removal
        builder = new AlertDialog.Builder(getContext(), R.style.AlertDialogTheme);
        builder.setMessage(R.string.user_delete_dialog_text);
        builder.setIcon(R.drawable.ic_edit);
        builder.setPositiveButton(R.string.user_delete_dialog_yes_text, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.dismiss();
                deleteUser();

            }
        });
        builder.setNegativeButton(R.string.user_delete_dialog_no_text, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.dismiss();
                deleteButton.setEnabled(true);
            }
        });
        // Add delete button functionality
        deleteButton = (Button) view.findViewById(R.id.btn_delete);
        deleteButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                deleteButton.setEnabled(false);

                alert = builder.create();
                alert.show();
            }
        });

        return view;
    }

    /**
     * On click, redirect to profile setup
     *
     * @param v
     */
    @Override
    public void onClick(View v) {
        Fragment fragment = ProfileSetupFragment.newInstance(getString(R.string.profile_setup_activity_label));
        FragmentManager fragmentManager = getFragmentManager();
        fragmentManager.beginTransaction().replace(R.id.fragment_layout, fragment).commit();
        getActivity().setTitle(getString(R.string.profile_setup_activity_label));
    }

    /**
     * Delete user account
     */
    private void deleteUser() {
        welcomeActivity.progressDialog =
                ProgressDialogUtils.createProgressDialog(getContext(), R.string.user_account_deleting_message);
        welcomeActivity.progressDialog.show();

        // Remove user from database
        welcomeActivity.dbReference = welcomeActivity.database.getReference("/users");
        welcomeActivity.dbReference.child(welcomeActivity.dataHolder.getUser().getUuid()).addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    child.getRef().removeValue();
                }
                dataSnapshot.getRef().removeValue();
                // Delete user from firebase auth
                welcomeActivity.firebaseAuth.getCurrentUser().delete().addOnSuccessListener(new OnSuccessListener() {
                    @Override
                    public void onSuccess(Object o) {
                        welcomeActivity.dataHolder.setUser(null);
                        welcomeActivity.dataHolder.setProfile(null);

                        welcomeActivity.progressDialog.dismiss();

                        onDeleteUser();
                    }
                });
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {
                Log.e(TAG, "Database error: " + databaseError.toException());
                deleteButton.setEnabled(true);
                welcomeActivity.progressDialog.dismiss();
            }
        });
    }

    /**
     * If removal was successful, show snackbar
     */
    private void onDeleteUser() {
        userAccountFragment = (UserAccountFragment) getFragmentManager()
                .findFragmentById(R.id.fragment_layout);
        relativeLayout = (RelativeLayout) userAccountFragment.getActivity()
                .findViewById(R.id.mainLayout);
        SnackBarUtils
                .showSnackBar(getContext(), relativeLayout, getString(R.string.user_delete_successful_message),
                        (int) userText.getTextSize(), false);
        new android.os.Handler().postDelayed(
                new Runnable() {
                    public void run() {
                        deleteButton.setEnabled(true);
                        // Start the login activity
                        Intent intent = new Intent(getContext(), LoginActivity.class);
                        startActivity(intent);
                        getActivity().overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);
                    }
                }, 1000);
    }
}